/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2008/01/23 17:17:58 $
 */
 
package kaos.core.csi;

import java.io.Serializable;
import java.util.List;

/**
 * Defines the operations to store and access information about a capability an agent offers.
 *
 * @author KAoS Team
 * $Revision: 1.3 $
 */
public interface AgentCapability extends Serializable
{
    /** 
     * Set the name of this AgentCapability.
     *
     * @param name      String containing the name of the AgentCapability. 
     */
    public void setName(String name);
   
    /**
     * Obtain the AgentCapability name.
     *
     * @return String containing the name of the AgentCapability. 
     */
    public String getName();
    
    /** 
     * Set the keywords used to search for this capability.
     *
     * @param keywords  List of Strings containing keywords to be used in search for this capability. 
     */
    public void setKeywords(List<String> keywords);
   
    /**
     * Obtain a List of String describing keywords to be used in search for this capability.
     *
     * @return List     containing Strings to describe this capability. 
     */
    public List<String> getKeywords();
    
    /** 
     * Set the capability cost
     *
     * @param cost      double containing the cost of this capability.
     */
    public void setCost(double  cost);
   
    /**
     * Obtain the capability cost.
     *
     * @return double   containing the cost of this capability.
     */
    public double getCost();
    
    /** 
     * Set a List of Strings describing how an agent requests another agent to perform the capability. 
     *
     * @param requestForInfo   List of String describing how to request this capabiity. 
     */
    public void setRequestForInfo(List<String> requestForInfo);
    
    /** 
     * Return Strings describing how to request this capability from an agent. 
     *
     * @return  List    containing String to request this capability from an agent.
     *               
     */
    public List<String> getRequestForInfo();
   
    /** 
     * Set the reliability of the capability (double/percentage).
     *
     * @param reliability   double specifying the reliability of the capability.            
     */
    public void setReliability(double reliability);
    
    /** 
     * Obtain a double specifying the reliability of the capability (double/percentage). 
     *
     * @return double   containing the the reliability of the capability.            
     */
    public double getReliability();
   
    /** 
     * Determine if this AgentCapability is contained in the other AgentCapability,
     * meaning that all properties of this AgentCapability are present in the other.
     *
     * @param other     AgentCapability to compare this AgentCapability with.
     *
     * @return boolean   indicating if the properties of this AgentCapabilities are present in the other AgentCapability.               
     */
    public boolean containedInCapability(AgentCapability other);
}
