/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2012/05/15 17:46:37 $
 */

package kaos.core.csi;

import kaos.core.csi.extension.KAoSExtensionComponent;
import kaos.core.csi.extension.KAoSExtensionComponentManager;
import kaos.core.csi.extension.KAoSExtensionsTypes;
import kaos.core.csi.extension.classification.InstanceClassifierManager;
import kaos.core.csi.extension.classification.InstanceClassifierManagerImpl;
import kaos.core.csi.extension.history.EventHistoryComponent;
import kaos.core.csi.extension.state.StateSensor;
import kaos.core.csi.observe.ObserverManager;
import kaos.core.csi.observe.ObserverManagerImpl;
import kaos.core.csi.ontology.OntologyRepositoryService;
import kaos.core.csi.ontology.OntologyRepositoryServiceImpl;
import kaos.core.csi.policy.*;
import kaos.core.csi.policy.enforcement.EnforcerManager;
import kaos.core.csi.policy.enforcement.EnforcerManagerImpl;
import kaos.core.csi.query.QueryRegistration;
import kaos.core.csi.query.QueryRegistrationImpl;
import kaos.core.csi.query.QueryState;
import kaos.core.csi.query.QueryStateImpl;
import kaos.core.csi.registration.Registration;
import kaos.core.csi.registration.RegistrationImpl;
import kaos.core.csi.request.RequestManager;
import kaos.core.csi.request.RequestManagerImpl;
import kaos.core.csi.transport.PolicyEnforcedTransportSupportImpl;
import kaos.core.csi.transport.TransportSupport;
import kaos.core.csi.transport.TransportSupportImpl;
import kaos.core.csi.util.xmlSupport.XMLToCSILink;
import kaos.core.service.boot.KAoSGuardBootstrapper;
import kaos.core.util.KAoSConfigurationParameters;
import kaos.policy.guard.GuardRetriever;
import kaos.policy.guard.KAoSGuard;
import kaos.spatial.SpatialReasoner;

import javax.agent.service.Service;

/**
 * This class provides access to specific KAoS services offered by the KAoS Common Services Interface
 * 
 * $Revision: 1.53 $
 */
public class CSIFactory
{
    public static final String   TCP_TRANSPORT        = "TCP";
    public static final String   CORBA_TRANSPORT      = "corba";
    public static final String   COABSGRID_TRANSPORT  = "grid";

    public static final String[] SUPPORTED_TRANSPORTS = { TCP_TRANSPORT, CORBA_TRANSPORT, COABSGRID_TRANSPORT };

    protected static String      currentTransportName = TCP_TRANSPORT;

    /**
     * @return Returns the currentTransportName.
     */
    public static String getCurrentTransport()
    {
        return currentTransportName;
    }

    /**
     * @param currentTransportName - the currentTransportName to set.
     */
    public static void setCurrentTransport(String currentTransportName)
    {
        CSIFactory.currentTransportName = currentTransportName;
    }

    /**
     * Retrieve a reference to the Registration.
     * 
     * @return a reference to the Registration
     */
    public static TransportSupport getTransportSupport()
    {
        if (Boolean.getBoolean("kaos.comm.policy.enforce"))
            return PolicyEnforcedTransportSupportImpl.getInstance();
        else
            return TransportSupportImpl.getInstance();
    }

    /**
     * Retrieve a reference to the ObserverManager.
     * 
     * @return a reference to the ObserverManager Service
     */
    public static ObserverManager getObserverManager() throws Exception
    {
        return ObserverManagerImpl.getInstance(currentTransportName);
    }

    /**
     * Retrieve a reference to the Registration.
     * 
     * @return a reference to the Registration Service
     */
    public static Registration getRegistration() throws Exception
    {
        return RegistrationImpl.getInstance(currentTransportName);
    }

    /**
     * Retrieve a reference to the QueryRegistration.
     * 
     * @return a reference to the QueryRegistration.
     */
    public static QueryRegistration getQueryRegistration() throws Exception
    {
        return QueryRegistrationImpl.getInstance(currentTransportName);
    }

    /**
     * Retrieve a reference to the QueryState.
     * 
     * @return a reference to the QueryState.
     */
    public static QueryState getQueryState() throws Exception
    {
        return QueryStateImpl.getInstance(currentTransportName);
    }

    /**
     * Retrieve a reference to the RequestManager.
     * 
     * @return a reference to the RequestManager
     */
    public static RequestManager getRequestManager() throws Exception
    {
        return RequestManagerImpl.getInstance(currentTransportName);
    }

    /**
     * Retrieve a reference to the OntologyService.
     * 
     * @return a reference to the OntologyService
     */
    public static OntologyRepositoryService getOntologyService() throws Exception
    {
        return OntologyRepositoryServiceImpl.getInstance(currentTransportName);
    }
    
    /**
     * Retrieve a reference to the PolicyManagement.
     * 
     * @return a reference to the PolicyManagement
     */
    public static PolicyManagement getPolicyManagement() throws Exception
    {
        return PolicyManagementImpl.getInstance(currentTransportName);
    }

    /**
     * Retrieve a reference to the PolicyChecking.
     * 
     * @return a reference to the PolicyChecking
     */
    public static PolicyChecking getPolicyChecking() throws Exception
    {
        return PolicyCheckingImpl.getInstance(currentTransportName);
    }

    /**
     * Retrieve a reference to the PolicyAdvice.
     * 
     * @return a reference to the PolicyAdvice.
     */
    public static PolicyAdvice getPolicyAdvice() throws Exception
    {
        return PolicyCheckingImpl.getPolAdviceInstance(currentTransportName);
    }

    /**
     * Retrieve a reference to a KAoSExtensionComponentManager, whose service name is specified
     * 
     * @param serviceMgrName   String containing the name as specified in the guardConfiguration file, at the beginning of the section 
     *                         describing the service, ex. stateMonitor-service
     * 
     * @return a reference to the KAoSExtensionComponentManager, or null if a KAoSExtensionComponentManager for the given service does not exist.
     */
    public static KAoSExtensionComponentManager getKAoSExtensionComponentManager(String serviceMgrName) throws Exception
    {
        Service serviceMgr = KAoSGuardBootstrapper.getService(serviceMgrName);

        if (serviceMgr instanceof KAoSExtensionComponentManager)
            return (KAoSExtensionComponentManager)serviceMgr;

        return null;
    }
    
    /**
     * Retrieve a reference to a KAoSExtensionComponent loaded in the guard
     * 
     * @return a reference to the KAoSExtensionComponent
     */
    public static KAoSExtensionComponent getKAoSExtensionComponent(String repType, String associatedOntType) throws Exception
    {
        KAoSGuard guard = GuardRetriever.getGuard();

        if (guard == null)
        {
            System.out.println("\n Unable to retrieve extension component!  Guard is not available!");
            return null;
        }

        return guard.getKAoSExtensionComponents(repType, associatedOntType);
    }
    
    /**
     * Retrieve a reference to the PolicyEnforcement.
     * 
     * @return a reference to the PolicyEnforcement
     */
    public static EnforcerManager getPolicyEnforcement() throws Exception
    {
        return EnforcerManagerImpl.getInstance();
    }

    /**
     * Retrieve a reference to the ClassifierManagement
     * 
     * @return a reference to the PolicyEnforcement
     */
    public static InstanceClassifierManager getClassifierManagement()
    {
        return InstanceClassifierManagerImpl.getInstance();
    }

    /**
     * Retrieve a reference to the EventHistoryComponent.
     * 
     * @return a reference to the EventHistoryComponent
     */
    public static EventHistoryComponent getHistoryMonitor() throws Exception
    {
        KAoSGuard guard = GuardRetriever.getGuard();

        if (guard == null)
        {
            System.out.println("\n Unable to Event History Component!");
            return null;
        }

        return guard.getEventHistoryManager();
    }
    
    /**
     * Retrieve a reference to the StateMonitor
     * 
     * @return a reference to the StateMonitor
     */
    public static StateSensor getStateMonitor(String associatedOntType) throws Exception
    {
        KAoSGuard guard = GuardRetriever.getGuard();

        if (guard == null)
        {
            System.out.println("\n Unable to retrieve StateMonitorManager!  Guard is not available!");
            return null;
        }

        return (StateSensor)guard.getKAoSExtensionComponents(KAoSExtensionsTypes.STATE_SENSORS, associatedOntType);
    }

    /**
     * Retrieve a reference to the SpatialReasoner.
     * 
     * @return a reference to the SpatialReasoner.
     */
    public static SpatialReasoner getSpatialReasoner() throws Exception
    {
        if ((System.getProperty(KAoSConfigurationParameters.GLOBAL_SPATIAL_REASONER)) != null)
            return SpatialReasoner.getInstance(currentTransportName, true);
        else
            return SpatialReasoner.getInstance(currentTransportName, false);
    }

    /**
     * Retrieve a reference to the XMLToCSILink.
     * 
     * @return a reference to the XMLToCSILink
     */
    public static XMLToCSILink getXMLToCSILink(String dsTransport) throws Exception
    {
        return XMLToCSILink.getInstance(dsTransport);
    }
}