/**
 * Last Modified by: $Author: mvignati $
 * On: $Date: 2012/01/26 19:30:06 $
 */

package kaos.core.csi;

import kaos.ontology.util.SerializableOntModelImpl;

import java.util.Set;

/**
 *   This interface contains methods allowing to create and investigate a description of an ontology property.
 *
 *   $Revision: 1.9 $
 */
public interface OntPropertyDescription  extends Cloneable
{
    /**
     * Set the value to this property, if previous values exists they will be deleted
     *
     * @param value      String containing the value of the property.
     */
    public void setValue(String value);
    
    /**
     * Add the value to this property.
     *
     * @param value      String containing the value of the property.
     */
    public void addValue(String value);

    /**
     * @see kaos.core.csi.OntPropertyDescription#addValue(Set<java.lang.String>)
     */
    public void addMultipleValues(Set<String> values);

    /**
     * Set the optional description of the given property value.
     *
     * @param value      String containing the value of the property.
     * @param desc       Object containing the description  of the value often it will be kaos.core.csi.OntInstanceDescription
     */
    public void setValueDescForValue(String value, Object desc);

    /**
     * Get the name of this property.
     *
     * @return           String containing the uri of the property.
     */
    public String getName();

    /**
     * Get the values of this property.
     *
     * @return           Set containing the value of the property.
     */
    public Set<String> getValues();
    
    /**
     * Get the first value of this property.
     *
     * @return           first value of the property.
     */
    public String getFirstValue();

    /**
     * Locate the given property nested within one of the value descriptions for this property and return the set of values for that nested property.
     *
     * @param nestedPropertyName
     * @return Set of String values for the nested property
     */
    public Set<String> getNestedPropertyValues(String nestedPropertyName);

    /**
     * Get the optional description of the given property value.
     *
     * @param value      String containing the value of the property.
     * @return           Object containing the description  of the value often it will be kaos.core.csi.OntInstanceDescription
     */
    public Object getValueDescForValue(String value);

    /**
     * Remove the property value.
     *
     * @param value      String containing the value of the property.
     */
    public void removeValue(String value);

    /**
     * Clear the property value.
     */
    public void removeAllValues();
    
    /**
     * Remove the optional description of the given property value.
     *
     * @param value      String containing the value of the property.
     */
    public void removeValueDescForValue(String value);
    
    /**
     * Add this property to the instance definition in the ontology model
     *
     * @param ontModel     ontology model to which the property is to be added
     * @param instanceName name of the instance the property is to be added to
     */
    public void addPropertyToOntModel(SerializableOntModelImpl ontModel, String instanceName);

    /**
     * Cone the property 
     * 
     * @return the new property 
     */
    public OntPropertyDescription clone();
}