/**
 * Last Modified by: $Author: auszok $ On: $Date: 2010/07/23 02:31:27 $
 */
package kaos.core.csi;

import kaos.core.csi.registration.Property;

import java.util.Hashtable;
import java.util.Map;

/**
 * $Revision: 1.6 $
 */
public class TransportImpl implements Transport
{
    private static final long serialVersionUID = -2163374529702308187L;
    
    protected String          _name;
    protected Map<String, Property>       _properties;

    /**
     * Defines the operations to store and access information about a given Transport.
     */
    public TransportImpl()
    {
        initialize();
    }

    /**
     * Defines the operations to store and access information about a given Transport.
     */
    public TransportImpl(String name)
    {
        setName(name);
        initialize();
    }

    // ------------- Implementation of the Transport interface

    /**
     * Set the name of this Transport.
     * 
     * @param name String containing the name of the Transport.
     */
    public void setName(String name)
    {
        _name = name;
    }

    /**
     * Obtain the Transport name.
     * 
     * @return String containing the name of the Transport.
     */
    public String getName()
    {
        return _name;
    }

    /**
     * Set a property for this Transport.
     * 
     * @param property Property describing this transport.
     */
    public void setProperty(Property property)
    {
        _properties.put(property.getName(), property);
    }

    /**
     * Obtain a Property of this Transport.
     * 
     * @return Property describing this transport.
     */
    public Property getProperty(String name)
    {
        return (Property)_properties.get(name);
    }

    private void initialize()
    {
        _properties = new Hashtable<String, Property>();
    }
}