/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2012/03/26 20:37:21 $
 */
 
package kaos.core.csi.extension;

import javax.agent.service.ServiceProperties;

import java.util.Set;

/**
 * This interface defines methods to access name and ontological attributes associated with this component
 * 
 *   $Revision: 1.7 $
 */
public interface KAoSExtensionComponent
{
    /**
     * Return the associated ontological concepts for the KAoSExtensionComponent object.
     *      
     * @return  List containing names of the concepts the KAoSExtensionComponent is associated with.
     */
     public Set<String> getOntologicalAttributes();
	
    /**
     * Return the (unique) name of this KAoSExtensionComponent object.
     *      
     * @return  name of this component.
     */
     public String getName();
	
    /**
     * Set 'true/false', indicating the 'enabled/disabled' status of the component.
     *      
     * @param	status   boolean parameter to set the 'enabled/disabled' status to
     */
     public void setEnabledStatus(boolean status);
	
    /**
     * Return 'true/false', indicating the 'enabled/disabled' status of the component.
     *      
     * @return	a boolean parameter indicating the 'enabled/disabled' status of this component.
     */
     public boolean getEnabledStatus();	
	
    /**
     * Add an ontological attribute the component is associated with.
     *      
     * @param	ontAttribute String containing the name of the ontological attribute.						 
     */
    public void addOntologicalAttribute(String ontAttribute);	
    
    /**
     * Set an ontological attribute the component is associated with.
     *      
     * @param	ontAttribute String containing the name of the ontological attribute.	
     * 
     * @param	subclasses List containing subclasses of the ontological attribute.						 
     */
    public void setSubclassesForAttribute(String ontAttribute, Set<String> subclasses);	
    
    /**
     * Get an ontological attribute the component is associated with.
     *      
     * @param	ontAttribute String containing the name of the ontological attribute.	
     * 
     * @return	List containing subclasses of the ontological attribute.						 
     */
    public Set<String> getSubclassesForAttribute(String ontAttribute);	
	
    /**
     * Set the name of this component.
     *      
     * @param	componentName String containing the name of this component.
     * 						 
     */
     public void setName(String componentName);	
	
    /**
     * Set the properties of this component, if there are any.
     *      
     * @param	props ServiceProperties (optionally) specifying additional ontological properties of the component.  						 
     */
     public void setProperties(ServiceProperties props);	
}