/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2012/02/22 17:42:34 $
 */

package kaos.core.csi.extension;

import javax.agent.service.Service;
import javax.agent.service.ServiceProperties;
import java.util.List;
import java.util.Map;

/**
 * This interface is exposed by the KAoS to the outside system allowing them to register with KAoS specific components extending default KAoS functionality.
 * 
 * This interface is implemented by different classes providing storage and management for specific kinds of extension components. 
 *
 * $Revision: 1.6 $
 */
public interface KAoSExtensionComponentManager extends Service
{
    /**
     * Get the repository type of this manager. e.g. if a Classifier or Enabler etc.
     * 
     * @return A String specifying the type of this manager.
     */
    public String getRepositoryManagerType();

    /**
     * Register a specific KAoSExtensionComponent with KAoS
     * 
     * @param extensionComponent KAoSExtensionComponent that is being registered
     * 
     * @return boolean indicating whether the component was successfully registered
     */
    public boolean registerComponent(KAoSExtensionComponent extensionComponent);

    public void loadConfiguration();
    
    /**
     * Deregister a specific KAoSExtensionComponent from KAoS
     * 
     * @param componentName name of the component
     * 
     * @return boolean indicating whether the component was successfully deregistered
     */
    public boolean deregisterComponent(String componentName);

    /**
     * Obtain a specific for the given manager kind of extension component associated with the provided ontology attribute. The component was either created during the bootstrap
     * process, was registered previously using the registerComponent method, or, if possible, will be created on the fly.
     * 
     * @param associatedOntType string specifying the ontology attribute, for which a component is requested
     * @param props ServiceProperties (optionally) specifying additional ontological properties of the component.
     * 
     * @throws KAoSExtensionComponentInitializationException if the
     */
    public KAoSExtensionComponent getComponent(String associatedOntType, ServiceProperties props);

    /**
     * Changes the 'enabled' status of the given component
     * 
     * @param associatedOntType A String indicating the ontology type associated with that component.
     * @param state A boolean parameter to which the status would change
     */
    public void changeComponentEnabledStatus(String associatedOntType, boolean state);

    /**
     * Get the 'enabled' status of the given component
     * 
     * @param associatedOntType A String indicating the ontology type associated with that component.
     */
    public boolean getComponentEnabledStatus(String associatedOntType);

    /**
     * Get all components contained in this repository.
     * 
     * @return A Vector containing all components.
     */
    public List<KAoSExtensionComponent> getAllKAoSExtensionComponents();

    /**
     * Remove all components contained in this repository.
     */
    public void removeAllKAoSExtensionComponents();

    /**
     * This method returns the concepts names and their corresponding component names in a map. The keys in this table are Strings specifying concept names and values are
     * Strings specifying corresponding component names
     */
    public Map<String, String> getRepositoryTableInformation();
}