/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2012/03/28 20:37:58 $
 */

package kaos.core.csi.extension;

import kaos.core.csi.CSIFactory;
import kaos.core.csi.ontology.OntologyService;
import kaos.core.util.Logger;
import kaos.core.csi.query.QueryState.UnknownConceptException;
import kaos.policy.builder.KAoSPolicyBuilderUtil;
import ri.service.boot.BootProperties;

import javax.agent.service.ServiceException;
import javax.agent.service.ServiceFailure;
import javax.agent.service.ServiceProperties;
import javax.agent.service.directory.DirectoryFailure;
import java.lang.reflect.Constructor;
import java.util.*;

/**
 * This class implements the interface defines methods to manage a repository of KAoSExtensionComponents
 */
public class KAoSExtensionComponentManagerImpl implements KAoSExtensionComponentManager
{
    private String                              _repositoryType;                        // see kaos.core.csi.extension.KAoSExtensionsTypes

    private Map<String, KAoSExtensionComponent> _components;
    private Map<String, KAoSExtensionComponent> _componentsConceptMapping;
    private BootProperties                      _componentsClassNames;

    private Map<String, Set<String>>            _subConcepts;

    /*
     * Constructor
     */
    protected KAoSExtensionComponentManagerImpl(String type)
    {
        _repositoryType = type;
        _components = new Hashtable<String, KAoSExtensionComponent>();
        _componentsConceptMapping = new Hashtable<String, KAoSExtensionComponent>();
        _subConcepts = new Hashtable<String, Set<String>>();
    }

    public void loadConfiguration()
    {
        // TODO Auto-generated method stub
    }
    
    /*
     * @see kaos.core.csi.extension.KAoSExtensionComponentManager#registerComponent(kaos.core.csi.extension.KAoSExtensionComponent)
     */
    public String getRepositoryManagerType()
    {
        return _repositoryType;
    }

    /*
     * @see kaos.core.csi.extension.KAoSExtensionComponentManager#registerComponent(kaos.core.csi.extension.KAoSExtensionComponent)
     */
    public boolean registerComponent(KAoSExtensionComponent extensionComponent)
    {
        _components.put(extensionComponent.getName(), extensionComponent);

        for (String currentAssociatedOntType : extensionComponent.getOntologicalAttributes())
        {
            _componentsConceptMapping.put(currentAssociatedOntType, extensionComponent);

            // Check if the sub concepts of the current associatedOntType has already been cached
            if (_subConcepts.get(currentAssociatedOntType) == null)
            {
            	Set<String> mySubconcept = extensionComponent.getSubclassesForAttribute(currentAssociatedOntType);
            	
            	if (mySubconcept != null)
            		_subConcepts.put(currentAssociatedOntType, mySubconcept);
            }
        }

        return true;
    }

    /*
     * @see kaos.core.csi.extension.KAoSExtensionComponentManager#deregisterComponent(java.lang.String)
     */
    public boolean deregisterComponent(String componentName)
    {
        Map<String, KAoSExtensionComponent> newComponentsConceptMapping = new Hashtable<String, KAoSExtensionComponent>();

        if (_components.get(componentName) == null)
            return false;
        else
        {
            _components.remove(componentName);

            for (String currentConcept : _componentsConceptMapping.keySet())
                if ( ! _componentsConceptMapping.get(currentConcept).equals(componentName))
                    newComponentsConceptMapping.put(currentConcept, _componentsConceptMapping.get(currentConcept));

            _componentsConceptMapping = newComponentsConceptMapping;

            return true;
        }
    }

    /*
     * @see kaos.core.csi.extension.KAoSExtensionComponentManager#getComponent(java.lang.String)
     */
    public KAoSExtensionComponent getComponent(String associatedOntType, ServiceProperties props)
    {
        if (associatedOntType == null)
            return null;

        // Check if the given component is already available
        for (KAoSExtensionComponent currentKAoSExtensionComponent : _components.values())
            if (currentKAoSExtensionComponent.getOntologicalAttributes().contains(associatedOntType))
                return currentKAoSExtensionComponent;

        KAoSExtensionComponent newComponent = null;

        if (_componentsClassNames == null)
            return null;
        
        // Check if there is a information in the boot file about the Java class implementing the needed component
        if (_componentsClassNames.containsKey(associatedOntType))
        {
            String className = (String)_componentsClassNames.get(associatedOntType);

            try
            {
                Class componentClass = null;

                try
                {
                    componentClass = Class.forName(className);
                }
                catch (java.lang.ClassNotFoundException cnfe)
                {
                    // This is what will happen if the class cannot be found
                    printDebugString(".getComponent: Unable to find component class - " + className, Logger.WARN);

                    throw cnfe;
                }

                try
                {
                    Constructor componentClassConstructor = componentClass.getConstructor(new Class[]{String.class, String.class});
                    newComponent = (KAoSExtensionComponent)componentClassConstructor.newInstance(KAoSPolicyBuilderUtil.getShortName(associatedOntType) 
                            + _repositoryType, associatedOntType);
                }
                catch (Exception e)
                {
                    printDebugString(".getComponent: error when creating a new instance of " + componentClass.getName() + " " + e, Logger.WARN);
                }

                // newComponent.addOntologicalAttribute(associatedOntType);
                // newComponent.setName(JenaListCellRenderer.getShortName(associatedOntType) + _repositoryType);
                
                if (props != null)
                    newComponent.setProperties(props);
                
                newComponent.setEnabledStatus(true);

                registerComponent(newComponent);
            }
            catch (Exception e)
            {
                e.printStackTrace();
                printDebugString(".getComponent: error when creating a new instance- see " + e, Logger.WARN);
            }
        }

        // If creation of the component was not successful try to find an existing component responsible for handling more generic ontology type
        for (String currentSuperConcept : _subConcepts.keySet())
        {
            Set<String> currentSuperConceptSubConcepts = _subConcepts.get(currentSuperConcept);

            // TODO: modify the algorithm in the future so the most specialized component is chosen
            if (currentSuperConceptSubConcepts.contains(associatedOntType))
                return (KAoSExtensionComponent)_componentsConceptMapping.get(currentSuperConcept);
        }

        return newComponent;
    }

    /*
     * @see kaos.core.csi.extension.KAoSExtensionComponentManager#changeComponentEnabledStatus(java.lang.String, boolean)
     */
    public void changeComponentEnabledStatus(String associatedOntType, boolean state)
    {
        KAoSExtensionComponent temp = (KAoSExtensionComponent)_components.get(associatedOntType);

        if (temp == null)
            return;

        temp.setEnabledStatus(state);
    }

    /*
     * @see kaos.core.csi.extension.KAoSExtensionComponentManager#getComponentEnabledStatus(java.lang.String)
     */
    public boolean getComponentEnabledStatus(String associatedOntType)
    {
        KAoSExtensionComponent temp = (KAoSExtensionComponent)_components.get(associatedOntType);

        if (temp == null)
            return false;

        return temp.getEnabledStatus();
    }

    /*
     * @see kaos.core.csi.extension.KAoSExtensionComponentManager#getAllKAoSExtensionComponents()
     */
    public List<KAoSExtensionComponent> getAllKAoSExtensionComponents()
    {
        List<KAoSExtensionComponent> ret = new Vector<KAoSExtensionComponent>();

        for (KAoSExtensionComponent currentKAoSExtensionComponent : _components.values())
            ret.add(currentKAoSExtensionComponent);

        return ret;
    }

    /*
     * @see kaos.core.csi.extension.KAoSExtensionComponentManager#removeAllKAoSExtensionComponents()
     */
    public void removeAllKAoSExtensionComponents()
    {
        _components = new Hashtable<String, KAoSExtensionComponent>();
        _componentsConceptMapping = new Hashtable<String, KAoSExtensionComponent>();
    }

    /*
     * @see kaos.core.csi.extension.KAoSExtensionComponentManager#getRepositoryTableInformation()
     */
    public Map<String, String> getRepositoryTableInformation()
    {
        Map<String, String> ret = new Hashtable<String, String>();

        if (_components.size() == 0)
            return ret;

        for (KAoSExtensionComponent currentKAoSExtensionComponent : _components.values())
        {
            Set<String> ontConcepts = currentKAoSExtensionComponent.getOntologicalAttributes();

            if (ontConcepts == null || ontConcepts.size() == 0)
                continue;

            for (String currentOntConcepts : ontConcepts)
                ret.put(currentOntConcepts, currentKAoSExtensionComponent.getName());
        }

        return ret;
    }

    // ------------- javax.agent.service.Service interface implementation - BEGIN

    /*
     * Obtain the properties of the service.
     * 
     * @return the properties of the service.
     */
    public ServiceProperties getServiceProperties() throws ServiceException, ServiceFailure
    {
        return null;
    }

    /*
     * Set the properties of the service.
     * 
     * @param props containing a set of the properties of the service.
     */
    public void setServiceProperties(ServiceProperties props) throws ServiceException, ServiceFailure
    {
    }

    public void initialize(ServiceProperties env) throws ServiceException
    {
        if (env == null)
            return;

        printDebugString("\n\n " + getRepositoryManagerType() + "ManagerImpl.initialize: received: env : " + env, Logger.INFO);

        // Read information about the Java classes implementing components from the boot file; associated with the javax.agent.service key
        _componentsClassNames = (BootProperties)env.get("javax.agent.service." + getRepositoryManagerType());

        for (Iterator<String> conceptsIT = _componentsClassNames.keySet().iterator(); conceptsIT.hasNext();)
        {
            String currentConcept = conceptsIT.next();
            KAoSExtensionComponent currentConceptKAoSComponent = getComponent(currentConcept, env);

            if (currentConceptKAoSComponent != null)
                _componentsConceptMapping.put(currentConcept, currentConceptKAoSComponent);
        }
    }

    // ------- javax.agent.service.Service interface implementation - END

    // --------------- PRIVATE METHODS BEGIN
    
    /*
     * A utility method to print a String. Add the name of the class printing the String.
     * 
     * @param msg The String to be printed.
     */
    protected static void printDebugString(String msg, int loggerDisplayLevel)
    {
        _logger.logMessage("\n                " + msg, loggerDisplayLevel);
    }

    private static Logger _logger = Logger.getLogger(KAoSExtensionComponentManagerImpl.class);
}