/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2012/04/02 07:12:11 $
 */

package kaos.core.csi.extension.history;

import kaos.core.csi.ActionInstanceDescription;
import kaos.core.csi.query.data.OntClassDescription;
import kaos.time.DurationDescription;

import java.util.List;

/**
 * This interface extends KAoSExtensionComponent interface. It adds a method signature that takes in an event class and 
 * returns the # events matching that event class from its set of logged events.
 */
public interface EventHistoryComponent
{
    /**
     * Log an event
     * 
     * @param event The event to be logged.
     */
    public void logEvent(ActionInstanceDescription event);
    
    /**
     * This method is used to find the list of occurrences of actual events of the type indicated by an event class.
     * 
     * @param eventClass        ActionInfo specifying the type of event class.
     * @param historyStartDuration   DurationDescription specifying the starting time in which event should be counted
     * @param historyEndDuration   DurationDescription specifying the ending time in which event should be counted
     * 
     * @return the events in this log that are instances of the specified event class.
     */
    public List<ActionInstanceDescription> findEventsOfType(OntClassDescription eventClass, DurationDescription historyStartDuration, DurationDescription historyEndDuration);

    public List<ActionInstanceDescription> getAllLogs();

    public List<ActionInstanceDescription> getLogsforActionType(String loginAction);
}
