/**
 * Last Modified by: $Author: auszok $ On: $Date: 2012/04/02 07:12:11 $
 */

package kaos.core.csi.extension.history;

import kaos.core.csi.ActionInstanceDescription;
import kaos.core.csi.extension.KAoSExtensionComponent;
import kaos.core.csi.extension.KAoSExtensionComponentManager;
import kaos.core.csi.extension.KAoSExtensionComponentManagerImpl;
import kaos.core.csi.extension.KAoSExtensionsTypes;
import kaos.core.csi.query.data.OntClassDescription;
import kaos.core.csi.query.data.OntClassPropertyDescription;
import kaos.ontology.vocabulary.ActionStatusConcepts;
import kaos.ontology.vocabulary.PolicyActionConcepts;
import kaos.time.DurationDescription;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Vector;

/**
 * This class extends guard repository manager implementation. It has a new method that finds the # events belonging to a specified event class. 
 * This finding is performed across all the components in this manager.
 */
public class EventsHistoryManager extends KAoSExtensionComponentManagerImpl implements EventHistoryComponent, KAoSExtensionComponentManager
{
    private static EventsHistoryManager __INSTANCE;
    private static boolean              __INITIALIZED = false;

    public static EventsHistoryManager getInstance()
    {
        if (__INITIALIZED)
            return __INSTANCE;

        __INSTANCE = new EventsHistoryManager(KAoSExtensionsTypes.HISTORY_MONITORS);
        __INITIALIZED = true;

        return __INSTANCE;
    }

    private EventsHistoryManager(String type)
    {
        super(type);
    }

    public void logEvent(ActionInstanceDescription event)
    {
        List<KAoSExtensionComponent> components = super.getAllKAoSExtensionComponents();

        String currentEventActionType = event.getActionClassName();

        for (KAoSExtensionComponent currentKAoSExtensionComponent : components)
            if (currentKAoSExtensionComponent.getOntologicalAttributes().contains(currentEventActionType))
            {
                if (currentKAoSExtensionComponent instanceof EventHistoryComponent)
                {
                    EventHistoryComponent currentKAoSEventHistoryComponent = (EventHistoryComponent)currentKAoSExtensionComponent;
                    currentKAoSEventHistoryComponent.logEvent(event);
                }
            }
    }

    /**
     * This method is used to find the list of occurrences of actual events of the type indicated by an event class.
     * 
     * @param eventClass ActionInfo object specifying the type of event class.
     * @param historyStartDuration   DurationDescription specifying the starting time in which event should be counted
     * @param historyEndDuration   DurationDescription specifying the ending time in which event should be counted
     * @return the list of events in all the components of this manager which are instances of the specified event class.
     */
    public List<ActionInstanceDescription> findEventsOfType(OntClassDescription eventClass, DurationDescription historyStartDuration, DurationDescription historyEndDuration)
    {
        List<KAoSExtensionComponent> components = super.getAllKAoSExtensionComponents();
        Vector<ActionInstanceDescription> result = new Vector<ActionInstanceDescription>();

        Set<String> currentEventActionTypes = eventClass.getClassTypes();

        // Iterate through active history monitors
        for (KAoSExtensionComponent currentKAoSExtensionComponent : components)
            if (currentKAoSExtensionComponent instanceof EventHistoryComponent)
            {
                boolean foundResult = false;
                
                // Check if this monitor has events of interest
                for (String currentEventActionType : currentEventActionTypes)
                    if (currentKAoSExtensionComponent.getOntologicalAttributes().contains(currentEventActionType))
                    {
                        EventHistoryComponent currentKAoSEventHistoryComponent = (EventHistoryComponent)currentKAoSExtensionComponent;
                        
                        // Count the events
                        result.addAll(currentKAoSEventHistoryComponent.findEventsOfType(eventClass, historyStartDuration, historyEndDuration));
                        
                        foundResult = true;
                        
                        break; // Use this monitor only once
                    }
                
                if (foundResult)
                    continue; // If found results for this history monitor then do not check if this is policy monitor
                
                // ---- Check if this is policy decision logger ----
                if ( ! currentKAoSExtensionComponent.getOntologicalAttributes().contains(PolicyActionConcepts.PolicyAction()))
                    continue;
                
                // Add counting policy decisions on this action if history has failure 
                if (eventClass.getProperty(ActionStatusConcepts.hasStatus()) == null)
                    continue;
                
                OntClassPropertyDescription hasStatus = eventClass.getProperty(ActionStatusConcepts.hasStatus());
                
                if ( ! hasStatus.getAllInstances().contains(ActionStatusConcepts.DefaultFailureActionStatus()))
                    continue;
                
                // The history is about the action failure so ask the policy decision logger how many time the given action type was not authorized
                
                if (! (currentKAoSExtensionComponent instanceof PolicyDecisionHistoryComponent))
                    continue;
                
                PolicyDecisionHistoryComponent currentPolicyDecisionHistoryComponent = (PolicyDecisionHistoryComponent)currentKAoSExtensionComponent;
                
   
                result.addAll(currentPolicyDecisionHistoryComponent.findAuthorizationFailureForAction(eventClass, historyStartDuration, historyEndDuration));
            }

        return result;
    }

    public List<ActionInstanceDescription> getLogsforActionType(String actionType)
    {
        List<ActionInstanceDescription> myResult = new ArrayList<ActionInstanceDescription>();

        List<KAoSExtensionComponent> components = super.getAllKAoSExtensionComponents();

        for (KAoSExtensionComponent currentKAoSExtensionComponent : components)
            if (currentKAoSExtensionComponent.getOntologicalAttributes().contains(actionType))
            {
                if (currentKAoSExtensionComponent instanceof EventHistoryComponent)
                {
                    EventHistoryComponent currentKAoSEventHistoryComponent = (EventHistoryComponent)currentKAoSExtensionComponent;
                    myResult.addAll(currentKAoSEventHistoryComponent.getAllLogs());
                }
            }

        return myResult;
    }

    public List<ActionInstanceDescription> getAllLogs()
    {
        List<ActionInstanceDescription> myResult = new ArrayList<ActionInstanceDescription>();

        List<KAoSExtensionComponent> components = super.getAllKAoSExtensionComponents();

        for (KAoSExtensionComponent currentKAoSExtensionComponent : components)
            if (currentKAoSExtensionComponent instanceof EventHistoryComponent)
            {
                EventHistoryComponent currentKAoSEventHistoryComponent = (EventHistoryComponent)currentKAoSExtensionComponent;
                myResult.addAll(currentKAoSEventHistoryComponent.getAllLogs());
            }

        return myResult;
    }
}