package kaos.core.csi.extension.history.logging;

import kaos.core.csi.ActionInstanceDescription;
import kaos.core.csi.extension.KAoSExtensionComponent;
import kaos.core.csi.extension.history.EventHistoryComponent;

import java.util.List;

/**
 * This interface defines methods to access logged events
 */
public interface HistoryMonitor extends EventHistoryComponent, KAoSExtensionComponent
{
    /**
     * Get all the events logged so far.
     * 
     * @return A Vector containing aid objects logged so far.
     */
    public List<ActionInstanceDescription> getAllLogs();

    /**
     * Delete all the events logged so far.
     */
    public void deleteAllLogs();

    /**
     * Get all events logged that contain the given action type.
     * 
     * @param actionType A String indicating the action type.
     * @return A vector containing aid objects logged for that action.
     */
    public List<ActionInstanceDescription> getLogsforActionType(String actionType);

    /**
     * Delete all logged events that contain the given action type.
     * 
     * @param actionType A String indicating the action type.
     * @return Vector A Vector containing deleted logs
     */
    public List<ActionInstanceDescription> deleteLogsforActionType(String actionType);

    /**
     * Get all logged events that contain the given actor.
     * 
     * @param actorID A String indicating the actor id.
     * @return A vector containing aid objects logged for that actor.
     */
    public List<ActionInstanceDescription> getLogsforActor(String actorID);

    /**
     * Delete all logged events that contain the given actor.
     * 
     * @param actorID A String indicating the actor id.
     * @return Vector A Vector containing deleted logs
     */
    public List<ActionInstanceDescription> deleteLogsforActor(String actorID);

    /**
     * This method returns the logged events specified by the given parameters. The input parameters are interpreted as follows:
     * 
     * @param actionType A String indicating the action type of requested logs. If it is null, 'all action types' would be assumed.
     * @param actorID A String indicating the actor of requested logs. If null, 'all actors' would be assumed.
     * @param startTime A long, indicating the start of time period for which logs are requested. If null, 'the begining of logs' time would be assumed.
     * @param endTime A long, indicating the end of time period for which logs are requested. If null, 'time of the latest log' would be assumed.
     * @param latest A boolean, indicating if only latest logs are requested. If it is set: a) start and end time parameters are ignored. b) If actionType is null, and actorID is
     *            null, the latest log for each actor is returned. c) If actorID is null and actionType is not null the latest logs for each actor for the given actionType are
     *            returned. d) If actorID is not null, the latest log for that actor is returned. If actionType also is not null, the latest log for that actor and that action is
     *            returned. If this boolean is not set, log are retrieved based on preceding parameters.
     */
    public List<ActionInstanceDescription> getSpecifiedLogs(String actionType, String actorID, String startTime, String endTime, Boolean latest);

    /**
     * This method calls 'getSpecifiedLogs' with its input parameters and then deletes the logs returned by that call. For method documentation, see the heading for
     * 'getSpecifiedLogs'.
     * 
     * @return Vector A Vector containing deleted logs
     */
    public List<ActionInstanceDescription> deleteSpecifiedLogs(String actionType, String actorID, String startTime, String endTime, Boolean latest);

    public void deleteLogs(List<ActionInstanceDescription> logs);
}
