package kaos.core.csi.extension.history.logging;

import kaos.core.csi.ActionInstanceDescription;
import kaos.core.csi.OntPropertyDescription;
import kaos.core.csi.extension.history.PolicyDecisionHistoryComponent;
import kaos.core.csi.query.data.OntClassDescription;
import kaos.ontology.vocabulary.ActionConcepts;
import kaos.ontology.vocabulary.PolicyActionConcepts;
import kaos.policy.guard.KAoSGuardPolicyProcessor;
import kaos.time.DurationDescription;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.Vector;

public class PolicyDecisionMonitorImpl extends HistoryMonitorImpl implements HistoryMonitor, PolicyDecisionHistoryComponent
{
	private List<String> PolicyActionSubclasses = Arrays.asList(PolicyActionConcepts.PolicyLoggingAction, PolicyActionConcepts.ObligateAction, 
			PolicyActionConcepts.PolicyModificationAction, PolicyActionConcepts.AddPolicyAction, PolicyActionConcepts.AuthorizationFailureAction, 
			PolicyActionConcepts.PolicyQueryAction, PolicyActionConcepts.GrantAuthorizationAction, PolicyActionConcepts.PolicyDecisionAction, 
			PolicyActionConcepts.RemovePolicyAction);
	
    public PolicyDecisionMonitorImpl(String name)
    {
        super(name, PolicyActionConcepts.PolicyAction());
        this.setSubclassesForAttribute(PolicyActionConcepts.PolicyAction(), new HashSet<String>(PolicyActionSubclasses));
    }

    public List<ActionInstanceDescription> findAuthorizationFailureForAction(OntClassDescription eventClass, DurationDescription historyStartDuration, DurationDescription historyEndDuration)
    {
        int startDuration;
        
        if (historyStartDuration != null)
            startDuration= historyStartDuration.getSeconds();
        else 
            startDuration = 0;

        int endDuration;
        
        if (historyEndDuration != null)
            endDuration= historyEndDuration.getSeconds();
        else 
            endDuration = 0;

        long currentTime = System.currentTimeMillis();
        
        // Calculate the time when the validity of event starts
        long earliestTimeOfEventValidity = currentTime - 1000 * endDuration;
        long latestTimeOfEventValidity = currentTime - 1000 * startDuration;
        
        Vector<ActionInstanceDescription> result = new Vector<ActionInstanceDescription>();
        
        for (ActionInstanceDescription currentEvent : _allLogs)
        {
            // Check if this is an authorization action
            if ( ! currentEvent.getActionClassName().equals(PolicyActionConcepts.AuthorizationFailureAction()))
                continue;
            
            // Check if the action is within the required period
            if (endDuration > 0)
            {
                if (currentEvent.getProperty(ActionConcepts.hasBeginTime()) == null)
                    continue;
                
                String _currentLogBeginTime = currentEvent.getProperty(ActionConcepts.hasBeginTime()).getFirstValue();
                
                if (_currentLogBeginTime == null)
                    continue;
                
                long currentLogBeginTime = (new Long(_currentLogBeginTime)).longValue();
                
                if (currentLogBeginTime <= earliestTimeOfEventValidity || currentLogBeginTime >= latestTimeOfEventValidity)
                    continue;
            }
             
            OntPropertyDescription failureActionProperty = currentEvent.getProperty(PolicyActionConcepts.onAction());
            
            if (failureActionProperty == null)
                continue;
            
            String failureActionID = failureActionProperty.getFirstValue();
            
            if (failureActionID == null)
                continue;
            
            Object _failureAction = failureActionProperty.getValueDescForValue(failureActionID);
            
            if ( ! (_failureAction instanceof ActionInstanceDescription))
                continue;
            
            ActionInstanceDescription failureAction = (ActionInstanceDescription)_failureAction;
            
            Set<String> missingProperties = null;
            if (eventClass.matchInstanceWithClass(failureAction, missingProperties) == KAoSGuardPolicyProcessor._FULL_MATCH_)
                result.add(failureAction);
        }

        return result;
    }
}