/*
 * ObligationMonitorImpl.java
 *
 * Created on April 5, 2007, 3:22 PM
 */

package kaos.core.csi.extension.obligation;

import kaos.core.csi.ActionInstanceDescription;
import kaos.core.csi.CSIFactory;
import kaos.core.csi.OntInstanceDescriptionImpl;
import kaos.core.csi.OntPropertyDescription;
import kaos.core.csi.extension.KAoSExtensionComponentImpl;
import kaos.core.csi.observe.KAoSObservable;
import kaos.core.csi.observe.KAoSObservableImpl;
import kaos.core.csi.observe.KAoSObserver;
import kaos.core.csi.ontology.OntologyService;
import kaos.core.csi.ontology.OntologyRepositoryServiceImpl;
import kaos.core.csi.policy.*;
import kaos.core.csi.query.data.OntClassDescription;
import kaos.core.csi.query.data.OntClassPropertyDescription;
import kaos.core.csi.transport.TransportSupport;
import kaos.core.csi.transport.TransportSupportImpl;
import kaos.core.service.boot.KAoSGuardBootstrapper;
import kaos.core.service.directory.DirectoryService;
import kaos.core.service.directory.KAoSAgentDescription;
import kaos.core.util.KAoSConstants;
import kaos.core.util.Logger;
import kaos.core.util.PolicyMsg;
import kaos.core.util.UniqueIdentifier;
import kaos.kpat.gui.PropertiesPanel;
import kaos.ontology.util.KAoSOntClassBuilder;
import kaos.ontology.util.KAoSOntClassBuilderImpl;
import kaos.ontology.util.SerializableOntModelImpl;
import kaos.ontology.vocabulary.ActionConcepts;
import kaos.ontology.vocabulary.PolicyActionConcepts;
import kaos.ontology.vocabulary.PolicyConcepts;
import kaos.ontology.vocabulary.timeConcepts;
import kaos.policy.guard.GuardRetriever;
import kaos.policy.guard.KAoSGuard;
import kaos.policy.information.PolicyInformation;
import kaos.policy.information.PreComputedPolicyDBManagerImpl;
import kaos.xmlschema.util.XMLSchemaBuilder;
import kaos.xmlschema.util.XMLSchemaBuilderImpl;

import javax.agent.service.transport.MessageTransportService;
import javax.agent.service.transport.TransportFailure;
import java.util.*;

/**
 * @author jlott
 */
public class ObligationMonitorImpl extends KAoSExtensionComponentImpl implements ObligationMonitor, KAoSObservable, PolicyListener
{
    private KAoSObservableImpl _observableImpl;
    private Hashtable          _obligationTable = new Hashtable();
    private PolicyChecking     _policyChecking;
    private Logger             _logger          = Logger.getLogger("kaos.core.csi.extension.obligation.ObligationMonitorImpl");
    private DirectoryService   _ontServ;
    private Timer              _timer           = new Timer(true);
    private Hashtable          _taskTable       = new Hashtable();
    private KAoSGuard          _guard;

    public ObligationMonitorImpl(String name, String ontType)
    {
        super(name, ontType);

        initObligationMonitor();
    }
    
    public ObligationMonitorImpl()
    {
        super("ObligationMonitor", ObligationMonitor.OBLIGATION_MONITOR_URL);

        initObligationMonitor();
    }
    
    private void initObligationMonitor()
    {
        TransportSupport ts = TransportSupportImpl.getInstance();
        String transportName = (String)ts.getSupportedTransportsNames().get(0);
        MessageTransportService mts = ts.getServiceRoot(transportName).getPreferredMessageTransportService();

        try
        {
            _observableImpl = new KAoSObservableImpl(mts, mts.newLocalLocator(), "ObligationMonitor_" + UniqueIdentifier.GenerateUID());
        }
        catch (TransportFailure ex)
        {
            ex.printStackTrace();
        }

        try
        {
            _policyChecking = CSIFactory.getPolicyChecking();
            PolicyManagementImpl.getInstance(transportName).addPolicyUpdateListener(this);
            _ontServ = KAoSGuardBootstrapper.getKAoSDirectoryProxyForTransport("tcp");
            _guard = GuardRetriever.getGuard();
        }
        catch (Exception xcp)
        {
            xcp.printStackTrace();
        }
    }

    // implements ObligationMonitor
    public void reportTrigger(ActionInstanceDescription trigger)
    {
        // TODO: right now we don't support policies of the type which have a deadline based on a trigger
        // try {
        // Vector obligations = _policyChecking.getObligationsForTriggerCondition(trigger, null);
        // synchronized (_obligationTable) {
        // _obligationTable.put(trigger.getName(), obligations);
        // }
        // Iterator it = obligations.iterator();
        // while (it.hasNext()) {
        // ActionInstanceDescription obligation = (ActionInstanceDescription) it.next();
        // // check to see if the obligation has a time limit, and if so, start a new thread
        // long timeLimit = -1;
        // if (timeLimit > 0) {
        // (new ObligationMonitorThread(trigger.getName(), obligation, timeLimit)).start();
        // }
        // }
        // }
        // catch (Exception xcp) {
        // xcp.printStackTrace();
        // }
    }

    public void obligationFulfilled(String triggerId, ActionInstanceDescription obligation)
    {
        removeObligation(triggerId, obligation);
    }

    public void obligationFailed(String triggerId, ActionInstanceDescription obligation)
    {
        removeObligation(triggerId, obligation);
        obligationViolated(obligation, "");
    }

    // implements KAoSObservable
    public void registerObserver(String updateType, Object updateCondition, KAoSObserver observer)
    {
        _observableImpl.registerObserver(updateType, updateCondition, observer);
    }

    public void deregisterObserver(String updateType, Object updateCondition, KAoSObserver observer)
    {
        _observableImpl.deregisterObserver(updateType, updateCondition, observer);
    }

    public void notify(String updateType, Object updateCondition, Object update)
    {
        _observableImpl.notify(updateType, updateCondition, update);
    }

    private void removeObligation(String triggerId, ActionInstanceDescription obligation)
    {
        String policyId = null;
        // see if the obligation contains the policyID that it fulfills
        OntPropertyDescription propDesc = obligation.getProperty(PolicyActionConcepts.involvedPolicy());
        if (propDesc != null)
        {
            policyId = (String)propDesc.getValues().iterator().next();
        }
        else
        {
            // try to determine the policyID by matching the obligation with an existing policy
            try
            {
                PreComputedPolicyDBManagerImpl policyManager = (PreComputedPolicyDBManagerImpl)_guard.getPolicyDBManager();
                for (PolicyInformation currentObligationPolicy : policyManager.getSortedObligationPolicies())
                {
                    Set<String> missingProperties = null;
                    if (_guard.getPolicyProcessor().matchPolicyWithAction(obligation, currentObligationPolicy, missingProperties, false))
                    {
                        // found a matching policy
                        policyId = currentObligationPolicy.getPolicyID();
                        break;
                    }
                }
            }
            catch (Exception xcp)
            {
                xcp.printStackTrace();
            }
        }

        if (policyId != null)
        {
            synchronized (_taskTable)
            {
                // TODO: this should really be a list of tasks, because a single policy might apply
                // to more than one agent. But for now (COI demo), there will only be one agent per guard.
                ObligationFulfillmentTask task = (ObligationFulfillmentTask)_taskTable.remove(policyId);
                if (task == null)
                {
                    _logger.logMessage("Could not find ObligationFulfillmentTask for policyId " + policyId, Logger.WARN);
                    return;
                }
                task.cancel();
                if (task._repeat)
                {
                    ObligationFulfillmentTask newTask = new ObligationFulfillmentTask(task._actorId, task._delay, true);
                    _taskTable.put(policyId, newTask);
                    _timer.schedule(newTask, task._delay, task._delay);
                }
            }
        }
        else
        {
            _logger.logMessage("Could not find a policy matching obligation " + obligation, Logger.WARN);
        }
    }

    private void obligationViolated(ActionInstanceDescription obligation, String message)
    {
        ObligationViolationException violation = new ObligationViolationException(message);
        
        ViolatedObligationDescription myViolatedObligationDescription = ViolatedObligationDescriptionImpl.createViolatedObligationDesc(obligation);
        violation.addUnfulfilledObligation(myViolatedObligationDescription);
        
        Map<String, String> condition = new Hashtable<String, String>();
        condition.put(KAoSConstants.NAME, obligation.getActor());
        
        notify(OBLIGATION_VIOLATION_UPDATE, condition, violation);
    }

    public void updatePolicies(List addedPolicies, List changedPolicies, List removedPolicies)
    {
        // TODO: handle changed policies
        synchronized (_taskTable)
        {
            Iterator it = removedPolicies.iterator();
            while (it.hasNext())
            {
                PolicyMsg policy = (PolicyMsg)it.next();
                ObligationFulfillmentTask task = (ObligationFulfillmentTask)_taskTable.remove(policy.getId());
                if (task != null)
                {
                    task.cancel();
                }
            }
        }

        checkAddedPoliciesForObligation(addedPolicies);
    }

    public void setPolicies(List policies)
    {
        // TODO: handle any existing obligation monitoring
        try
        {
            checkAddedPoliciesForObligation(policies);
        }
        catch (Exception xcp)
        {
            xcp.printStackTrace();
        }
    }

    private void checkAddedPoliciesForObligation(List policies)
    {
        for (Iterator it = policies.iterator();  it.hasNext(); )
        {
            PolicyMsg pMsg = (PolicyMsg)it.next();
            PolicyInformation pi = pMsg.getPolicyInformation();

            if (pi == null)
                continue;

            // TODO: Fix these strings
            if (pi.getModality().equals(PolicyConcepts.PosObligationPolicy()))
            {
                OntClassDescription actionDesc = (OntClassDescription)pi.getTriggerAction();

                OntClassPropertyDescription prop = actionDesc.getProperty(ActionConcepts.hasFulfillmentDurationValidity());
                if (prop != null)
                {
                    long timePeriod = 0;
                    String className = prop.getParentRangeName();
                    if (className != null)
                    {
                        if (className.startsWith(PropertiesPanel.GENERATED_CLASS_NAMESPACE))
                        {
                            try
                            {
                                SerializableOntModelImpl ontModel = _ontServ.getConstructedOntologyStructure(className);
                                KAoSOntClassBuilder builder = new KAoSOntClassBuilderImpl(ontModel);
                                timePeriod = getMillisecondsForTimeClass(builder);
                            }
                            catch (Exception xcp)
                            {
                                xcp.printStackTrace();
                            }
                        }
                    }
                    else
                    {
                        try
                        {
                            Set instances = prop.getAllInstances();
                            if (!instances.isEmpty())
                            {
                                String instanceName = (String)instances.iterator().next();
                                SerializableOntModelImpl ontModel = _ontServ.getConstructedOntologyStructure(instanceName);
                                OntInstanceDescriptionImpl instanceBuilder = new OntInstanceDescriptionImpl(ontModel);
                                timePeriod = getMillisecondsForTimeInstance(instanceBuilder);
                            }
                        }
                        catch (Exception xcp)
                        {
                            xcp.printStackTrace();
                        }
                    }
                    if (timePeriod > 0)
                    {
                        KAoSGuard guard = GuardRetriever.getGuard();
                        Collection<KAoSAgentDescription> agentDescs = guard.getGuardedAgentDescriptions();
                        
                        for( KAoSAgentDescription desc : agentDescs)
                        {
                            ObligationFulfillmentTask task = new ObligationFulfillmentTask(desc.getEntityNameAsString(), timePeriod, true);
                            
                            synchronized (_taskTable)
                            {
                                _taskTable.put(pi.getPolicyID(), task);
                                _timer.schedule(task, timePeriod, timePeriod);
                            }
                        }
                    }
                }
            }
        }
    }

    private long getMillisecondsForTimeClass(KAoSOntClassBuilder builder)
    {
        // check hours, minutes, seconds for now
        // TODO: check days, weeks, months, years
        List<String> v = builder.getPropertyNames();
        long time = 0;
        try
        {
            if (v.contains(timeConcepts.hours()))
            {
                String className = builder.getBasePropertyRangeClass(timeConcepts.hours());
                XMLSchemaBuilder xmlSchema = new XMLSchemaBuilderImpl(_ontServ.getSchema(className));
                // for now, assume the restrictions are equals, which means that all values should be the same
                List<String> values = xmlSchema.getCurrentRestrictionNames();
                if (!values.isEmpty())
                {
                    time += Integer.valueOf(xmlSchema.getRestrictionValue((String)values.get(1))).intValue() * 3600;
                }
            }
            if (v.contains(timeConcepts.minutes()))
            {
                String className = builder.getBasePropertyRangeClass(timeConcepts.minutes());
                XMLSchemaBuilder xmlSchema = new XMLSchemaBuilderImpl(_ontServ.getSchema(className));
                List<String> values = xmlSchema.getCurrentRestrictionNames();
                if (!values.isEmpty())
                {
                    time += Integer.valueOf(xmlSchema.getRestrictionValue((String)values.get(1))) * 60;
                }
            }
            if (v.contains(timeConcepts.seconds()))
            {
                String className = builder.getBasePropertyRangeClass(timeConcepts.seconds());
                XMLSchemaBuilder xmlSchema = new XMLSchemaBuilderImpl(_ontServ.getSchema(className));
                List<String> values = xmlSchema.getCurrentRestrictionNames();
                if (!values.isEmpty())
                {
                    time += Integer.valueOf(xmlSchema.getRestrictionValue((String)values.get(1))).intValue();
                }
            }
        }
        catch (Exception xcp)
        {
            xcp.printStackTrace();
        }

        // convert to ms
        time *= 1000;

        return time;
    }

    private long getMillisecondsForTimeInstance(OntInstanceDescriptionImpl instanceBuilder)
    {
        // check hours, minutes, seconds for now
        // TODO: check days, weeks, months, years
        Set s = instanceBuilder.getProperties();
        long time = 0;
        Iterator it = s.iterator();
        try
        {
            while (it.hasNext())
            {
                OntPropertyDescription prop = (OntPropertyDescription)it.next();
                Set values = prop.getValues();
                if (!values.isEmpty())
                {
                    String value = (String)values.iterator().next();
                    if (prop.getName().equals(timeConcepts.hours()))
                    {
                        time += Double.valueOf(value).doubleValue() * 3600;
                    }
                    else if (prop.getName().equals(timeConcepts.minutes()))
                    {
                        time += Double.valueOf(value).doubleValue() * 60;
                    }
                    else if (prop.getName().equals(timeConcepts.seconds()))
                    {
                        time += Double.valueOf(value).doubleValue();
                    }
                }
            }
        }
        catch (Exception xcp)
        {
            xcp.printStackTrace();
        }

        // convert to ms
        time *= 1000;

        return time;
    }

    private class ObligationFulfillmentTask extends TimerTask
    {
        public ObligationFulfillmentTask(String actorId, long delay, boolean repeat)
        {
            _actorId = actorId;
            _delay = delay;
            _repeat = repeat;
        }

        public void run()
        {
            ObligationViolationException violation = new ObligationViolationException("time expired");
            // violation.addUnfulfilledObligation(obligation);
            Hashtable condition = new Hashtable();
            condition.put(KAoSConstants.NAME, _actorId);
            ObligationMonitorImpl.this.notify(OBLIGATION_VIOLATION_UPDATE, condition, violation);
        }

        public String  _actorId;
        public long    _delay;
        public boolean _repeat;
    }

    private class ObligationMonitorThread extends Thread
    {
        public ObligationMonitorThread(String triggerName, ActionInstanceDescription obligation, long timeLimit)
        {
            _triggerName = triggerName;
            _obligation = obligation;
            _timeLimit = timeLimit;
        }

        public void run()
        {
            try
            {
                sleep(_timeLimit);
                // see if the obligation has not been fulfilled yet (it will still be in the table)
                boolean fulfilled = true;
                synchronized (_obligationTable)
                {
                    Vector obligations = (Vector)_obligationTable.get(_triggerName);
                    if (obligations != null)
                    {
                        if (obligations.contains(_obligation))
                        {
                            fulfilled = false;
                        }
                    }
                }
                if (!fulfilled)
                {
                    obligationViolated(_obligation, "time expired");
                }
            }
            catch (Exception xcp)
            {
                xcp.printStackTrace();
            }
        }

        public String                    _triggerName;
        public ActionInstanceDescription _obligation;
        public long                      _timeLimit;
    }

   
    @Override
    public void updateInForceValue(String policyId, Boolean inForce)
    {
        // TODO Auto-generated method stub
        
    }

   
}