package kaos.core.csi.extension.state;

import kaos.core.csi.OntPropertyDescription;
import kaos.core.csi.extension.KAoSExtensionComponentImpl;
import kaos.core.csi.query.data.OntClassDescription;

import javax.agent.service.ServiceProperties;
import javax.swing.*;
import javax.swing.border.BevelBorder;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class FlightConditionMonitor extends KAoSExtensionComponentImpl implements StateSensor, StateUpdateListener
{
   private StateManager staterManager;

   private JFrame f;
   private JPanel panel = new JPanel();
   private GridBagConstraints gbc = new GridBagConstraints();
   private HashMap<String, JLabel> mapStationsToJLabel = new HashMap<String, JLabel>();
   private boolean TEST_GUI_ONLY = false;

   public FlightConditionMonitor()
   {
      buildGUI();

      if (!TEST_GUI_ONLY)
      {
         staterManager = StateManagerImpl.getInstance();
         staterManager.registerAsListenerForStateInstance("Ceiling", this);
         staterManager.registerAsListenerForStateInstance("Visibility", this);
      }
   }

   public void initialize(ArrayList<String> parameters)
   {
      for (String station : parameters)
      {
         // show current database status for PNS
         addStation(station);

         if (!TEST_GUI_ONLY)
         {
//            String flightCondition = staterManager.getStateValueForEntity(station, "FlightCondition");
//            if (flightCondition == null)
//            {
//               flightCondition = "UNKNOWN";
//            }
//
//            updateStation(station, flightCondition);
         }
      }
   }

    @Override
    public Component getGui()
    {
        return this.panel;
    }

    private void buildGUI()
   {
      // main panel
      panel.setLayout(new GridBagLayout());
      panel.setBorder(new TitledBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED), "Flight Condition Monitor", TitledBorder.LEFT, TitledBorder.TOP));

      // layout components
      gbc.gridx = 0;
      gbc.gridy = 0;
      gbc.fill = GridBagConstraints.BOTH;
      gbc.weightx = 1;
      gbc.weighty = 1;
      gbc.gridwidth = 1;
      gbc.gridwidth = 1;

      f = new JFrame("Flight Condition Monitor");
      f.addWindowListener(new WindowAdapter()
      {
         public void windowClosing(WindowEvent e)
         {
            // do nothing
         }
      });

      f.getContentPane().add(new JScrollPane(panel), BorderLayout.CENTER);
      f.pack();
      f.setVisible(true);
   }

   private void addStation(String stationID)
   {
      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(new JLabel(stationID), gbc);
      gbc.gridx++;
      panel.add(new JLabel(" is currently: "), gbc);
      gbc.gridx++;
      JLabel stationLbl = new JLabel("UNKNOWN");
      panel.add(stationLbl, gbc);
      mapStationsToJLabel.put(stationID, stationLbl);
      f.pack();
   }

   private void updateStation(String stationID, String flightCondition)
   {
      JLabel stationLbl = mapStationsToJLabel.get(stationID);
      stationLbl.setText(flightCondition);
      f.pack();
   }

   public static void main(String[] args)
   {
      FlightConditionMonitor monitor = new FlightConditionMonitor();
      ArrayList<String> parameters = new ArrayList<String>();
      parameters.add("PNS");
      parameters.add("MOB");
      monitor.initialize(parameters);
   }

   public void stateUpdate(String stateID, String stateClass, List<OntPropertyDescription> properties)
   {
      System.out.println("Needs to be updated to new interface");
//      if (mapStationsToJLabel.get(entityID) == null)
//      {
//         addStation(entityID);
//      }

//      HashMap<String, String> mapStatesToValues = staterManager.getStateValuesForEntity(entityID);
//
//      if (mapStatesToValues == null)
//      {
//         staterManager.updateState(entityID, "FlightCondition", "UNKNOWN");
//         updateStation(entityID, "UNKNOWN");
//
//         return;
//      }
//
//      String ceiling = mapStatesToValues.get("Ceiling");
//      if (ceiling != null)
//      {
//         if (Integer.parseInt(ceiling) < 1000)
//         {
//            staterManager.updateState(entityID, "FlightCondition", "IMC");
//            updateStation(entityID, "IMC");
//
//            return;
//         }
//      }
//
//      String visibility = mapStatesToValues.get("Visibility");
//      if (visibility != null)
//      {
//         if (Double.parseDouble(visibility) < 3.0)
//         {
//            staterManager.updateState(entityID, "FlightCondition", "IMC");
//            updateStation(entityID, "IMC");
//
//            return;
//         }
//      }
//
//      if ((ceiling == null) || (visibility == null))
//      {
//         staterManager.updateState(entityID, "FlightCondition", "UNKNOWN");
//         updateStation(entityID, "UNKNOWN");
//
//         return;
//      }
//
//      staterManager.updateState(entityID, "FlightCondition", "VMC");
//      updateStation(entityID, "VMC");
   }

   @Override
   public String getName()
   {
      return null;  //To change body of implemented methods use File | Settings | File Templates.
   }

   @Override
   public void setEnabledStatus(boolean status)
   {
      //To change body of implemented methods use File | Settings | File Templates.
   }

   @Override
   public boolean getEnabledStatus()
   {
      return false;  //To change body of implemented methods use File | Settings | File Templates.
   }

   @Override
   public void addOntologicalAttribute(String ontAttribute)
   {
      //To change body of implemented methods use File | Settings | File Templates.
   }

   @Override
   public void setName(String componentName)
   {
      //To change body of implemented methods use File | Settings | File Templates.
   }

   @Override
   public void setProperties(ServiceProperties props)
   {
      //To change body of implemented methods use File | Settings | File Templates.
   }

   @Override
   public void registerInterest(OntClassDescription state)
   {
      //To change body of implemented methods use File | Settings | File Templates.
   }

   @Override
   public void deregisterInterest(OntClassDescription state)
   {
      //To change body of implemented methods use File | Settings | File Templates.
   }
}
