/**
 * 
 */
package kaos.core.csi.extension.state;

import kaos.core.csi.*;
import kaos.core.csi.extension.KAoSExtensionComponentImpl;
import kaos.core.csi.ontology.OntologyService;
import kaos.core.csi.query.data.ActionClassDescription;
import kaos.core.csi.query.data.OntClassDescription;

import java.awt.*;
import java.net.URISyntaxException;
import java.util.ArrayList;

public class StateMonitorImpl extends KAoSExtensionComponentImpl implements StateSensor
{
    private ActionClassDescription       _stateClassDesc;
    private OntInstanceDescription _currentState;
    private boolean                _initialized = false;

    public StateMonitorImpl(String name, String ontType)
    {
        super(name, ontType);

        initOntAttributes(ontType);
    }

    /*****************************************************************************************
     * kaos.core.csi.extension.KAoSExtensionComponent interface overridden methods - begin
     *****************************************************************************************/

    /**
     * Set the name of this component.
     * 
     * @param componentName String containing the name of this component.
     */
    public void setName(String componentName)
    {
        if (_componentName == null)
            _componentName = "StateMonitorFor-" + componentName;
    }

    /**
     * Add an ontological attribute the component is associated with.
     * 
     * @param ontAttribute String containing the name of the ontological attribute.
     */
    public void addOntologicalAttribute(String ontAttribute)
    {
        super.addOntologicalAttribute(ontAttribute);
        if (!_initialized)
            initOntAttributes(ontAttribute);
    }

    /*****************************************************************************************
     * kaos.core.csi.extension.KAoSExtensionComponent interface overriden methods - begin
     *****************************************************************************************/

    /*****************************************************************************************
     * kaos.core.csi.extension.state.StateMonitor interface implementation - begin
     *****************************************************************************************/

    /*
     * (non-Javadoc)
     * @see kaos.core.csi.extension.state.StateMonitor#updateProperty(java.lang.String, java.lang.String, java.lang.Object)
     */
    public void updateProperty(String propertyName, String value, Object valueDescription) throws InvalidPropertyForMonitoredStateException
    {
        if (_stateClassDesc.getProperty(propertyName) != null)
            throw new InvalidPropertyForMonitoredStateException("Specified property: " + propertyName + " is invalid for state: " + _stateClassDesc.getActionClassName());

        OntPropertyDescription propertyDesc = null;

        try
        {
            propertyDesc = new OntPropertyDescriptionImpl(propertyName);
        }
        catch (URISyntaxException urie)
        {
            urie.printStackTrace();
            throw new InvalidPropertyForMonitoredStateException("Specified property: " + propertyName + " is invalid!");
        }

        propertyDesc.addValue(value);

        if (valueDescription != null)
            propertyDesc.setValueDescForValue(value, valueDescription);

        _currentState.addProperty(propertyDesc);
    }

    /*
     * (non-Javadoc)
     * @see kaos.core.csi.extension.state.StateMonitor#removeProperty(java.lang.String)
     */
    public void removeProperty(String propertyName) throws InvalidPropertyForMonitoredStateException
    {
        if (_stateClassDesc.getProperty(propertyName) != null)
            throw new InvalidPropertyForMonitoredStateException("Specified property: " + propertyName + " is invalid for state: " + _stateClassDesc.getActionClassName());
        _currentState.removeProperty(propertyName);
    }

    /*
     * (non-Javadoc)
     * @see kaos.core.csi.extension.state.StateMonitor#getCurrentState()
     */
    public OntInstanceDescription getCurrentState()
    {
        return _currentState;
    }

    /*
     * (non-Javadoc)
     * @see kaos.core.csi.extension.state.StateMonitor#getStateInstance()
     */
    public ActionClassDescription getStateDescription()
    {
        return _stateClassDesc;
    }

    /*
     * (non-Javadoc)
     * @see kaos.core.csi.extension.state.StateMonitor#isStateConformantWithMonitoredState(java.lang.String)
     */
    public boolean isStateConformantWithMonitoredState(String stateName)
    {
        // TODO implement algorithm determining is the given state conforms with the monitored state
        return false;
    }

    /*****************************************************************************************
     * kaos.core.csi.extension.state.StateMonitor interface implementation - end
     *****************************************************************************************/
   
    // -------------- PRIVATE METHODS
    private void initOntAttributes(String ontType)
    {
        OntologyService ontService = null;

        try
        {
            ontService = CSIFactory.getOntologyService();
        }
        catch (Exception e)
        {
            e.printStackTrace();
            System.out.println("\n\t*** Unable to obtain a reference to OntologyService.  StateMonitor for " + ontType + " cannot be instantiated!\n");
            return;
        }

        try
        {
            _currentState = new OntInstanceDescriptionImpl(null, ontType);

        }
        catch (URISyntaxException urie)
        {
            urie.printStackTrace();
            System.out.println("\n\t*** State class name: " + ontType + " is syntactically incorrect. StateMonitor cannot be instantiated!\n ");
            return;
        }

        _stateClassDesc = new ActionClassDescription(ontType, ontService);
        _initialized = true;
    }

   @Override
   public void initialize(ArrayList<String> parameters)
   {
      //To change body of implemented methods use File | Settings | File Templates.
   }

    @Override
    public Component getGui()
    {
        return null;
    }

    @Override
   public void registerInterest(OntClassDescription state)
   {
      //To change body of implemented methods use File | Settings | File Templates.
   }

   @Override
   public void deregisterInterest(OntClassDescription state)
   {
      //To change body of implemented methods use File | Settings | File Templates.
   }
}