package kaos.core.csi.extension.state;

import kaos.core.csi.OntInstanceDescription;
import kaos.core.csi.OntInstanceDescriptionImpl;
import kaos.core.csi.extension.KAoSExtensionComponentImpl;
import kaos.core.csi.query.data.OntClassDescription;

import javax.agent.service.ServiceProperties;
import javax.swing.*;
import javax.swing.border.BevelBorder;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;

public class WeatherMonitor extends KAoSExtensionComponentImpl implements StateSensor
{
   private StateManager staterManager;
   private boolean TEST_GUI_ONLY = false;
   private JPanel _panel = null;

   public WeatherMonitor()
   {
      if (!TEST_GUI_ONLY)
      {
         staterManager = StateManagerImpl.getInstance();
      }
   }

    public void initialize(ArrayList<String> parameters)
    {
        this._panel = buildGUI(parameters);

        JFrame f = new JFrame("Weather Monitor");
        f.addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {
                // do nothing
            }
        });

        f.getContentPane().add(new JScrollPane(this._panel), BorderLayout.CENTER);
        f.pack();
        f.setVisible(true);
    }

    @Override
    public Component getGui()
    {
        return this._panel;
    }

    private JPanel buildGUI(ArrayList<String> stations)
   {
      // main panel
      JPanel panel = new JPanel();
      GridBagConstraints gbc = new GridBagConstraints();
      panel.setLayout(new GridBagLayout());
      panel.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));

      // layout components
      gbc.gridx = 0;
      gbc.gridy = 0;
      gbc.fill = GridBagConstraints.BOTH;
      gbc.weightx = 1;
      gbc.weighty = 1;
      gbc.gridwidth = 1;
      gbc.gridwidth = 1;

      for (String station : stations)
      {
         gbc.gridx++;
         panel.add(buildStationPanel(station), gbc);
      }

      return panel;
   }

   private JPanel buildStationPanel(String stationID)
   {
      final String identifier = stationID;

      // main panel
      JPanel panel = new JPanel();
      GridBagConstraints gbc = new GridBagConstraints();
      panel.setLayout(new GridBagLayout());
      panel.setBorder(new TitledBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED), stationID + " weather", TitledBorder.LEFT, TitledBorder.TOP));

      // layout components
      gbc.gridx = 0;
      gbc.gridy = 0;
      gbc.fill = GridBagConstraints.BOTH;
      gbc.weightx = 1;
      gbc.weighty = 1;
      gbc.gridwidth = 1;
      gbc.gridwidth = 1;

      gbc.gridx = 0;
      gbc.gridy++;
      gbc.gridwidth = 5;
      panel.add(new JLabel("Weather State Monitor"), gbc);
      gbc.gridwidth = 1;

      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(new JLabel("Temperature (F) - "), gbc);
      gbc.gridx++;
      panel.add(new JLabel("currently: "), gbc);
      final JLabel currentTemperatureLbl = new JLabel("unknown");
      gbc.gridx++;
      panel.add(currentTemperatureLbl, gbc);
      gbc.gridx++;
      panel.add(new JLabel("new state: "), gbc);
      final JTextField temperatureTF = new JTextField("75");
      gbc.gridx++;
      panel.add(temperatureTF, gbc);

      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(new JLabel("Ceiling (feet)- "), gbc);
      gbc.gridx++;
      panel.add(new JLabel("currently: "), gbc);
      final JLabel currentCeilingLbl = new JLabel("unknown");
      gbc.gridx++;
      panel.add(currentCeilingLbl, gbc);
      gbc.gridx++;
      panel.add(new JLabel("new state: "), gbc);
      final JTextField ceilingTF = new JTextField("1500");
      gbc.gridx++;
      panel.add(ceilingTF, gbc);

      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(new JLabel("Visibility (miles) - "), gbc);
      gbc.gridx++;
      panel.add(new JLabel("currently: "), gbc);
      final JLabel currentVisibilityLbl = new JLabel("unknown");
      gbc.gridx++;
      panel.add(currentVisibilityLbl, gbc);
      gbc.gridx++;
      panel.add(new JLabel("new state: "), gbc);
      final JTextField visibilityTF = new JTextField("7");
      gbc.gridx++;
      panel.add(visibilityTF, gbc);

      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(new JLabel("Precipitation - "), gbc);
      gbc.gridx++;
      panel.add(new JLabel("currently: "), gbc);
      final JLabel currentPrecipLbl = new JLabel("unknown");
      gbc.gridx++;
      panel.add(currentPrecipLbl, gbc);
      gbc.gridx++;
      panel.add(new JLabel("new state: "), gbc);
      final JComboBox precipCmb = new JComboBox();
      precipCmb.addItem("None");
      precipCmb.addItem("Rain");
      precipCmb.addItem("Snow");
      gbc.gridx++;
      panel.add(precipCmb, gbc);

      JButton updateWeatherBtn = new JButton("Update Weather");
      updateWeatherBtn.addActionListener(new ActionListener()
      {
         public void actionPerformed(ActionEvent ae)
         {
            try
            {
               if (!TEST_GUI_ONLY)
               {
                  String stateId = "wx@" + identifier;
                  OntInstanceDescription stateInstance = new OntInstanceDescriptionImpl(stateId, "Weather");
                  stateInstance.addProperty("Temperature", temperatureTF.getText());
                  stateInstance.addProperty("Ceiling", ceilingTF.getText());
                  stateInstance.addProperty("Visibility", visibilityTF.getText());
                  stateInstance.addProperty("Precipitation", precipCmb.getSelectedItem().toString());
                  staterManager.registerState(stateInstance);

                  currentTemperatureLbl.setText(staterManager.getPropertyValueForState(stateId, "Temperature").toArray()[0].toString());
                  currentCeilingLbl.setText(staterManager.getPropertyValueForState(stateId, "Ceiling").toArray()[0].toString());
                  currentVisibilityLbl.setText(staterManager.getPropertyValueForState(stateId, "Visibility").toArray()[0].toString());
                  currentPrecipLbl.setText(staterManager.getPropertyValueForState(stateId, "Precipitation").toArray()[0].toString());
               }
               else
               {
                  System.out.println("In GUI test mode. Set TEST_GUI_ONLY to false to use State Manager.");
               }
            }
            catch (Exception ex)
            {
               ex.printStackTrace();
            }
         }
      });
      gbc.gridx = 0;
      gbc.gridy++;
      gbc.gridwidth = 5;
      panel.add(updateWeatherBtn, gbc);
      gbc.gridwidth = 1;

      return panel;
   }

   public static void main(String[] args)
   {
      WeatherMonitor monitor = new WeatherMonitor();

      ArrayList<String> parameters = new ArrayList<String>();
      parameters.add("PNS");
      parameters.add("MOB");
      monitor.initialize(parameters);
   }

   public String getName()
   {
      return null;    // To change body of implemented methods use File | Settings | File Templates.
   }

   public void setEnabledStatus(boolean status)
   {
      // To change body of implemented methods use File | Settings | File Templates.
   }

   public boolean getEnabledStatus()
   {
      return false;    // To change body of implemented methods use File | Settings | File Templates.
   }

   public void addOntologicalAttribute(String ontAttribute)
   {
      // To change body of implemented methods use File | Settings | File Templates.
   }

   public void setName(String componentName)
   {
      // To change body of implemented methods use File | Settings | File Templates.
   }

   public void setProperties(ServiceProperties props)
   {
      // To change body of implemented methods use File | Settings | File Templates.
   }

   @Override
   public void registerInterest(OntClassDescription state)
   {
      //To change body of implemented methods use File | Settings | File Templates.
   }

   @Override
   public void deregisterInterest(OntClassDescription state)
   {
      //To change body of implemented methods use File | Settings | File Templates.
   }
}
