/**
 * Last Modified by: $Author: mvignati $
 * On: $Date: 2012/01/26 19:30:05 $
 */
 
package kaos.core.csi.observe;

import kaos.core.service.directory.KAoSEntityDescription;

import javax.agent.Locator;
import javax.agent.service.transport.NotLocatableException;
import javax.agent.service.transport.TransportFailure;
import java.io.Serializable;

/**
 * Defines the operations to map KAoSObservers with KAoSObservables and forwards updates from KAoSObservable 
 * to the associated KAoSObservers.
 * 
 * @author KAoS Team
 * $Revision: 1.9 $
 */
public interface ObserverManager
{
   /** 
    * Register the given KAoSObserver with the KAoSObservable, whose GUID is given for updates
    * described by the updateType and updateConditions.
    *
    * @param observer           KAoSObserver requesting registration for updates. 
    *
    * @param updateType         String indicating type of update the KAoSObserver is interested in.
    *
    * @param updateCondition    Serializable describing the conditions under which to send the update.
    *
    * @param observableGUID     String containing the GUID of the KAoSObservable to register with.
    *
    * @throws	    NotLocatableException   when the KAoSObservable cannot be located.
	*				                        
	* @throws	    TransportFailure        when transport services fails.
    *
    */
   public void registerObserver(KAoSObserver observer,
                                String updateType, 
                                Serializable updateCondition,
                                String observableGUID) 
     throws NotLocatableException, TransportFailure;
                                
   /** 
    * Register the given KAoSObserver with the KAoSObservable, whose KAoSEntityDescription is given, for updates
    * described by the updateType and updateConstraints.
    *
    * @param observer           KAoSObserver registering with for updates. 
    *
    * @param updateType         String indicating type of update the KAoSObserver is interested in.
    *
    * @param updateCondition    Object describing the conditions under which to send the update. 
    *
    * @param observableDesc     KAoSEntityDescription describing the observable. 
    *
    *
    * @throws	    NotLocatableException   when the KAoSObservable cannot be located.
	*				                        
	* @throws	    TransportFailure        when transport services fail.
    
    */
   public void registerObserver(KAoSObserver observer,
                                String updateType, 
                                Serializable updateCondition,
                                KAoSEntityDescription observableDesc)
    throws NotLocatableException, TransportFailure;
    
   /** 
    * Register the given KAoSObserver with the KAoSObservable, whose GUID is given for updates
    * described by the updateType and updateConditions.
    *
    * @param observer           KAoSObserver requesting registration for updates. 
    *
    * @param updateType         String indicating type of update the KAoSObserver is interested in.
    *
    * @param updateCondition    Serializable describing the conditions under which to send the update.
    *
    * @param observableGUID     String containing the GUID of the KAoSObservable to register with.
    
    * @param brokerGUID         String containing the GUID of the broker, through which to reach the observable.
    *
    * @throws	    NotLocatableException   when the KAoSObservable or broker cannot be located.
	*				                        
	* @throws	    TransportFailure        when transport services fail.
                               the rgistration will be processed.
    */
   public void registerObserver(KAoSObserver observer,
                                String updateType, 
                                Serializable updateCondition,
                                String observableGUID,
                                String brokerGUID)
        throws NotLocatableException, TransportFailure;
                                

    public void registerObserver(KAoSObserver observer,
                                 String updateType,
                                 Serializable updateCondition,
                                 String remoteEntityId,
                                 Locator remoteEntityLocator) throws NotLocatableException, TransportFailure;

   public void registerKAoSExtensionComponentObserver(String repType,
                                                       String ontType,
                                                       KAoSObserver observer,
                                                       String updateType, 
                                                       Serializable updateCondition,
                                                       KAoSEntityDescription entityDesc)
        throws NotLocatableException, TransportFailure;
    
   /** 
    * Deregister the given KAoSObserver from the registry of observers associated
    * with the KAoSObservable, whose GUID is given.
    *
    * @param observer           KAoSObserver deregistering. 
    *
    * @param updateType         String indicating type of update the KAoSObserver is no longer interested in.
    *
    * @param updateCondition    Serializable containing the condition the KAoSObserver is no longer interested in.
    *
    * @param observableGUID     String specifying the GUID of the observable, from which to deregister 
    *                           for the given update type.
    *
    * @throws	    NotLocatableException   when the KAoSObservable cannot be located.
	*				                        
	* @throws	    TransportFailure        when transport services fail.
    */
   public void deregisterObserver(KAoSObserver observer,
                                  String updateType,
								  Serializable updateCondition,
                                  String observableGUID)
        throws NotLocatableException, TransportFailure;
   
   /** 
    * Deregister the given KAoSObserver from the registry of observers asoociated
    * with the KAoSObservable, whose KAoSEntityDescription is given.
    *
    * @param observer           KAoSObserver deregistering. 
    *
    * @param updateType         String indicating type of update the KAoSObserver is no longer interested in.
    *
    * @param updateCondition    Serializable containing the condition the KAoSObserver is no longer interested in.
    *
    * @param observableDesc     KAoSEntityDescription of the observable, from which to deregister 
    *                           for the given update type.
    *
    * @throws	    NotLocatableException   when the KAoSObservable cannot be located.
	*				                        
	* @throws	    TransportFailure        when transport services fail.
    
    */
   public void deregisterObserver(KAoSObserver observer,
                                  String updateType,
								  Serializable updateCondition,
                                  KAoSEntityDescription observableDesc)
    throws NotLocatableException, TransportFailure;
   
    public void deregisterObserver(KAoSObserver observer,
                                   String updateType,
                                   Serializable updateCondition,
                                   String observableId,
                                   Locator observableLoc) throws NotLocatableException, TransportFailure;
} 