/**
 * Last Modified by: $Author: mvignati $
 * On: $Date: 2012/01/26 19:30:05 $
 */
 
package kaos.core.csi.observe;

import kaos.core.service.util.CallbackTransportImpl;
import kaos.core.service.util.CallbackTransportInfo;
import kaos.core.util.Logger;
import kaos.core.util.MethodCallRequestMsg;
import kaos.core.util.Msg;

import javax.agent.Locator;
import javax.agent.service.transport.MessageTransportService;
import javax.agent.service.transport.NotLocatableException;
import javax.agent.service.transport.TransportFailure;
import java.io.Serializable;
import java.util.Vector;

/**
 * RemoteKAoSObserverImpl is used to forward updates from the observable to the ObserverManager, 
 * which in turn, forwards the update to the correct observer.
 * 
 * @author KAoS Team
 * $Revision: 1.11 $
 */

public class RemoteKAoSObserverImpl implements RemoteKAoSObserver
{
    /*
     * Instance Variables
     */
    
    /**
	 * 
	 */
	private static final long serialVersionUID = 2472901642708194051L;
	protected String                          _observerID;
   // private String                          _senderID;
    protected static Logger                   _logger = Logger.getLogger(RemoteKAoSObserverImpl.class.getName());
    
    private CallbackTransportImpl           _callback;
    /*
     * Constructor
     */
    public RemoteKAoSObserverImpl(Locator receiverLocator, String receiverName, String observerID)
	{
		_callback = new CallbackTransportImpl(receiverLocator,receiverName);
		_observerID = observerID;
    } 
    
    /*
     * Instance Methods
     */
    
    /** 
     * Receive an update from the KAoSObservable when the KAoSObservable has a change or event to report. 
     *
     * @param observableID       String identifying the KAoSObservable sending the update. 
     *
     * @param updateType         String indicating type of update.
     *
     * @param update             Object containing the update. 
     *
     */
    public void update(String observableID,
                       String updateType, 
					   Object updateCondition,
                       Object update)
		throws NotificationException		
    {
        //System.out.println("\n  RemoteKAoSObserver.update: updateType" + updateType + " update value: " + update);
        if ( (observableID == null)  || (updateType == null) || (update == null)) {
            System.out.println("\n  RemoteKAoSObserver.update: One of the arguments is null: \n updateType" + updateType 
                           + " update value: " + update + "\n   observable id: " + observableID 
                           + "\n    Unable to forward update!");
            return;
        }
                           
        
        if ( ! (update instanceof Serializable ) ) {
	        System.out.println("\n RemoteKAoSObserver.update:  received update is not Serializable: " 
	                           +  update + "\n    Unable to foward the update to observer!  " + _observerID);// ;
	        return;
	    }
	     // Create MethodCallRequestMsg 
        MethodCallRequestMsg request = new MethodCallRequestMsg();
		Vector args = new Vector(4);
	    args.addElement(observableID);
	    args.addElement(updateType);
		if(updateCondition == null){
			updateCondition = new String("no-condition");	
		}
		args.addElement(updateCondition);
	    args.addElement((Serializable)update);
		
	    //set method o request action and add args
	    request.setMethodNameArgs("update", args);
		try{
			sendMessageObj(request);
		}
        catch(TransportFailure e){
            //logMessage("", e);
			throw new NotificationException("Problem notifying: " + e.getMessage());
        }
        catch(NotLocatableException e2){
            //logMessage("", e2);
			throw new NotificationException("Problem notifying: " + e2.getMessage());
        }		
		
    }
    
    /** 
    * Return the unique identifier of this observer.
    *
    * @return   String  containing the unique ID of this observer.
    */
   public String getID()
   {
        return _observerID;
   }
   
   /** 
    * Set the transport parameters of the observable, so this remote observer object will be able to 
    * send messages back to the ObserverManager.
    *
    * @param msgTransportService    MessageTransportService that can be used to send messages out.
    *
    * @param senderLocator          Locator of the sender to include in all outgoing TransportMessages. 
    *
    * @param senderName             String containing the sender name to include in all outgoing TransportMessages.  
    *
    */
    public void setTransportInfo(MessageTransportService msgTransportService,
                                Locator senderLocator,
                                String senderName)
    {
        _callback.setTransportInfo(msgTransportService,
                                   senderLocator,
                                   senderName);
    }
    
    public boolean equals(Object o)
    {
        if ( o == this )
            return true;
      
        if ( !(o instanceof RemoteKAoSObserver) )
            return false;
        RemoteKAoSObserver remoteO =  (RemoteKAoSObserver) o;
        
        return ( _observerID.equals(remoteO.getID()))  && (_callback.equals(remoteO.getTransportCallback()));   
    }
    
    public int hashCode()
    {
        if ( _callback == null )
         return 0x0F0F0F0F;

      return _observerID.hashCode() + _callback.hashCode();
    }
    
    public CallbackTransportInfo getTransportCallback()
    {
        return _callback;
    }
    
    public String toString()
    {
        StringBuffer buffer = new StringBuffer();
        buffer.append("Remote Observer ID: "+ _observerID + "\n"); 
        buffer.append("Remote Observer Locator: " + _callback.getReceiverLocator() + "\n");  
        buffer.append("Remote Observer Name: " + _callback.getReceiverName() + "\n"); 
        return buffer.toString(); 
    }
    
    /////////////////////////////////////////////////////////
    protected void sendMessageObj(Msg msg)
		throws TransportFailure, NotLocatableException
    {
        try {
            
            _callback.sendMsg(msg);
        }
        catch(TransportFailure e){
            //logMessage("", e);
			throw e;
        }
        catch(NotLocatableException e2){
            //logMessage("", e2);
			throw e2;
        }
    }
    
    // Utility methods ///////////////////////////////////////////
      
    protected void logMessage(String msg)
    {
   		System.out.println(msg);
    }
	
    protected void logMessage(String msg, Exception e)
    {
   		System.out.println(msg);
   	    e.printStackTrace();
    }
  } // RemoteKAoSObserverImpl