/**
 * Last Modified by: $Author: auszok $ On: $Date: 2012/05/16 17:14:47 $
 */

package kaos.core.csi.ontology;

import kaos.core.csi.query.QueryKAoSOntologyService;
import kaos.ontology.util.SerializableOntModelImpl;
import kaos.xmlschema.repository.XMLRepository;

import javax.agent.service.directory.DirectoryFailure;

import java.io.IOException;
import java.util.List;

/**
 * Defines the operations allowing to access KAoS constructed ontology structures
 * Adds XML Schema repository access and KAoS Ontology specific queries to the generic ontology service functionality
 * 
 * @author KAoS Team
 * @version 1.0 $Revision: 1.2 $
 */
public interface OntologyRepositoryService extends OntologyService, XMLRepository, QueryKAoSOntologyService
{
    /**
     * This method retrieves all user defined ontology structures in the repository.
     * 
     * @return list containing SerializableOntModelImpl objects
     */
    public List<SerializableOntModelImpl> getConstructedOntologyStructures() throws DirectoryFailure;

    /**
     * This method retrieves user defined ontology structure identified by the id from the repository.
     * 
     * @param structureId
     * @return SerializableOntModelImpl objects
     */
    public SerializableOntModelImpl getConstructedOntologyStructure(String structureId) throws OntologyService.ReasoningException,
            UnknownConceptException, IOException, DirectoryFailure;

    /**
     * Adds the constructed ontology structure to Directory Service Proxy
     * 
     * @param list of SerializableOntModelImpl object
     * @throws OntologyService.ReasoningException
     * @throws IOException
     * @throws DirectoryFailure
     */
    public void addConstructedOntologyStructures(List<SerializableOntModelImpl> list) throws OntologyService.ReasoningException, IOException,
            DirectoryFailure;

    /**
     * Adds ontology included in the provide objects into the ontology repository.
     * 
     * @param myOntModel The object of type Jena OntModel containing the added ontology
     * @param recursiveLoad Indicates if the imported ontologies (by the ontology contained in myOntModel)
     *            should also be recursively loaded.
     * @throws OntologyService.ReasoningException or IOException when loading of the ontology failed.
     */
    public void addConstructedOntologyStructure(SerializableOntModelImpl myOntModel) throws OntologyService.ReasoningException, IOException,
            DirectoryFailure;

    /**
     * This method removes a user defined ontology structure identified by the id from the repository.
     * 
     * @param structureId: uri name of the concept
     */
    public void removeConstructedOntologyStructure(String structureId) throws DirectoryFailure;

    /**
     * This method removes multiple user defined ontology structure identified by the id from the repository.
     * 
     * @param structureIds of uri's identifying the concepts to remove
     */
    public void removeConstructedOntologyStructures(List<String> structureIds) throws DirectoryFailure;
}