/**
 * Last Modified by: $Author: mvignati $
 * On: $Date: 2012/01/26 19:30:01 $
 */
package kaos.core.csi.policy;

import kaos.core.csi.ActionInstanceDescriptionImpl;
import kaos.core.csi.OntPropertyDescription;
import kaos.core.csi.OntPropertyDescriptionImpl;
import kaos.ontology.vocabulary.PolicyConcepts;

import java.net.URISyntaxException;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

/**
 * This class contains methods that allow to set and retrieve alternate actors of the obligation described by this ObligationActionDescription.
 * Those actors may be asked to perform the obligation if the primary obligation actor is unable to execute the obligation.
 *   
 *   $Revision: 1.5 $
 */
public class ObligationActionDescriptionImpl extends ActionInstanceDescriptionImpl implements ObligationActionDescription
{
    private static final long serialVersionUID = 3132220164187189515L;

    /*
     * Create an instance of the ObligationActionDescription described by the given arguments.
     * 
     * @param actionId String containing the id of the given action.
     * @param actionClassName String containing the ontological name of the given action.
     * @param actor String containing the id of the actor performing the action.
     * 
     * @throws URISyntaxException if the argument to create an URI is syntactically incorrect.
     */
    public ObligationActionDescriptionImpl(String actionId, String actionClassName, String actor) throws URISyntaxException
    {
        super(actionId, actionClassName, actor);
    }

    /*
     * Create an instance of the ObligationActionDescription described by the given arguments.
     * 
     * @param actionId String containing the id of the given action.
     * @param actionClassName String containing the ontological name of the given action. 
     * @param actor String containing the id of the actor performing the action.
     * @param properties HashMap containing the ontological names and their values of properties of the given action.
     * 
     * @throws URISyntaxException if the argument to create an URI is syntactically incorrect .
     */
    public ObligationActionDescriptionImpl(String actionId, String actionClassName, String actor, Map<String, Object> properties) throws URISyntaxException
    {
        super(actionId, actionClassName, actor, properties);
    }

    /*
     * Create an instance of the ObligationActionDescription described by the given OWL String.
     * 
     * @param owlString String the action id, class name, actor id and, optionally, properties.
     * 
     * @throws URISyntaxException if the argument to create an URI is syntactically incorrect .
     */
    public ObligationActionDescriptionImpl(String owlString) throws URISyntaxException
    {
        super(owlString);
    }

    /*
     * @see kaos.core.csi.policy.ObligationActionDescription#getAlternateActors()
     */
    public Set<String> getAlternateActors()
    {
        OntPropertyDescription alternateActors = getProperty(PolicyConcepts.hasAlternativeActor());

        if (alternateActors != null)
            return alternateActors.getValues();

        return null;
    }

    /*
     * @see kaos.core.csi.policy.ObligationActionDescription#setAlternateActors(java.util.Set)
     */
    public void setAlternateActors(Set<String> alternateObligationActors)
    {
        OntPropertyDescription hasAlternateActors = null;

        try
        {
            hasAlternateActors = new OntPropertyDescriptionImpl(PolicyConcepts.hasAlternativeActor());
        }
        catch (URISyntaxException use)
        {
            use.printStackTrace();
            return;
        }

        for (Iterator<String> actorsIt = alternateObligationActors.iterator(); actorsIt.hasNext();)
            hasAlternateActors.addValue(actorsIt.next());

        addProperty(hasAlternateActors);
    }
}