/**
 * Last Modified by: $Author: mvignati $
 * On: $Date: 2012/01/26 19:30:01 $
 */

package kaos.core.csi.policy;

import java.util.ArrayList;
import java.util.List;

/**
 *   ObligationViolationException defines an exception, which is thrown when the evaluation of
 *   a trigger against applicable obligation policy determines that some part of obligation 
 *   constraint has been violated, or that for some reason an obligation cannot be fulfilled.
 *
 *   @author         KAoS Team
 *   $Revision: 1.5 $
 */
public class ObligationViolationException extends Exception
{
    private static final long serialVersionUID = -4880938040515915768L;
    
    private List<ViolatedObligationDescription>              _violatedObligations;

    /*
     * Constructs a new ObligationViolationException with no detailed message.
     */
    public ObligationViolationException()
    {
        super();
    }

    /*
     * Constructs a new ObligationViolationException with a detailed message. A detail message is a String that describes the exception.
     * 
     * @param msg String containing the detailed message.
     */
    public ObligationViolationException(String msg)
    {
        super(msg);
    }

    /*
     * Add an ActionInstanceDescription describing an violated obligation.
     * 
     * @param violatedObligation ActionInstanceDescription of the violated obligation to be added to the list.
     */
    public void addUnfulfilledObligation(ViolatedObligationDescription violatedObligation)
    {
        if (_violatedObligations == null)
            _violatedObligations = new ArrayList<ViolatedObligationDescription>();

        _violatedObligations.add(violatedObligation);
    }

    public List<ViolatedObligationDescription> getViolatedObligations()
    {
        return _violatedObligations;
    }

    /*
     * Set _violatedObligations to the given List. This operation will replace the existing _violatedObligations.
     * 
     * @param violatedObligations List of ActionInstanceDescriptions describing violated obligations.
     */
    public void setViolatedObligations(List<ViolatedObligationDescription> violatedObligations)
    {
        if (violatedObligations == null)
            return;
        
        _violatedObligations = violatedObligations;
    }
}
