/**
 * Last Modified by: $Author: auszok $ On: $Date: 2012/03/05 04:43:46 $
 */
package kaos.core.csi.policy.enforcement;

import kaos.core.csi.CSIFactory;
import kaos.core.csi.extension.KAoSExtensionComponentImpl;
import kaos.core.csi.policy.PolicyChecking;
import kaos.core.util.Logger;

import java.util.Vector;

public class EnforcerImpl extends KAoSExtensionComponentImpl implements Enforcer
{
    protected Logger         _logger = Logger.getLogger(this);
    protected PolicyChecking _policyChecking;

    private EnforcerManager  _enforcerManager;

    public EnforcerImpl(String name, Vector ontTypes)
    {
        super(name, ontTypes);
        if (!initEnforcer())
        {
            System.out.println(" Unable to initialize Enforcer!  Exiting...");
            return;
        }
    }

    public EnforcerImpl(String name, String ontType)
    {
        super(name, ontType);
        if (!initEnforcer())
        {
            System.out.println(" Unable to initialize Enforcer!  Exiting...");
            return;
        }
    }

    private boolean initEnforcer()
    {
        setEnabledStatus(false); // superclass sets it to true by default
        if (!initEnforcerManager())
            return false;

        if (!_enforcerManager.registerComponent(this))
            return false;

        try
        {
            _policyChecking = CSIFactory.getPolicyChecking();
        }
        catch (Exception e)
        {
            e.printStackTrace();
            System.out.println(" Unable to obtain Policy Checking service for the Enforcer!  Exiting...");
            return false;
        }
        setEnabledStatus(true);

        return true;
    }

    private boolean initEnforcerManager()
    {
        try
        {
            _enforcerManager = CSIFactory.getPolicyEnforcement();
        }
        catch (Exception e)
        {
            e.printStackTrace();
            if (_logger.isEnabledFor(Logger.WARN))
                _logger.logMessage(".initEnforcerManager: Unable to obtain a reference to the EnforcerManager! ", Logger.WARN);
            return false;
        }
        return true;
    }

    /**
     * @param args
     */
    public static void main(String[] args)
    {

    }
}
