
/**
 * Last Modified by: $Author: mvignati $
 * On: $Date: 2012/01/26 19:30:06 $
 */
package kaos.core.csi.policy.enforcement;

import kaos.core.csi.extension.KAoSExtensionComponentManagerImpl;
import kaos.core.csi.extension.KAoSExtensionsTypes;
import kaos.core.util.Logger;
import kaos.policy.enforcement.ActionEnforcer;

import java.io.File;
import java.io.FileInputStream;
import java.util.Properties;
import java.util.StringTokenizer;

/**
 * Defines the operations to check permissions, get obligations, analyze policies, etc.
 *
 * @author KAoS Team
 * $Revision: 1.9 $
 */
public class EnforcerManagerImpl extends KAoSExtensionComponentManagerImpl implements EnforcerManager
{
   /**
    * Comment for <code>serialVersionUID</code>
    */
   private static final long serialVersionUID = -7635148275549105848L;

   private static EnforcerManagerImpl __INSTANCE;
   private static boolean __INITIALIZED = false;
   public static final String ENFORCER_PROPERTY = "kaos.enforcer";
   private Logger _logger = Logger.getLogger(this);

   /**
    * Private constructor
    */
   private EnforcerManagerImpl()
   {
      super(KAoSExtensionsTypes.ENFORCERS);
   }

   /**
    * Retrieve an instance of PolicyEnforcement.
    *
    * @return a singleton, PolicyEnforcement.
    */
   public static EnforcerManager getInstance()
   {
      if (__INITIALIZED)
         return __INSTANCE;

      __INSTANCE = new EnforcerManagerImpl();
      __INITIALIZED = true;

      return __INSTANCE;
   }

   /**
    *  kaos.core.csi.policy.enforcement.EnforcerManager interface implementation - begin
    */

   /**
    * @see kaos.core.csi.policy.enforcement.EnforcerManager#getEnforcer(java.lang.String)
    */
   public Enforcer getEnforcer(String ontologyAttribute)
   {
      return (Enforcer) getComponent(ontologyAttribute, null);
   }

   /**
    *  kaos.core.csi.policy.enforcement.EnforcerManager interface implementation - end
    */

   public void loadConfiguration()
   {
      createActionEnforcers();
   }

   private void createActionEnforcers()
   {
      // create enforcers monitors from configuration file
      String configFileName = System.getProperty(ENFORCER_PROPERTY);
      if (configFileName == null)
      {
         if (_logger.isEnabledFor(Logger.INFO))
            _logger.logMessage("no enforcer configuration property found: " + ENFORCER_PROPERTY
                               + " (e.g. kaos.enforcer=.\\config\\guard\\enforcer.properties)", Logger.INFO);

         return;
      }

      File configFile = new File(configFileName);
      if (configFile.isFile())
      {
         Properties properties = new Properties();
         FileInputStream input = null;

         try
         {
            input = new FileInputStream(configFile);
            properties.load(input);


            EnforcerManager enforcerManager = EnforcerManagerImpl.getInstance();
            for (Object key : properties.keySet())
            {
               // find state to monitor for
               String classAndParametersString = properties.getProperty((String) key);

               // federate property is an url so the parser returns just http and the rest a a value
               if (key.equals("http") && (classAndParametersString.indexOf('=') > 0))
               {
                  // This is a federate property
                  String wholePropertyWithValue = key + classAndParametersString;
                  key = wholePropertyWithValue.substring(0, wholePropertyWithValue.indexOf('=')).trim();
                  classAndParametersString = wholePropertyWithValue.substring(wholePropertyWithValue.indexOf('=') + 1).trim();
               }

               System.out.println("creating enforcer for " + key + " using " + classAndParametersString);

               // instantiate appropriate class
               StringTokenizer st = new StringTokenizer(classAndParametersString, " \t");
               String enforcerClassName = (String) st.nextToken();
               Class enforcerClass = Class.forName(enforcerClassName);
               ActionEnforcer enforcer = (ActionEnforcer) enforcerClass.newInstance();
               enforcerManager.registerComponent(enforcer);

               // initialize class
//             ArrayList<String> parameters = new ArrayList<String>();
//             while (st.hasMoreTokens())
//             {
//                parameters.add(st.nextToken());
//             }
//
//             enforcer.initialize(parameters);
            }
         }
         catch (Exception e)
         {
            e.printStackTrace();
         }
      }
      else
      {
         if (_logger.isEnabledFor(Logger.INFO))
            _logger.logMessage("no enforcer configuration file found: " + configFileName, Logger.INFO);
      }
   }

   // ///////////////////////////   PRIVATE METHODS BEGIN

   protected void logMessage(String msg, int logLevel)
   {
      if (_logger.isEnabledFor(logLevel))
         _logger.logMessage(msg, logLevel);
   }
}
