/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2012/04/14 05:50:40 $
 */
 
package kaos.core.csi.query.data;

import kaos.core.csi.ontology.OntologyService;
import kaos.core.csi.ontology.OntologyService.ReasoningException;
import kaos.core.csi.query.QueryState.UnknownConceptException;

import javax.agent.service.directory.DirectoryFailure;

import java.util.Collection;
import java.util.List;
import java.util.Vector;

/**
 *   This class contains information describing an Action class.
 *
 *   $Revision: 1.4 $
 */
public class ActionClassDescription extends OntClassDescription
{    
    private static final long serialVersionUID = -1618846656318110844L;
    
    private String actionClassName;
    private List<ActionPropertyDescription> targets;
        
    public ActionClassDescription (String _actionClassName, List<ActionPropertyDescription> _targets)
    {
        super(_actionClassName);
        
        actionClassName = _actionClassName;
        targets = _targets;
    }
        
    public ActionClassDescription (String _actionClassName, OntologyService _myOntologyContext)
    {
        super(_actionClassName);
        
        actionClassName = _actionClassName;
        
        try 
        {
            targets = new Vector<ActionPropertyDescription>();
            
            OntClassDescription _targetClassDesc = _myOntologyContext.getClassDesc(_actionClassName);
            if (_targetClassDesc != null)
            { 
                Collection<OntClassPropertyDescription> _tcds = _targetClassDesc.getAllProperties();
                for (OntClassPropertyDescription currentProperty : _tcds)
                    targets.add(new ActionPropertyDescription(currentProperty.getPropertyName(), currentProperty.getRangeName()));
            }
        } 
        catch (DirectoryFailure e) 
        {
            // Ignore
        }
        catch (UnknownConceptException e)
        {
            // Ignore
        }
        catch (ReasoningException e)
        {
            // Ignore
        }
    }
    
    /**
     *   Get the name of the class of actions.
     */
    public String getActionClassName ()
    {
        return actionClassName;
    }
        
    /**
     *   Get TargetClassDescription(s) for this action. 
     */
    public List<ActionPropertyDescription> getTargets ()
    {
        return targets;
    }
    
    public ActionPropertyDescription getProperty(String propertyName)
    {
        return (ActionPropertyDescription)_classProperties.get(propertyName);
    }
}