/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2008/09/10 14:56:39 $
 */

package kaos.core.csi.registration;

import kaos.core.service.directory.DomainDescription;
import kaos.core.service.directory.EntityHostEnvDescription;
import kaos.core.service.directory.KAoSEntityDescription;
import kaos.policy.guard.GuardInstantiationException;

import javax.agent.service.directory.AlreadyRegisteredException;
import javax.agent.service.directory.DirectoryFailure;
import javax.agent.service.directory.NotRegisteredException;

/**
 * Defines the operations to register and deregister entities in the specified domain.
 * 
 * @author KAoS Team
 * $Revision: 1.5 $
 */
public interface DomainRegistration
{
    /**
     * Register domain with the given name in the KAoS Directory Service.
     * 
     * @param domainName String containing the name of domain to register.
     * 
     * @throws DirectoryFailure if the connection to the DS cannot be established.
     * 
     * @throws AlreadyRegisteredException if the registering domain has been already registered in the DS
     */
    public void registerDomain(String domainName) throws DirectoryFailure, AlreadyRegisteredException;

    /**
     * Register the entity described by the given KAoSEntityDescription in the specified domain.
     * 
     * @param domain DomainDescription describing the domain, in which to register the given entity.
     * 
     * @param entityDesc KAoSEntityDescription describes the entity to be registered in the given domain.
     * 
     * @throws DirectoryFailure if the connection to the DS cannot be established.
     * 
     * @throws AlreadyRegisteredException if the registering entity has been already registered in the the given domain.
     */
    public KAoSEntityDescription registerEntityInDomain(DomainDescription domain, KAoSEntityDescription entityDesc) throws DirectoryFailure, AlreadyRegisteredException;

    /**
     * Deregister the entity described by the given KAoSEntityDescription from the specified domain.
     * 
     * @param domain DomainDescription describing the domain, from which to deregister the given entity.
     * 
     * @param entityDesc KAoSEntityDescription describes the entity to be deregistered from the given domain.
     * 
     * @throws DirectoryFailure if the connection to the DS cannot be established.
     * 
     * @throws NotRegisteredException if the deregistering entity has not been registered in the the given domain.
     */
    public KAoSEntityDescription deregisterEntityFromDomain(DomainDescription domain, KAoSEntityDescription entityDesc) throws DirectoryFailure, NotRegisteredException;

    /**
     * Register the entity hosting environment described by the given EntityHostEnvDescription in the KAoS Directory Service.
     * 
     * @param eheDescription EntityHostEnvDescription describes the hosting environment for agents and guards.
     * 
     * @param guarded boolean indicating the guarded/unguarded status for the environment. If the hosted environemnt is to be guarded, the implementation of this method will
     *          instantiate a guard, or check if a guard already exists and proceed with the registration via the guard.
     * @param replace boolean indicating whether to replace existing EntityHostEnvDescription.
     * 
     * @throws DirectoryFailure if the connection to the DS cannot be established.
     * @throws AlreadyRegisteredException if the registering hosting environment has been already registered in the DS (if the replace = false and there is already registration for
     *          the specified EntityHostEnvDescription).
     */
    public void registerEntityHostingEnv(EntityHostEnvDescription eheDescription, boolean guarded, boolean replace) throws DirectoryFailure, AlreadyRegisteredException,
            GuardInstantiationException;

    /**
     * Deregister the entity hosting environment described by the given EntityHostEnvDescription from the KAoS Directory Service.
     * 
     * @param hostEnvDescription EntityHostEnvDescription of the entity hosting environment to be deregistered.
     * 
     * @throws DirectoryFailure if the connection to the DS cannot be established.
     * 
     * @throws NotRegisteredException if the entity to be deregistered has not been registered in the DS.
     */
    public void deregisterEntityHostingEnv(EntityHostEnvDescription hostEnvDescription) throws DirectoryFailure, NotRegisteredException;
}
