/**
 * Last Modified by: $Author: mvignati $
 * On: $Date: 2012/01/26 19:29:58 $
 */

package kaos.core.csi.registration;

import java.util.ArrayList;
import java.util.List;

/**
 * Contains the modification type, its success/failure flag and the description and error code for the modification failure.
 *
 * @author KAoS Team
 * $Revision: 1.3 $
 */
public class ModificationTypeImpl implements ModificationType
{
    private static final long serialVersionUID = 6112619058828806738L;

    private String            _type;
    private boolean           _success;
    private String            _failureDescription;
    private int               _errorCode;
    private static List<String>       __modificationTypes;
    
    static
    {
        __modificationTypes = new ArrayList<String>();
        
        __modificationTypes.add(ModificationType.ADD);
        __modificationTypes.add(ModificationType.ADD_ELEMENT_TO_LIST);
        __modificationTypes.add(ModificationType.ADD_ELEMENTS_TO_LIST);
        __modificationTypes.add(ModificationType.REPLACE);
        __modificationTypes.add(ModificationType.DELETE);
        __modificationTypes.add(ModificationType.DELETE_ELEMENT_FROM_LIST);
        __modificationTypes.add(ModificationType.DELETE_ELEMENTS_FROM_LIST);
        __modificationTypes.add(ModificationType.ADD_ELEMENT_TO_TABLE);
        __modificationTypes.add(ModificationType.DELETE_ELEMENT_FROM_TABLE);
        __modificationTypes.add(ModificationType.DELETE_ELEMENTS_FROM_TABLE);
    }

    public static boolean isUpdateTypeValid(String updateType)
    {
        return __modificationTypes.contains(updateType);
    }

    /*
     * Constructor.
     */
    public ModificationTypeImpl()
    {
        this (null);
    }

    public ModificationTypeImpl (String modType)
    {
        _success = false;
        _errorCode = 0;
        if (modType != null) {
            setModificationType(modType);
        }
    }

    public String getModificationType()
    {
        return _type;
    }

    /*
     * Set the type of the modification.
     * 
     * @param type String containing the type of modification.
     */
    public void setModificationType(String type)
    {
        _type = type;
    }

    /*
     * Set the success/failure status of the modification.
     * 
     * @param flag boolean indicating modification success/failure; true means success.
     */
    public void setSuccess(boolean flag)
    {
        _success = flag;
    }

    /*
     * Obtain the success/failure status of the modification.
     * 
     * @return boolean indicating modification success/failure; true means success
     */
    public boolean isSuccess()
    {
        return _success;
    }

    /*
     * Return the description of modification failure.
     * 
     * @return String describing modification failure.
     */
    public String getModificationFailureDescription()
    {
        return _failureDescription;
    }

    /*
     * Store the description of modification failure.
     * 
     * @param desc String containing the description of modification failure.
     */
    public void setModificationFailureDescription(String desc)
    {
        _failureDescription = desc;
    }

    /*
     * Return the error code of the modification failure.
     * 
     * @return int describing modification failure.
     */
    public int getModificationFailureCode()
    {
        return _errorCode;
    }

    /*
     * Store the error code of the modification failure.
     * 
     * @param code int containing the error code of the modification failure.
     */
    public void setModificationFailureCode(int code)
    {
        _errorCode = code;
    }

    /*
     * A utility method to create a String representation of the ModificationResult.
     * 
     * @return String representing the ModificationResult.
     */
    public String toString()
    {
        StringBuffer buffer = new StringBuffer();
        buffer.append("Modification type: " + _type + "\n");
        buffer.append("Modification success: " + _success + "\n");
        buffer.append("Failure description: " + _failureDescription + "\n");
        buffer.append("Error code: " + _errorCode + "\n");

        return buffer.toString();
    }
}