/**
 * Last Modified by: $Author: mvignati $
 * On: $Date: 2012/01/26 19:29:58 $
 */
 
package kaos.core.csi.registration;

import kaos.core.csi.OntPropertyDescription;

import java.io.Serializable;

/**
 * Specifies operations used to update entity properties. In addition to the property name, value, description and
 * modification type, operations include also accessing modification result.
 *
 * @author KAoS Team
 * $Revision: 1.9 $
 */
public interface Property extends Serializable
{
    /** 
     * Set the property name.
     *
     * @param name      String containing the name of the property. 
     */
    public void setName(String name);
   
    /**
     * Obtain the property name.
     *
     * @return String containing the name of the property. 
     */
    public String  getName();
    
    /** 
     * Set the result of the property modification.
     *
     * @param result    ModificationResult describing the result of modifying the property.
     */
    public void setModificationType(ModificationType result);
   
    /**
     * Obtain the result of the property modification.
     *
     * @return ModificationResult describing the result of modifying the property.
     */
    public ModificationType getModificationType();
    
    /** 
     * Set the new value of the property.
     *
     * @param value      Serializable containing the new valueof the property. 
     */
    public void setValue(Serializable value);
   
    /**
     * Obtain the new value of the property.
     *
     * @return Serializable containing the new value of the property.
     */
    public Serializable  getValue();
    
    /** 
     * Set the OWL description of the property to be modified. 
     *
     * @param desc   String containing the OWL description of the property to be modified.
     */
    public void setDescription(String desc);
    
    /** 
     * Return the OWL description of the property to be modified.
     *
     * @return          String containing the OWL description of the property to be modified
     *               
     */
    public String  getDescription();
   
    /** 
     * Set the scope of the modification:
     * 0 = PUBLIC scope indicates updates to the Directory Service 
     * 1 = PRIVATE scope indicates local updates only
     *
     * @param scope       int specifying the scope of the property update.             
     */
    public void setScope(int scope);
    
    /** 
     * Obtain the scope of the modification:
     * 0 = PUBLIC scope indicates updates to the Directory Service 
     * 1 = PRIVATE scope indicates local updates only
     *
     * @return       int specifying the scope of the property update.               
     */
    public int getScope( );
   
    /** 
     * Obtain the unique sequence id of this property.
     *
     * This id is used in remote updates of this property in the DS.  If the property
     * update fails, then the Property is returned back to the requester
     * and the requester can identify the failed property and not update its local
     * collection of Properties.
     *
     * @return       String containing the unique sequence id of this Property.             
     */
    public String getSequenceId();
    
    /** 
     * Return a Serializable, encapsulating a property, which this Property represents.  
     *
     * This method is used to manipulate spatial properties, ex. Location3D, Orientation3D, etc.
     *
     * @return       Serializable containing a property.              
     */
    public Serializable getPropertyObject();
    
    /** 
     * Set the given Serializable property, which this Property represents.  The Serializable may have its own API.
     *
     * @param property       Serializable encapsulating a property.             
     */
    public void setPropertyObject(Serializable property);
    
    /** 
     * Return mapping from to the OntPropertyDescription.  
     *
     * @return       OntPropertyDescription              
     */
    public OntPropertyDescription getOntPropertyRep();
}
