/**
 * Last Modified by: $Author: mvignati $
 * On: $Date: 2012/01/26 19:30:02 $
 */

package kaos.core.csi.request;

import kaos.core.service.directory.KAoSEntityDescription;

import javax.agent.service.transport.NotLocatableException;
import javax.agent.service.transport.TransportFailure;
import java.util.HashMap;

/**
 * Defines the operations to send a request expressed by a String containing the action name
 * and, in some cases, a HashMap containing the action properties.
 * 
 * @author KAoS Team
 */
public interface HashMapRequestManager 
{
    /**
	 * This method sends a request to an entity identified by the given receiverID to perform an action described  
	 * by the given String and a HashMap containing the action properties.  Feedback from the action execution is sent to the
	 * given feedbackReceiver.
	 * 
	 * @param		feedbackReceiver	    is the FeedbackReceiver requesting the action.  
	 *							            All feedback from the action execution will be sent to it.
	 * 
	 * @param	    receiverID	            is the String that identifies the entity, which is  
	 *							            requested to perform the action.  The receiverID is the one
	 *							            that the action executor has registered with KAoS.
	 * 
     * @param       actionName		        is a String representing the name of the action as defined in 
	 *                                      the associated ontology.
	 *
	 * @param		properties	            is a HashMap of ontological Strings representing 
	 *							            the desired properties of the action and the 
	 *							            associated values.
	 *
	 * @return		String		            containing the unique id of the requested action. This id may be used
	 *                                      by the FeedbackReceiver to track the status of action execution.
	 * 
	 * @throws	    ActionFailureException  containing the reason for failure.  
	 *				                        See ontology to determine specific reasons available for 
	 *				                        a given action
	 *
	 * @throws	    NotLocatableException   when the entity reqested to perform the action cannot be located.
	 *				                        
	 * @throws	    TransportFailure        when the request cannot be sent due to transport failure.
	 * 
	 **/
    public String sendHashMapRequest(FeedbackReceiver feedbackReceiver, 
							         String receiverID, 
                                     String actionName,
								     HashMap properties)
		throws ActionFailureException, NotLocatableException, TransportFailure;
		
	/**
	 * This method sends a request to an entity identified by the given receiverID to perform an action described  
	 * by the actionName.  Feedback from the action execution is sent to the
	 * given feedbackReceiver.
	 * 
	 * @param		feedbackReceiver	    is the FeedbackReceiver requesting the action.  
	 *							            All feedback from the action execution will be sent to it.
	 * 
	 * @param	    receiverID	            is the String that identifies the entity, which is  
	 *							            requested to perform the action.  The receiverID is the one
	 *							            that the action executor has registered with KAoS.
	 * 
     * @param       actionName		        is a String representing the name of the action as defined in 
	 *                                      the associated ontology.
	 *
	 * @return		String		            containing the unique id of the requested action. This id may be used
	 *                                      by the FeedbackReceiver to track the status of action execution.
	 * 
	 * @throws	    ActionFailureException  containing the reason for failure.  
	 *				                        See ontology to determine specific reasons available for 
	 *				                        a given action
	 *
	 * @throws	    NotLocatableException   when the entity reqested to perform the action cannot be located.
	 *				                        
	 * @throws	    TransportFailure        when the request cannot be sent due to transport failure.
	 * 
	 **/
    public String sendHashMapRequest(FeedbackReceiver feedbackReceiver, 
							         String receiverID, 
                                     String actionName)
		throws ActionFailureException, NotLocatableException, TransportFailure;
		
	/**
	 * This method sends a request to an entity identified by the given receiverDesc to perform an action described  
	 * by actionName and a HashMap containing the action properties.  Feedback from the action execution is sent to the
	 * given feedbackReceiver.
	 * 
	 * @param		feedbackReceiver        is the FeedbackReceiver requesting the action.  
	 *							            All feedback from the action execution will be sent to it.
	 * 
	 * @param	    receiverDesc	        is a KAosEntityDescription of the entity, which is requested
	 *								        to perform the action.
	 * 
	 * @param       actionName		        is a String representing the name of the action as defined in 
	 *                                      the associated ontology.
	 *
	 * @param		properties	            is a HashMap of ontological Strings representing 
	 *							            the desired properties of the action and the 
	 *							            associated values.
	 * 
	 * @return		String		            containing the unique id of the requested action. This id may be used
	 *                                      by the FeedbackReceiver to track the status of the action execution.
	 * 
	 * @throws	    ActionFailureException  containing the reason for failure.  
	 *				                        See ontology to determine specific reasons available for 
	 *				                        a given action
	 *
	 * @throws	    NotLocatableException   when the entity reqested to perform the action cannot be located.
	 *
	 * @throws	    TransportFailure        when the request cannot be sent due to transport failure.
	 *				                        
	 **/
    public String sendHashMapRequest(FeedbackReceiver feedbackReceiver,
								     KAoSEntityDescription receiverDesc,
								     String actionName,
								     HashMap properties)
		throws ActionFailureException, NotLocatableException, TransportFailure;

	/**
	 * This method sends a request to an entity identified by the given receiverDesc to perform an action described  
	 * by actionName.  Feedback from the action execution is sent to the
	 * given feedbackReceiver.
	 * 
	 * @param		feedbackReceiver        is the FeedbackReceiver requesting the action.  
	 *							            All feedback from the action execution will be sent to it.
	 * 
	 * @param	    receiverDesc	        is a KAosEntityDescription of the entity, which is requested
	 *								        to perform the action.
	 * 
	 * @param       actionName		        is a String representing the name of the action as defined in 
	 *                                      the associated ontology.
	 * 
	 * @return		String		            containing the unique id of the requested action. This id may be used
	 *                                      by the FeedbackReceiver to track the status of the action execution.
	 * 
	 * @throws	    ActionFailureException  containing the reason for failure.  
	 *				                        See ontology to determine specific reasons available for 
	 *				                        a given action
	 *
	 * @throws	    NotLocatableException   when the entity reqested to perform the action cannot be located.
	 *
	 * @throws	    TransportFailure        when the request cannot be sent due to transport failure.
	 *				                        
	 **/
    public String sendHashMapRequest(FeedbackReceiver feedbackReceiver,
								     KAoSEntityDescription receiverDesc,
								     String actionName)
		throws ActionFailureException, NotLocatableException, TransportFailure;
		
	/**
	 * This method sends a request described by actionName and a HashMap containing the action properties 
	 * to perform an action by an entity identified by the receiverID.
	 * 
	 * The result is returned synchronously to the caller.
	 *
	 * @param	    receiverID	            is the String that identifies the entity, which is  
	 *							            requested to perform the action.  The receiverID is the one
	 *							            that the action executor has registered with KAoS.
	 *
     * @param       actionName		        is a String representing the name of the action as defined in 
	 *                                      the associated ontology.
	 *
	 * @param		properties	            is a HashMap of ontological Strings representing 
	 *							            the desired properties of the action and the 
	 *							            associated values.
	 * 
	 * @return		Object		            containing the final result of the action.
	 * 
	 * @throws	    ActionFailureException  containing the reason for failure.  
	 *				                        See ontology to determine specific reasons available for 
	 *				                        a given action
	 *
	 * @throws	    NotLocatableException   when the entity reqested to perform the action cannot be located.
	 *
	 * @throws	    TransportFailure        when the request cannot be sent due to transport failure.
	 
	 **/
    public Object sendHashMapRequest(String receiverID, 
                                     String actionName,
								     HashMap properties)
        throws ActionFailureException, NotLocatableException, TransportFailure;
        
	/**
	 * This method sends a request described by the given actionName 
	 * to perform  by an entity identified by the receiverID.
	 * 
	 * The result is returned synchronously to the caller.
	 *
	 * @param	    receiverID	            is the String that identifies the entity, which is  
	 *							            requested to perform the action.  The receiverID is the one
	 *							            that the action executor has registered with KAoS.
	 *
     * @param       actionName		        is a String representing the name of the action as defined in 
	 *                                      the associated ontology.
	 * 
	 * @return		Object		            containing the final result of the action.
	 * 
	 * @throws	    ActionFailureException  containing the reason for failure.  
	 *				                        See ontology to determine specific reasons available for 
	 *				                        a given action
	 *
	 * @throws	    NotLocatableException   when the entity reqested to perform the action cannot be located.
	 *
	 * @throws	    TransportFailure        when the request cannot be sent due to transport failure.
	 
	 **/
    public Object sendHashMapRequest(String receiverID, 
                                     String actionName)
        throws ActionFailureException, NotLocatableException, TransportFailure;
    
        
    /**
	 * This method sends a request described by actionName and properties to perform an action by an entity identified 
	 * by the receiverDesc.
	 * The result is returned synchronously to the caller.
	 *
	 * @param	    receiverDesc	        is the KAoSEntityDescription that identifies the entity, which is  
	 *							            requested to perform the action.  
	 *
	  * @param      actionName		        is a String representing the name of the action as defined in 
	 *                                      the associated ontology.
	 *
	 * @param		properties	            is a HashMap of ontological Strings representing 
	 *							            the desired properties of the action and the 
	 *							            associated values.
	 *
	 * @return		Object		            containing the final result of the action.
	 * 
	 * @throws	    ActionFailureException  containing the reason for failure.  
	 *				                        See ontology to determine specific reasons available for 
	 *				                        a given action
	 *
	 * @throws	    NotLocatableException   when the entity reqested to perform the action cannot be located.
	 *
	 * @throws	    TransportFailure        when the request cannot be sent due to transport failure.
	 
	 **/
    public Object sendHashMapRequest(KAoSEntityDescription receiverDesc, 
                                     String actionName,
								     HashMap properties)
        throws ActionFailureException, NotLocatableException, TransportFailure;
        
   /**
	 * This method sends a request described by actionName to perform an action by an entity identified 
	 * by the receiverDesc.
	 * The result is returned synchronously to the caller.
	 *
	 * @param	    receiverDesc	        is the KAoSEntityDescription that identifies the entity, which is  
	 *							            requested to perform the action.  
	 *
	 * @param      actionName		        is a String representing the name of the action as defined in 
	 *                                      the associated ontology.
	 *
	 * @return		Object		            containing the final result of the action.
	 * 
	 * @throws	    ActionFailureException  containing the reason for failure.  
	 *				                        See ontology to determine specific reasons available for 
	 *				                        a given action
	 *
	 * @throws	    NotLocatableException   when the entity reqested to perform the action cannot be located.
	 *
	 * @throws	    TransportFailure        when the request cannot be sent due to transport failure.
	 
	 **/
    public Object sendHashMapRequest(KAoSEntityDescription receiverDesc, 
                                     String actionName)
        throws ActionFailureException, NotLocatableException, TransportFailure;


}
