/**
 * Last Modified by: $Author: mvignati $
 * On: $Date: 2012/01/26 19:30:02 $
 */

package kaos.core.csi.request;

import kaos.core.service.ontology.request.ActionStatusCallback;

import java.util.HashMap;

/**
 * Defines the operations to process a request expressed by a name and a HashMap of properties. 
 * 
 * @author KAoS Team
 */
public interface HashMapRequestReceiver 
{
    /**
	 * This method accepts a request described by the given String containing action name and HashMap of properties.
	 * Feedback from the action execution is sent to the requestor via the given ActionStatusCallback.
	 * 
	 * @param       actionName		        is a String representing the name of the action as defined in 
	 *                                      the associated ontology.
	 * 
	 * @param		properties	            is an HashMap of ontological Strings representing 
	 *							            the desired properties of the action and the 
	 *							            associated values.
	 * 
	 * @param		statusCallback		    ActionStatusCallback to be called to pass back to the 
     *                                      action requestor information about progression of the action execution
     *                                      and the final result.
	 * 
	 * @throws	    ActionFailureException  containing the reason for failure.  
	 *				                        See ontology to determine specific reasons available for 
	 *				                        a given action
	 * 
	 **/
    public void processHashMapRequest(String actionName,
								      HashMap properties,
                                      ActionStatusCallback statusCallback)
		throws ActionFailureException;
		
	/**
	 * This method accepts a request described by the given String containing the action name.
	 * Feedback from the action execution is sent to the requestor via the given ActionStatusCallback.
	 * 
	 * @param       actionName		        is a String representing the name of the action as defined in 
	 *                                      the associated ontology.
	 * 
	 * @param		statusCallback		    ActionStatusCallback to be called to pass back to the 
     *                                      action requestor information about progression of the action execution
     *                                      and the final result.
	 * 
	 * @throws	    ActionFailureException  containing the reason for failure.  
	 *				                        See ontology to determine specific reasons available for 
	 *				                        a given action
	 * 
	 **/
    public void processHashMapRequest(String actionName,
                                      ActionStatusCallback statusCallback)
		throws ActionFailureException;
		
	/**
	 * This method accepts a request described by the given String containing action name and HashMap of properties.
	 * The result is returned synchronously.
	 * 
	 * @param       actionName		        is a String representing the name of the action as defined in 
	 *                                      the associated ontology.
	 * 
	 * @param		properties	            is an HashMap of ontological Strings representing 
	 *							            the desired properties of the action and the 
	 *							            associated values.
	 * 
	 * @return		Object		            containing the final result of the action.
	 * 
	 * @throws	    ActionFailureException  containing the reason for failure.  
	 *				                        See ontology to determine specific reasons available for 
	 *				                        a given action
	 * 
	 **/
    public Object processHashMapRequest(String actionName,
								        HashMap properties)
        throws ActionFailureException;
        
    /**
	 * This method accepts a request described by the given String containing action name.
	 * The result is returned synchronously.
	 * 
	 * @param       actionName		        is a String representing the name of the action as defined in 
	 *                                      the associated ontology.
	 * 
	 * @return		Object		            containing the final result of the action.
	 * 
	 * @throws	    ActionFailureException  containing the reason for failure.  
	 *				                        See ontology to determine specific reasons available for 
	 *				                        a given action
	 * 
	 **/
    public Object processHashMapRequest(String actionName)
        throws ActionFailureException;


}
