/**
 * Last Modified by: $Author: mvignati $
 * On: $Date: 2012/01/26 19:30:02 $
 */

package kaos.core.csi.request;

import kaos.core.service.ontology.request.ActionStatusCallback;

/**
 * Defines the operations to process a request expressed by an OWL String. 
 * 
 * @author KAoS Team
 */
public interface OWLRequestReceiver 
{
    /**
	 * This method accepts a request described by an OWL String to perform an action.
	 * Feedback from the action execution is sent to the requestor via the given ActionStatusCallback.
	 * 
	 * @param	    owlActionDescription    is an OWL String that describes the action.  
	 *							            It includes the action name and all action properties and their values.
	 * 
	 * @param		statusCallback		    ActionStatusCallback to be called to pass back to the 
     *                                      action requestor information about progression of the action execution
     *                                      and the final result.
	 * 
	 * @throws	    ActionFailureException  containing the reason for failure.  
	 *				                        See ontology to determine specific reasons available for 
	 *				                        a given action
	 * 
	 **/
    public void processOWLRequest(String owlActionDescription,
                                  ActionStatusCallback statusCallback)
		throws ActionFailureException;
		
	/**
	 * This method accepts a request described by an OWL String to perform an action.
	 * The result is returned synchronously.
	 * 
	 * @param	    owlActionDescription    is an OWL String that describes the action.  
	 *							            It includes the action name and all action properties and their values.
	 * 
	 * @return		Object		            containing the final result of the action.
	 * 
	 * @throws	    ActionFailureException  containing the reason for failure.  
	 *				                        See ontology to determine specific reasons available for 
	 *				                        a given action
	 * 
	 **/
    public Object processOWLRequest(String owlActionDescription)
        throws ActionFailureException;

}
