/**
 * Last Modified by: $Author: mvignati $
 * On: $Date: 2012/01/26 19:30:02 $
 */
package kaos.core.csi.request;

import kaos.core.csi.ActionInstanceDescription;
import kaos.core.csi.KAoSActorImpl;
import kaos.core.util.*;
import kaos.policy.builder.KAoSPolicyBuilderUtil;

import javax.agent.TransportMessage;
import javax.agent.service.transport.MessageListener;
import javax.agent.service.transport.NotLocatableException;
import javax.agent.service.transport.TransportFailure;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

/**
 *
 * Defines the operations to process an obligation request described by the given obligation ActionInstanceDescription.  
 * In addition, the obligation trigger is given as well to provide more information about the context, in which the trigger action has occurred.
 * 
 * @author KAoS Team
 */
public abstract class ObligationRequestReceiverImpl extends KAoSActorImpl implements ObligationRequestReceiver, MessageListener
{
    /**
     * Instantiates a new KAoS-aware agent using JAS services for transport, naming and agent directory.
     */
    public ObligationRequestReceiverImpl(String [] args) throws Exception
    {
    	super(args);
    }
    
    // ------------- kaos.core.csi.request.ObligationRequestReceiver interface - Begin
    
	/**
	 * @see kaos.core.csi.request.ObligationRequestReceiver
	 */
	public void processObligationRequest(ActionInstanceDescription obligationAID, ActionInstanceDescription triggerAID) throws ActionFailureException
	{
		if (obligationAID == null || triggerAID == null)
	        throw new ActionFailureException("Required argument is null\n\tobligationAID: " + obligationAID + "\n\ttriggerAID: " + triggerAID);
	     
	    String actionName = obligationAID.getActionClassName();
		if (_logger.isEnabledFor(Logger.INFO))
		    _logger.logMessage("getActionMethodNameFromAID: got long name: " + actionName, Logger.INFO);
		
		String methodName = KAoSPolicyBuilderUtil.getShortName(actionName);
		if (_logger.isEnabledFor(Logger.INFO))
		    _logger.logMessage("getActionMethodNameFromAID: got short name: " + methodName, Logger.INFO);
		
	    if (methodName == null) 
	        throw new ActionFailureException("Action name is null, unable to continue processing of:\n " + obligationAID );
	        
	    try 
	    {
	    	executeObligationAction(methodName, obligationAID, triggerAID);
	    }
	    catch (Exception e) 
	    {
	    	if (e instanceof ActionFailureException)
	    		throw (ActionFailureException) e;
		    else 
		    {
		    	e.printStackTrace();
		    	throw new ActionFailureException(e.getMessage());
		    }
	    }	    
	}

    // ---------------- kaos.core.csi.request.ObligationRequestReceiver interface - End
	
    // ---------------- javax.agent.service.transport.MessageListener - Begin

    /**
     * Implements the receiveMessage method mandated by the MessageListener interface. Sends a reply, if requested, back to the receiver.
     */
    public void receiveMessage(TransportMessage tmsg)
    {
    	if (_logger.isEnabledFor(Logger.INFO))
            _logger.logMessage("receiveMessage: received transport message:\n" + tmsg, Logger.INFO);
    
        Msg msgContent = MsgUtils.unwrapMsgContent(tmsg);

        if (msgContent instanceof MethodCallRequestMsg) 
        {
            MethodCallRequestMsg content = (MethodCallRequestMsg) msgContent;
            
            if (_logger.isEnabledFor(Logger.INFO))
                _logger.logMessage("receiveMessage: retrieved message content:  \n" + content, Logger.INFO);

            MethodCallResultMsg reply = satisfyRequest(content);

            if ( !content.isReplyRequested())
            	return;
            
            if (reply == null) 
            {
                if (_logger.isEnabledFor(Logger.INFO))
                    _logger.logMessage("receiveMessage:  reply is null and will NOT be returned to the caller! \n", Logger.INFO);
                
                return;
            }
            
            if (_logger.isEnabledFor(Logger.INFO))
                _logger.logMessage("receiveMessage: sending reply message: \n" + reply, Logger.INFO);
   
            try 
            {
                 _transportSupport.sendMessageTo(_actorDesc, tmsg, reply);
            }
            catch (TransportFailure e)
            {
                if (_logger.isEnabledFor(Logger.WARN))
                    _logger.logMessage("", e, Logger.WARN);
            }
            catch (NotLocatableException e2)
            {
                if (_logger.isEnabledFor(Logger.WARN))
                    _logger.logMessage("", e2, Logger.WARN);
            }
        }
        else
            handleMsgContent(msgContent);
    }

    
    // ------------------  javax.agent.service.transport.MessageListener - End
   
    private Object executeObligationAction(String methodName, ActionInstanceDescription obligationAID, ActionInstanceDescription triggerAID) throws Exception
    {
    	if ( methodName == null ||  obligationAID == null || triggerAID == null)  
    		throw new Exception("Required argument is null! Unable to proceed!\n\tmethodName: " 
    							+ methodName + "\n\tobligationAID: " + obligationAID + "\n\ttriggerAID: " + triggerAID);  
    	
    	Object result = null;

    	if (_logger.isEnabledFor(Logger.INFO))
    		_logger.logMessage("executeObligationAction:  action name: " + methodName, Logger.INFO);
 	
    	try 
    	{
    		List<Object> argObjectList = new ArrayList<Object>();
    			
    		argObjectList.add(obligationAID);
    		argObjectList.add(triggerAID);
		
    		Object argObjects[]= argObjectList.toArray(new Object[]{});
    		result = executeMethodWithArgs(methodName, argObjects);
    	}           		
    	catch (Exception xcp) 
    	{
    		if (_logger.isEnabledFor(Logger.WARN))
    			_logger.logMessage(".executeObligationAction: problem executing: " + methodName, xcp, Logger.WARN);
    		
    		throw xcp;	
    	}
    	
    	return result;
    }

   private Object executeMethodWithArgs(String methodName, Object [] argObjects) throws Exception
   {
	   Object result = null;
	   Method methods[] = this.getClass().getMethods();
	   boolean foundMethod = false;
	   
	   for (int i=0; i< methods.length; i++) 
	   {
		   if (methods[i].getName().equals(methodName)) 
		   {
		       // Check to see if the parameter classes match the arguments
			   Class parameters[] = methods[i].getParameterTypes();
			   boolean classesMatch = false;
			   
			   if (parameters.length == argObjects.length) 
			   {
				   classesMatch = true ;
				   
				   for (int j=0; j<parameters.length; j++) 
				   {
					   if (!parameters[j].isInstance(argObjects[j])) 
					   {
						   classesMatch = false;
						   break;
					   }
				   }
			   }
			   
			   if ( classesMatch ) 
			   {
				   foundMethod = true;
				   result = methods[i].invoke(this, argObjects);
				   
				   break;
			   }
		   }
	   }
		   
	   if ( ! foundMethod)                   
		   throw new NoSuchMethodException(methodName);
	     		
	   return result;
   }
   
   private void handleMsgContent(Msg content)
   {
	   _logger.logMessage("handleMsgContent: unable to handle msgContent:" + content, Logger.WARN);
   }
  
   /**
    * Currently, there is only 1 method in the ObligationRequestReceiver, which is being asked to satisfy, so the processing could be just 
    * the retrieve the method name, make sure that it is "processObligationRequest" then retrieve the args, make sure that they are
    * 2 instances of ActionInstanceDescription and call: "processObligationRequest(ActionInstanceDescription obligationAID, ActionInstanceDescription triggerAID)".
    * However, if the ObligationRequestReceiver evolves then it would not work and that is why reflection is used even though for now it is an overwritten. 
    */
   private MethodCallResultMsg satisfyRequest (MethodCallRequestMsg requestMsg)
   {   
       MethodCallResultMsg resultMsg = null;
       Object result = null;
       
       try 
       {         
           String methodName = requestMsg.getMethodName();
           Vector<Object> args = requestMsg.getArgs();
           
           if (args != null) 
           {
               Object argObjects[] = new Object[args.size()];
               
               for (int i=0; i<args.size(); i++)
                   argObjects[i] = args.elementAt(i);
               
               result = executeMethodWithArgs(methodName, argObjects);
               resultMsg = new MethodCallResultMsg(requestMsg.getSequenceId(), result);
           }
           else 
           {
               result = this.getClass().getMethod(methodName, (Class[])null).invoke(this, (Object[])null);
               resultMsg = new MethodCallResultMsg(requestMsg.getSequenceId(), result);
           }
       }
       catch (Exception xcp) 
       {
           if (xcp instanceof java.lang.reflect.InvocationTargetException) 
           {
               java.lang.reflect.InvocationTargetException inex = (java.lang.reflect.InvocationTargetException) xcp;
               resultMsg = new MethodCallResultMsg(requestMsg.getSequenceId(), inex.getTargetException());
           }
           else
               resultMsg = new MethodCallResultMsg(requestMsg.getSequenceId(), xcp);
           
           if (_logger.isEnabledFor(Logger.ERROR))
               _logger.logMessage("", xcp, Logger.ERROR);              
       }
       
       return resultMsg;
   }
}
