/**
 * Last Modified by: $Author: auszok $ On: $Date: 2012/03/21 14:22:35 $
 * 
 * This is a test agent simulating users login to the system and enforcing different policies on them
 */
package kaos.core.csi.usecase;

import kaos.core.csi.CSIFactory;
import kaos.core.csi.Transport;
import kaos.core.csi.TransportImpl;
import kaos.core.csi.KAoSActorImpl;

import kaos.core.csi.ActionInstanceDescription;
import kaos.core.csi.ActionInstanceDescriptionImpl;
import kaos.core.csi.OntPropertyDescription;
import kaos.core.csi.OntPropertyDescriptionImpl;

import kaos.core.csi.extension.history.EventHistoryComponent;
import kaos.core.csi.policy.KAoSSecurityException;
import kaos.core.csi.policy.PolicyChecking;
import kaos.core.csi.policy.ObligationViolationException;
import kaos.core.csi.policy.ViolatedObligationDescription;
import kaos.core.csi.query.QueryState;

import kaos.ontology.vocabulary.ActionStatusConcepts;
import kaos.ontology.vocabulary.ActorConcepts;
import kaos.ontology.vocabulary.computing.ComputingActionConcepts;

import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.Set;

import java.io.InputStreamReader;
import java.io.LineNumberReader;

/**
 * $Revision: 1.17 $
 */
public class LoginSimulationAgent extends KAoSActorImpl
{
    public LoginSimulationAgent(String guid, String name, List<String> domainNames, Transport transport) throws Exception
    {
        // Nothing special here
        super(guid, name, domainNames, transport); // This will register an agent in KAoS
        CSIFactory.setCurrentTransport(transport.getName());
    }

    public static void main(String[] args) throws Exception
    {
        // -- Begin -- Login Agent Initialization

        LoginSimulationAgent logginAgent;
        String logginAgentName = "LoginSimulationAgent";
        String transport = CSIFactory.TCP_TRANSPORT; 
        List<String> domainNames = new Vector<String>();
        domainNames.add("LoginSystemDomain");

        Transport trans = new TransportImpl();
        trans.setName(transport);

        logginAgent = new LoginSimulationAgent(logginAgentName, logginAgentName, domainNames, trans);

        // Add actor to an ontology class
        logginAgent.addOntologicalType(ActorConcepts.SoftwareAgent(), null);

        logginAgent.registerWithKAoS();

        // -- End of Login Agent Initialization

        int noOfFailureForGivenUsername = 0;

        // -- Obtain the list of known humans

        QueryState queryState = CSIFactory.getQueryState();
        PolicyChecking policyChecking = CSIFactory.getPolicyChecking();
        EventHistoryComponent historyMonitor = CSIFactory.getHistoryMonitor();

        Set<String> humans = queryState.getInstancesOf(ActorConcepts.Human());

        Map<String, String> userNames = new Hashtable<String, String>(); // keeps mapping from short name (username) to the full (url) name of the human
        Map<String, String> logins = new Hashtable<String, String>(); // keeps mapping from short name (username) to password
        Map<String, Integer> nameNoOfFailure = new Hashtable<String, Integer>(); // maps username to the current number of logging failure for a given username

        if (humans != null)
        {
            System.out.println("\nAvailable logins: [name, password]\n");

            int i = 0; // needed to build unique password for each username
            for (String currentHuman : humans)
            {
                String currentUsername = currentHuman.substring((currentHuman.indexOf("#") + 1));
                System.out.println("\n " + currentUsername + " pwd" + i);

                // Put information about this human into the mapping tables
                userNames.put(currentUsername, currentHuman);
                logins.put(currentUsername, "pwd" + i);
                nameNoOfFailure.put(currentUsername, new Integer(noOfFailureForGivenUsername));
                i++;
            }
        }
        else
        {
            System.out.println("\nNo humans found\n");
            System.exit(0);
        }
        
        // --- End obtain the list of known humans

        boolean loggedIn = false; // Status of the login agent - indicates if somebody is currently logged in

        InputStreamReader console = new InputStreamReader(System.in);
        LineNumberReader lnr = new LineNumberReader(console);

        int logginAIDno = 0; // Needed to build unique id for each action instance descriptions

        while (true) // Wait for a user input
        {
            String providedUsername;
            String providedPassword;

            if (loggedIn)
            {
                System.out.println("\nPlease enter 'logout' to log out.\n ");
                providedUsername = lnr.readLine();

                if ( ! providedUsername.equals("logout"))
                    continue;

                loggedIn = false;
            }

            // Ask for username
            System.out.println("\nEnter login name: \n");
            System.out.flush();
            providedUsername = lnr.readLine();
            providedUsername.trim();

            // Find if the provided user name is known
            String currentUserPassword = (String)logins.get(providedUsername);
            if (currentUserPassword == null)
            {
                System.out.println("\nGiven username is not known!");
                continue;
            }

            // Ask for password
            System.out.println("\nEnter password: \n");
            System.out.flush();
            providedPassword = lnr.readLine();
            providedPassword.trim();

            // --------------- Build Action Instance Description for the LoginAction

            ActionInstanceDescription logginAID = new ActionInstanceDescriptionImpl("logginAID" + logginAIDno, ComputingActionConcepts.LoginAction(), userNames.get(providedUsername));
            logginAIDno++; // increase the counter of constructed AIDs

            OntPropertyDescription hasUsernameProperty = new OntPropertyDescriptionImpl(ComputingActionConcepts.hasUsername());
            hasUsernameProperty.addValue(userNames.get(providedUsername));
            logginAID.addProperty(hasUsernameProperty);

            OntPropertyDescription hasPasswordProperty = new OntPropertyDescriptionImpl(ComputingActionConcepts.hasPassword());
            hasPasswordProperty.addValue(providedPassword);
            logginAID.addProperty(hasPasswordProperty);

            try
            {
                // Check if the login attempt action (without checking if password is correct yet) is allowed and if there are some obligation associated with it
                policyChecking.findPolicyDecision(logginAID, null);
            }
            catch (KAoSSecurityException e)
            {
                // Login not allowed
                System.out.println("\nPolicy forbids this login action!");
                continue;
            }
            catch (ObligationViolationException ove)
            {
                // Some obligation were violated by this login action
                if (ove.getViolatedObligations() != null)
                {
                    List<ViolatedObligationDescription> violatedobligations = ove.getViolatedObligations();
                    for (ViolatedObligationDescription violationDesc : violatedobligations)
                    {
                        System.out.println("\nViolated obligation:\n\t" + violationDesc);
                        System.out.println("\nReason for violation:\n\t" + violationDesc.getReasonForViolation());
                    }
                }
                continue;

            }

            // Check if the password is correct
            if (providedPassword.equals(currentUserPassword))
            {
                System.out.println("\nLogged in to the system");
                /*
                // Add the status information to the current Action Instance Description of the Logging Action
                OntPropertyDescriptionImpl hasStatusProperty = new OntPropertyDescriptionImpl(ActionStatusConcepts.hasStatus());
                hasStatusProperty.addValue(ActionStatusConcepts.DefaultCompletedActionStatus());
                logginAID.addProperty(hasStatusProperty);

                // Add the information about the failed login action to the history monitor
                historyMonitor.logEvent(logginAID);
                */
                
                loggedIn = true; 
            }
            else
            {
                System.out.println("\nIncorrect password - login failed!");

                /*
                // Add the status information to the current Action Instance Description of the Logging Action
                OntPropertyDescriptionImpl hasStatusProperty = new OntPropertyDescriptionImpl(ActionStatusConcepts.hasStatus());
                hasStatusProperty.addValue(ActionStatusConcepts.DefaultFailureActionStatus);
                logginAID.addProperty(hasStatusProperty);

                // Add the information about the failed login action to the history monitor
                historyMonitor.logEvent(logginAID);
                */
                
                try
                {
                    // If the password is not correct then check if there are policies controlling failed login
                    policyChecking.getObligationsForTriggerCondition(logginAID, null);
                }
                catch (ObligationViolationException ove)
                {
                    if (ove.getViolatedObligations() != null)
                    {
                        List<ViolatedObligationDescription> violatedobligations = ove.getViolatedObligations();
                        for (ViolatedObligationDescription violationDesc : violatedobligations)
                        {
                            System.out.println("\nViolated obligation:\n\t" + violationDesc);
                            System.out.println("\nReason for violation:\n\t" + violationDesc.getReasonForViolation());
                        }
                    }
                }
                
                int newNoOfFailure = ((Integer)nameNoOfFailure.get(providedUsername)).intValue() + 1;
                System.out.println("\nNumber of failed login attempts so far for this username: " + newNoOfFailure);

                // Increase the counter of failure for a given user
                nameNoOfFailure.put(providedUsername, new Integer(newNoOfFailure++));
            }
            
            // Verify history log
            System.out.println("\n----------- So far history has:\n");
            for(ActionInstanceDescription currentHistoryEvent: historyMonitor.getLogsforActionType(ComputingActionConcepts.LoginAction()))
                System.out.println("\n-------------------------\n" + currentHistoryEvent.toString());
            System.out.println("\n----------- End of History List \n");
        }
    }
}