/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package kaos.core.csi.usecase;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;
import javax.agent.service.directory.AlreadyRegisteredException;
import javax.agent.service.directory.DirectoryFailure;
import kaos.core.csi.CSIFactory;
import kaos.core.csi.policy.PolicyChecking;
import kaos.core.csi.registration.Registration;
import kaos.core.service.directory.KAoSAgentDescription;

/**
 *
 * @author jlott
 */
public class SimpleAgent {
    
    private KAoSAgentDescription _agentDescription;
    private PolicyChecking _policyChecking;

    public void initialize(String name, String transportName, Vector domainNames, boolean registerDomains)
    throws Exception
    {
    	// Set the message transport (e.g. tcp) and get a kaos registration object
        Registration registration = null;
        try {
            CSIFactory.setCurrentTransport(transportName);
            registration = CSIFactory.getRegistration();
        }
        catch(Exception e){
            e.printStackTrace();
            System.out.println ("  **** Unable to obtain CSI services: Registration. Exiting...");
            return;
        }

        // register the domains to which the agent belongs before registering the agent
        // it's ok if the domain is already registered
        if (registerDomains) {
            Enumeration domainsEnum = domainNames.elements();
            while ( domainsEnum.hasMoreElements() ) {
                String domainName  = (String) domainsEnum.nextElement();
                System.out.println("\n     <>  Will register domain: " + domainName + "\n");

                try {
                    registration.registerDomain(domainName);
                }
                catch(DirectoryFailure de) {
                    System.out.println("\n   Due to Directory Failure unable to register domain: " + domainName);
                    de.printStackTrace();
                    return;
                }
                catch(AlreadyRegisteredException ae) {
                    System.out.println("    The following domain has already been registered: " + domainName);
                }
            }
        }
        
        // Ask KAoS registration to create a KAoSAgentDescription for the sender.
        _agentDescription = registration.createKAoSAgentDescription(name,       // use nickname as guid
        															name,       // nickname
                                                                    domainNames,  // domains to register in
                                                                    transportName,
                                                                    null);      // no extra transport info necessary

        // Bind this agent to the KAoS transport (necessary to use kaos messaging)
        //_transportSupport = CSIFactory.getTransportSupport();
        //_transportSupport.bindMsgListenerToTransport(this, name, _agentDescription);

        registration.registerEntity(_agentDescription,
                                    true,       // guarded
                                    false);     // replace registration

        System.out.println("Registered agent " + name);
        // Obtain a reference to the KAoS registration query service, used later by sendMessageTo() to lookup the receiver agent
        //_query = CSIFactory.getQueryRegistration();
        // Obtain a reference to the KAoS policy checking service, used later by authorizeCommunicationAction() to check policies
        _policyChecking = CSIFactory.getPolicyChecking();
    }

   public static void main(String [] args)
      throws Exception
    {
        String sender = null;
        String transport = null;
        String receiver = null;
        Vector domainNames = null;
        boolean registerDomains = true;
        if (args.length > 0) {
	        int i = 0;
	        while (i < args.length) {
	            if (args[i].equalsIgnoreCase ("-name")) {
	                if ((i+1) < args.length) {
    	                sender = args[i+1];
    	                i++;
    	            }
    	            else {
    	                System.out.println ("Must specify Entity Name with -sender");
    	                System.exit(0);
    	            }
	            }
	            else if (args[i].equalsIgnoreCase ("-transport")) {
	                if ((i+1) < args.length) {
    	                transport = args[i+1];
    	                i++;
    	            }
    	            else {
    	                System.out.println ("Must specify  transport with -transport.  Valid transports: corba, grid, tcp.");
    	                System.exit(0);
    	            }
	            }
	            else if (args[i].equalsIgnoreCase ("-domains")) {
                    if ((i+1) < args.length) {
                        String domainsStr = args[i+1];
                        domainNames = new Vector();
                        StringTokenizer tokenizer = new StringTokenizer(domainsStr, ",");
                        while (tokenizer.hasMoreTokens()) {
                            domainNames.add(tokenizer.nextToken());
                        }
                        i++;
                    }
                }
                else if (args[i].equalsIgnoreCase("-registerDomains")) {
                    if ((i+1)<args.length) {
                        registerDomains = Boolean.parseBoolean(args[i+1]);
                        i++;
                    }
                }
                i++;
	        }// while
	    }

        if ( sender == null || sender.trim().length() == 0) {
            System.out.println ("\n     *** Must specify agent name with -name\n");
            return;
        }

        if ( transport == null) {
            System.out.println ("\n     *** Must specify  transport with -transport.  Valid transports: corba, grid, tcp.");
            return;
        }

        if ( domainNames == null ) {
            domainNames = new Vector();
            domainNames.add("SimpleAgentDomain");
            System.out.println (" <>    Warning: No Domain Names specified with -domains, using default: " + domainNames);
        }


        System.out.println (" <>  Agent name:                 " + sender);
	    System.out.println (" <>  Requested transport:        " + transport);
	    System.out.println (" <>  Requested domains:          " + domainNames);
        System.out.println (" <>  Register domains:           " + registerDomains);


        SimpleAgent usecase = new SimpleAgent();

        try {
            usecase.initialize(sender, transport, domainNames,registerDomains);
        }
        catch(Exception e) {
            e.printStackTrace();
            return;
        }

        // do something here
        
        synchronized ( Thread.currentThread() ) {
         Thread.currentThread().wait();
        }
    }        
}
