package kaos.core.csi.usecase;
/**
 * Last Modified by: $Author$
 * On: $Date$
 */

/**
 * This is a test agent simmulating users loggin to the system and enforcing different policies on them
 */

import kaos.core.csi.CSIFactory;
import kaos.core.csi.Transport;
import kaos.core.csi.TransportImpl;
import kaos.core.csi.KAoSActorImpl;

import kaos.core.csi.ActionInstanceDescription;
import kaos.core.csi.ActionInstanceDescriptionImpl;
import kaos.core.csi.OntPropertyDescription;
import kaos.core.csi.OntPropertyDescriptionImpl;

import kaos.core.csi.extension.history.logging.HistoryMonitor;
import kaos.core.csi.policy.KAoSSecurityException;
import kaos.core.csi.policy.PolicyChecking;
import kaos.core.csi.policy.ObligationViolationException;
import kaos.core.csi.policy.ViolatedObligationDescription;
import kaos.core.csi.query.QueryState;

import kaos.ontology.vocabulary.ActionStatusConcepts;
import kaos.ontology.vocabulary.ActorConcepts;
import kaos.ontology.vocabulary.computing.ComputingActionConcepts;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.util.Set;

import java.io.InputStreamReader;
import java.io.LineNumberReader;

/**
 * $Revision$
 */
public class TestAgent extends KAoSActorImpl
{
    public TestAgent(String guid, String name, Vector domainNames, Transport transport) throws Exception
    {
        // Nothing special here
        super(guid, name, domainNames, transport);  // This will register an agent in KAoS
        CSIFactory.setCurrentTransport(transport.getName());
    }

    public static void main(String [] args) throws Exception
    {
        // -- Begin -- Login Agent Initilization

        TestAgent logginAgent;
        String logginAgentName = "TestAgent"; // defult agent name
        String transport = CSIFactory.TCP_TRANSPORT;     // defulat communitation protocol
        Vector domainNames = new Vector();
        domainNames.add("http://ontology.ihmc.us/DFRARoles.owl#Scout");

        Transport trans = new TransportImpl();
        trans.setName(transport);

        logginAgent = new TestAgent(logginAgentName,     //guid
                                               logginAgentName,     //nickname
                                               domainNames,         //domain names
                                               trans);              //agent transport

        // Add actor to an ontology class
        logginAgent.addOntologicalType(ActorConcepts.SoftwareAgent(), null);
        while (true)
        {
            try
            {
                logginAgent.runTests(logginAgentName);
            }
            catch(Exception e)
            {
                e.printStackTrace();
            }
        }
    }

    public void runTests(String agentName) throws Exception
    {
        try
        {
            registerWithKAoS();
        }
        catch(Exception re)
        {
            re.printStackTrace();
        }

            // --------------- Build Action Instance Description for the LoginAction
        int logginAIDno = 0;
        ActionInstanceDescription logginAID =
            new ActionInstanceDescriptionImpl("logginAID" + logginAIDno,
                                              ComputingActionConcepts.LoginAction(),
                                              agentName);
        logginAIDno++; // increase the counter of constructed AIDs

        OntPropertyDescription hasUsernameProperty = new OntPropertyDescriptionImpl(ComputingActionConcepts.hasUsername());
        hasUsernameProperty.addValue("urn:KAoS#Larry_Bunch");
        logginAID.addProperty(hasUsernameProperty);

        OntPropertyDescription hasPasswordProperty = new OntPropertyDescriptionImpl(ComputingActionConcepts.hasPassword());
        hasPasswordProperty.addValue("password");
        logginAID.addProperty(hasPasswordProperty);

        PolicyChecking policyChecking = CSIFactory.getPolicyChecking();
        try
        {
            policyChecking.findPolicyDecision(logginAID, null);
        }
        catch (KAoSSecurityException e)
        {
            System.out.println("\n Policy forbids this login action!");
        }
        catch(ObligationViolationException ove){
            if (  ove.getViolatedObligations() != null ){
                List violatedobligations = ove.getViolatedObligations();
                for (Iterator it = violatedobligations.iterator(); it.hasNext(); )
                {
                    ViolatedObligationDescription violationDesc = (ViolatedObligationDescription)it.next();
                    System.out.println("\n\t****    Violated obligation:\n\t" + violationDesc);
                    System.out.println("\n\t****    Reason for violation:\n\t" + violationDesc.getReasonForViolation());
                }
            }
        }

        this.deregisterFromKAoS();
        System.out.println("Deregistered");
     }
}