package kaos.core.csi.usecase.infofed;

import kaos.core.csi.*;
import kaos.core.csi.registration.Property;
import kaos.core.csi.registration.PropertyImpl;

import kaos.core.csi.usecase.infofed.vocabulary.InfoFedActorConcepts;
import kaos.spatial.vocabulary.PhysicalAddressConcepts;

import java.util.Vector;

/**
 * User: mbreedy
 * Date: Oct 28, 2009
 * Time: 1:59:08 PM
 */

public class InfofedAgent extends KAoSActorImpl
{
    public InfofedAgent (String guid, String name, Vector domainNames, Transport transport)
           throws Exception
    {
        super (guid, name, domainNames, transport);
        System.out.println ("*InfofedAgent is created*");
        CSIFactory.setCurrentTransport (transport.getName());
    }

    public static InfofedAgent createAgent()
            throws Exception
    {
        String infofedAgentName = AGENT_NAME;
        String transport = CSIFactory.TCP_TRANSPORT;
        Vector domainNames = new Vector();
        domainNames.add ("Infofed");            // new domain for the agent

        Transport trans = new TransportImpl();
        trans.setName (transport);
        InfofedAgent infoAgent = new InfofedAgent (infofedAgentName, infofedAgentName, domainNames, trans);
        infoAgent.addOntologicalType (InfoFedActorConcepts.Federate(), null);

        infoAgent.registerWithKAoS();

        //Add property to agent description
        Property hasMissionId = new PropertyImpl();
        hasMissionId.setName (InfoFedActorConcepts.hasMissionId());
        hasMissionId.setValue ("25");
        infoAgent.addProperty (hasMissionId);

        Property hasLocalFederatPriority = new PropertyImpl();
        hasLocalFederatPriority.setName (InfoFedActorConcepts.hasLocalFederatePriority);
        hasLocalFederatPriority.setValue ("1");
        infoAgent.addProperty (hasLocalFederatPriority);

        //hasGeoArea
        Property hasGeoArea = new PropertyImpl();
        hasGeoArea.setName (InfoFedActorConcepts.hasGeoArea);
        hasGeoArea.setValue ("location51");
        infoAgent.addProperty (hasGeoArea);

        OntInstanceDescription locationAddress = new OntInstanceDescriptionImpl(PhysicalAddressConcepts.XYZCoordinateAddress(),
                                                                                "location51");
        OntPropertyDescription xCoordinateProperty = new
        OntPropertyDescriptionImpl(PhysicalAddressConcepts.XCoordinate());
        xCoordinateProperty.addValue ((new Double (1.0)).toString());
        locationAddress.addProperty (xCoordinateProperty);

        OntPropertyDescription yCoordinateProperty = new
        OntPropertyDescriptionImpl(PhysicalAddressConcepts.YCoordinate());
        xCoordinateProperty.addValue ((new Double (2.0)).toString());
        locationAddress.addProperty (yCoordinateProperty);

        hasGeoArea.setPropertyObject ("locationAddress");

        return infoAgent;
    }

    public static void main (String [] args) throws Exception
    {
        InfofedAgent infoAgent = createAgent();

        synchronized (Thread.currentThread())
        {
            Thread.currentThread().wait();

        }
    }

    //Variables
    public static String AGENT_NAME = "InfofedAgent";
}
