/**
 * Last Modified by: $Author: mvignati $
 * On: $Date: 2012/01/26 19:29:57 $
 */

package kaos.core.csi.util.tcpSupport;

import kaos.core.util.Logger;

import java.io.BufferedReader;
import java.net.Socket;

/**
 * This class is a communication handler for the client. It sends client messages to the server and listens for  messages
 * from the server to the client.
 */
public class ClientCommHandler 
     extends CommHandler  
{
    protected ResponseReceiver  _responseReceiver;
    
	public ClientCommHandler(Socket socket, ResponseReceiver receiver)
	throws Exception
	{
        super(socket);
        if ( receiver == null ) throw new Exception("\n\n    ClientCommHandler.constructor: ResponseReceiver is null!!!! Terminating..." );
	    
		_responseReceiver = receiver;
	}
		
	/**
	 */
	public synchronized void sendString (String msg)
    {
        if (_logger.isEnabledFor(Logger.DEBUG)){
			_logger.logMessage("sendString: sending a client message to server:\n " + msg, Logger.DEBUG);
		}
		
        if (_os != null) {
             _os.println(msg);
		}
		else {
		    _logger.logMessage("sendString: Communication to the server disabled! Unable to send the following msg to the server:\n " + msg, Logger.WARN);
		}
    }
    
    /**
     */
    protected void processBuffer(String line, BufferedReader bufferedReader)
	{
	    // System.out.println("    Read new input line: " + line);
	    if   ( line.length()== 10 ) {
		    if ( ( line.equals("ENDED-TEST")) ) { 
			    _responseReceiver.receiveResponse(line);
		    }
		    else if ( ( line.equals("S-FEEDBACK")) ) { 
			    String feedbackContent = getFeedbackContent(bufferedReader);
			    _responseReceiver.receiveResponse(feedbackContent);
		    }
        }// end if   ( line.length()== 10 ) {
	}
	
	/*
     **/
    private String getFeedbackContent(BufferedReader bufferedReader) 
    {
        String endOfFile = "E-FEEDBACK";
		StringBuffer buffer = new StringBuffer();
		String next = null;
		boolean end = false;
	    while ( ! end ) {
	        if ( ( next = readLine(bufferedReader)) != null ){
	            if ( next.equals(endOfFile) ){
	                end = true;
	            }
	            else{
	                buffer.append(next + "\n");
	            }
	        }
	        else {
	            end = true; 
	        }
	       
	    }// end while
	    if (_logger.isEnabledFor(Logger.DEBUG)){
			_logger.logMessage("getFeedbackContent: received feedback from server:\n " + buffer.toString(), Logger.DEBUG);
		}
	    return buffer.toString();
    }
      
}// end ClientCommHandler