/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2012/04/27 21:41:13 $
 */

package kaos.core.tutorial;

import kaos.core.csi.*;
import kaos.core.csi.extension.history.EventHistoryComponent;
import kaos.core.csi.policy.KAoSSecurityException;
import kaos.core.csi.policy.ObligationViolationException;
import kaos.core.csi.policy.PolicyChecking;
import kaos.core.csi.policy.ViolatedObligationDescription;
import kaos.core.csi.query.QueryState;
import kaos.ontology.vocabulary.ActionStatusConcepts;
import kaos.ontology.vocabulary.ActorConcepts;
import kaos.ontology.vocabulary.computing.ComputingActionConcepts;

import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.util.*;

public class LoginAgent
    extends KAoSActorImpl
{

    // data structures used to store info
    Hashtable userNames = new Hashtable(); // keeps mapping from short name (username) to the full (url) name of the human
    Hashtable logins = new Hashtable(); // keeps mapping from short name (username) to password
    Hashtable nameNoOfFailure = new Hashtable(); // maps username to the current number of logging failure for a given username

    public LoginAgent(String name, List<String> domainNames) throws Exception
    {
        super(name, domainNames);
        // add ontological type
        addOntologicalType(ActorConcepts.SoftwareAgent(), null);
        // register
        registerWithKAoS();
        // display humans
        displayUsersList();
    }

    public LoginAgent(String[] args) throws Exception
    {
       super(args);
    }

    public void displayUsersList() throws Exception
    {
        int noOfFailureForGivenUsername = 0;

        QueryState queryState = CSIFactory.getQueryState();

        Set humans = queryState.getInstancesOf(ActorConcepts.Human());

        if (humans != null)
        {
            System.out.println("\n Available logins: [name, password]\n");

            int i = 0; // needed to build unique password for each username
            for (Iterator it = humans.iterator(); it.hasNext(); )
            {
                String currentHuman = (String) it.next();
                String currentUsername = currentHuman.substring( (currentHuman.indexOf("#") + 1));
                System.out.println("\n " + currentUsername + " pwd" + i);

                // put information about this human into the mapping tables
                userNames.put(currentUsername, currentHuman);
                logins.put(currentUsername, "pwd" + i);
                nameNoOfFailure.put(currentUsername, new Integer(noOfFailureForGivenUsername));
                i++;
            }
        }
        else
        {
            System.out.println("\n No humans found");
        }
    }

    public void attemptLogin(String username, String password) throws Exception
    {
        PolicyChecking policyChecking = CSIFactory.getPolicyChecking();
        EventHistoryComponent historyMonitor = CSIFactory.getHistoryMonitor();

        boolean loggedIn = false; // Status of the login agent - indicates if somebody is currently logged in

        InputStreamReader console = new InputStreamReader(System.in);
        LineNumberReader lnr = new LineNumberReader(console);

        int logginAIDno = 0; // needed to build unique id for each action instance descriptions


            System.out.println("***************************************************\n" +
                               "You entered: " + username + "   " + password);

            // Find if the provided user name is known
            String currentUserPassword = (String) logins.get(username);
            if (currentUserPassword == null)
            {
                System.out.println("\n Given username is not known!");
                System.out.println(logins);
                return;
            }

            // --------------- Build Action Instance Description for the LoginAction

            ActionInstanceDescription logginAID =
                new ActionInstanceDescriptionImpl("logginAID" + logginAIDno,
                                                  ComputingActionConcepts.LoginAction(),
                                                  _actorDesc.getAgentNickname());
            logginAIDno++; // increase the counter of constructed AIDs

            OntPropertyDescription hasUsernameProperty = new OntPropertyDescriptionImpl(ComputingActionConcepts.
                                                                                        hasUsername());
            hasUsernameProperty.addValue( (String) userNames.get(username));
            logginAID.addProperty(hasUsernameProperty);

            OntPropertyDescription hasPasswordProperty = new OntPropertyDescriptionImpl(ComputingActionConcepts.
                                                                                        hasPassword());
            hasPasswordProperty.addValue(password);
            logginAID.addProperty(hasPasswordProperty);

            try
            {
                policyChecking.findPolicyDecision(logginAID, null);
            }
            catch (KAoSSecurityException e)
            {
                System.out.println("\n Policy forbids this login action!");
                return;
            }
            catch (ObligationViolationException ove)
            {
                if (ove.getViolatedObligations() != null)
                {
                    List violatedobligations = ove.getViolatedObligations();
                    for (Iterator it = violatedobligations.iterator(); it.hasNext(); )
                    {
                        ViolatedObligationDescription violationDesc = (ViolatedObligationDescription) it.next();
                        System.out.println("\n\t****	Violated obligation:\n\t" + violationDesc);
                        System.out.println("\n\t****	Reason for violation:\n\t" + violationDesc.getReasonForViolation());
                    }
                }
                return;

            }

            if (password.equals(currentUserPassword))
            {
                System.out.println("\n Logged in to the system");
                loggedIn = true;
            }
            else
            {
                System.out.println("\n Incorrect password! login failed!");

                // Add the status information to the current Action Instance Description of the Logging Action
                OntPropertyDescriptionImpl hasStatusProperty = new OntPropertyDescriptionImpl(ActionStatusConcepts.
                                                                                              hasStatus());
                hasStatusProperty.addValue(ActionStatusConcepts.DefaultFailureActionStatus);
                logginAID.addProperty(hasStatusProperty);

                // TODO: move this call to CSI Interface when it will be available in CSI
                System.out.println("logging to history:\n" + logginAID);
                historyMonitor.logEvent(logginAID);

                //verify history log
                System.out.println("So far history has:\n");
                List<ActionInstanceDescription> history = historyMonitor.getAllLogs();
                for(Object obj: history)
                {
                    System.out.println(obj.toString());
                }

//                try
//                {
//                    System.out.println("why are we chacking again:\n" + logginAID);
//                    policyChecking.findPolicyDecision(logginAID, null);
//                }
//                catch (ObligationViolationException ove)
//                {
//                    if (ove.getViolatedObligations() != null)
//                    {
//                        List violatedobligations = ove.getViolatedObligations();
//                        for (Iterator it = violatedobligations.iterator(); it.hasNext(); )
//                        {
//                            ViolatedObligationDescription violationDesc = (ViolatedObligationDescription) it.next();
//                            System.out.println("\n\t****	Violated obligation:\n\t" + violationDesc);
//                            System.out.println("\n\t****	Reason for violation:\n\t" +
//                                               violationDesc.getReasonForViolation());
//                        }
//                    }
//
//                }
                int newNoOfFailure = ( (Integer) nameNoOfFailure.get(username)).intValue() + 1;
                System.out.println("\n # Failed LOGIN attempts so far for this username: " + newNoOfFailure);

                // Increase the counter of failure for a given user
                nameNoOfFailure.put(username, new Integer(newNoOfFailure++));
            }
//        }
    }

    public static void main(String[] args) throws Exception
    {
        // create name and domain
        String logginAgentName = "LoginSimulationAgent"; // default agent name
        List<String> domainNames = new Vector<String>();
        domainNames.add("LoginSystemDomain"); // default domain for the agent
        // create agent
        LoginAgent logginAgent = new LoginAgent(args);
    }
}
