package kaos.core.tutorial;

import kaos.core.csi.*;
import kaos.core.csi.extension.history.EventHistoryComponent;
import kaos.core.csi.extension.state.StateManager;
import kaos.core.csi.extension.state.StateManagerImpl;
import kaos.core.csi.policy.ObligationActionDescription;
import kaos.core.csi.policy.PolicyChecking;
import kaos.core.util.NamespaceValidator;
import kaos.ontology.vocabulary.ActionConcepts;
import kaos.ontology.vocabulary.ActionStatusConcepts;
import kaos.ontology.vocabulary.ActorConcepts;
import kaos.ontology.vocabulary.computing.ComputingActionConcepts;
import kaos.policy.builder.KAoSPolicyBuilderUtil;

import java.io.Serializable;
import java.lang.reflect.Method;
import java.net.URISyntaxException;
import java.util.*;

//import kaos.robots.ontology.TeleoperationConcepts;

public class SimpleAgent extends KAoSActorImpl
{
   /**
    * _curStatePlace: the current place of this agent as reflected by
    * the current value of the hasPlace attribute of the local KAoSGuard's State for this actor
    */
   private String _curStatePlace = "";

   public SimpleAgent(String[] args) throws Exception
   {
      super(args);
      initialize();
   }

   public SimpleAgent(String name, Vector domainNames) throws Exception
   {
      super(name, domainNames);
      initialize();
   }

   private void initialize() throws Exception
   {
      // modify default agent description
      setEntityOntologicalType();
      List<AgentCapability> capabilites = getAgentCapabilities();
      _actorDesc.setCapabilities(capabilites);

      // register agent
      super.registerWithKAoS();
   }

   /**
    *  define the type of entity the agent is
    */
   protected void setEntityOntologicalType()
   {
      addOntologicalType(ActorConcepts.Actor());
   }

   protected List getActorProperties()
   {
      return null;
   }

   public void registerWithKAoS() throws Exception
   {
      List properties = getActorProperties();
      if (properties == null)
      {
         super.registerWithKAoS();
      }
      else
      {
         super.registerWithKAoS(getActorProperties());
      }
   }

   /**
    * define properties that describe what capabilities the agent has
    */
   protected List<AgentCapability> getAgentCapabilities()
   {
      return new ArrayList<AgentCapability>();
   }

   public void receiveMessage(Serializable messageContent, KAoSActor sender)
   {
      System.out.println(_actorDesc.getAgentNickname() + " received message from " + sender.getName() + ": " + messageContent);
   }


   public void CommunicationAction(HashMap properties) throws Exception
   {
      String actionProperty = ((OntPropertyDescription) properties.get(ActionConcepts.carriesMessage())).getFirstValue();
      OntInstanceDescription actionOntInstDesc =
         (OntInstanceDescription) ((OntPropertyDescription) properties.get(ActionConcepts.carriesMessage())).getValueDescForValue(actionProperty);
      String action = KAoSPolicyBuilderUtil.getShortName(actionOntInstDesc.getFirstType());
      Object status = KAoSPolicyBuilderUtil.getShortName(actionOntInstDesc.getProperty(ActionStatusConcepts.hasStatus()).getFirstValue());
      String message = action + " " + status;
      String receiverName = ((OntPropertyDescription) properties.get(ActionConcepts.hasDestination())).getFirstValue();
      CommunicationAction(message, receiverName);
   }

   public void CommunicationAction(String message, String receiverName) throws Exception
   {
      // build aid
      HashMap properties = new HashMap();
      properties.put(ActionConcepts.hasDestination(), NamespaceValidator.validateURI(receiverName));

//    properties.put(ActionConcepts.carriesMessage(), message);
      OntPropertyDescriptionImpl opd = new OntPropertyDescriptionImpl(ActionConcepts.hasDataResource());
      opd.setValue(message);
      properties.put(ActionConcepts.hasDataResource(), opd);
      ActionInstanceDescription aid = buildActionInstanceDescription(ActionConcepts.CommunicationAction(), properties);

      // perform enforcement
      enforcePolicies(aid);

      // log the communication action to the History Monitor
      EventHistoryComponent historyMonitor = CSIFactory.getHistoryMonitor();
      historyMonitor.logEvent(aid);
      System.out.println("recorded communication action to History Monitor from: " + _actorDesc.getAgentNickname() + " to " + receiverName);

      // perform action
      super.sendMessage(message, receiverName);
   }

   public ActionInstanceDescription buildActionInstanceDescription(String action, HashMap properties)
   {
      ActionInstanceDescription aid = null;
      try
      {
         if (properties == null)
            aid = new ActionInstanceDescriptionImpl(null, action, NamespaceValidator.validateURI(this.getName()));
         else
            aid = new ActionInstanceDescriptionImpl(null, action, NamespaceValidator.validateURI(this.getName()), properties);
      }
      catch (URISyntaxException ex)
      {
         ex.printStackTrace();
      }

      return aid;
   }

   protected void enforcePolicies(ActionInstanceDescription aid) throws Exception
   {
      PolicyChecking policyChecking = CSIFactory.getPolicyChecking();
      List<ObligationActionDescription> obligations = policyChecking.findPolicyDecision(aid, null);
      for (ActionInstanceDescription action : obligations)
      {
         performAction(action);
      }
   }

   private void performAction(ActionInstanceDescription aid) throws Exception
   {
      // get action name from aid
      String actionName = aid.getActionClassName();

      // get short name from fully qualified name
      String methodName = KAoSPolicyBuilderUtil.getShortName(actionName);

      Map<String, OntPropertyDescription> properties = aid.getPropertiesMap();
      Object[] argObjects = {properties};

      // find method in class
      Method methods[] = this.getClass().getMethods();
      for (int i = 0; i < methods.length; i++)
      {
         if (methods[i].getName().equals(methodName))
         {
            // System.out.println("found method match");
            // if found
            // check number of args
            Class[] parameters = methods[i].getParameterTypes();
            if (parameters.length != argObjects.length)
            {
               // System.out.println("\t but number of args does not match");
               continue;
            }

            // check parameter type
            boolean match = true;
            for (int j = 0; j < parameters.length; j++)
            {
               if (!parameters[j].isInstance(argObjects[j]))
               {
                  // System.out.println("\t but arg tpes don't match");
                  match = false;

                  break;
               }
            }

            if (!match)
               continue;
            else
            {
               // System.out.println("\t arg number and type seem OK");
               // execute and return
               methods[i].invoke(this, argObjects);

               return;
            }
         }
      }

      // System.out.println("Did not find method for " + methodName);
   }

   public void Move(String requestor, double distance) throws Exception
   {
      // build aid
      HashMap properties = new HashMap();
      properties.put(ActionConcepts.requestedBy(), NamespaceValidator.validateURI(requestor));

//    OntPropertyDescriptionImpl distanceOPD = new OntPropertyDescriptionImpl(TeleoperationConcepts.hasDistance());
//    distanceOPD.addValue("" + distance);
//    properties.put(TeleoperationConcepts.hasDistance(), distanceOPD);
      ActionInstanceDescription aid = buildActionInstanceDescription("http://ontology.ihmc.us/Robot/Teleoperation.owl#Move", properties);

      // perform enforcement
      enforcePolicies(aid);

      // perform move
      System.out.println(_actorDesc.getAgentNickname() + ": moving " + distance + " as requested by " + requestor);

      // move is now finished, so check policies again
      OntPropertyDescription status = new OntPropertyDescriptionImpl(ActionStatusConcepts.hasStatus());
      status.setValue(ActionStatusConcepts.DefaultFinishedActionStatus());
      aid.addProperty(status);

      // perform enforcement
//    enforcePolicies(aid);

      // log the Move action to the History Monitor
      EventHistoryComponent historyMonitor = CSIFactory.getHistoryMonitor();
      historyMonitor.logEvent(aid);
      System.out.println("recorded Move action with Finished status to History Monitor.");
   }

   public void sendMessage(String message, String receiverName) throws Exception
   {
      // build aid
      HashMap properties = new HashMap();
      properties.put(ActionConcepts.hasDestination(), NamespaceValidator.validateURI(receiverName));
      properties.put(ActionConcepts.carriesMessage(), message);
      ActionInstanceDescription aid = buildActionInstanceDescription(ActionConcepts.CommunicationAction(), properties);

      // perform enforcement
      enforcePolicies(aid);

      // log the communication action to the History Monitor
      EventHistoryComponent historyMonitor = CSIFactory.getHistoryMonitor();
      historyMonitor.logEvent(aid);
      System.out.println("recorded SendCommunicationAction to History Monitor from: " + _actorDesc.getAgentNickname() + " to " + receiverName);

      // perform action
      super.sendMessage(message, receiverName);
   }

   // This is just for display purposes
   private int numberOfLoginFailures = 0;

   public void LoginAction(String password) throws Exception
   {
      System.out.println("trying to login:" + password);
      PolicyChecking policyChecking = CSIFactory.getPolicyChecking();
      EventHistoryComponent historyMonitor = CSIFactory.getHistoryMonitor();

      HashMap properties = new HashMap();
      properties.put(ComputingActionConcepts.hasUsername(), _actorDesc.getAgentNickname());
      properties.put(ComputingActionConcepts.hasPassword(), password);

      ActionInstanceDescription logginAID = buildActionInstanceDescription(ComputingActionConcepts.LoginAction(), properties);
      policyChecking.findPolicyDecision(logginAID, null);
      System.out.println("passed policy");

      if (!password.equals(_actorDesc.getAgentNickname().toLowerCase()))
      {
         System.out.println("bad password for actor: " + _actorDesc.getAgentNickname());

         // Add the status information property
         OntPropertyDescriptionImpl hasStatusProperty = new OntPropertyDescriptionImpl(ActionStatusConcepts.hasStatus());
         hasStatusProperty.addValue(ActionStatusConcepts.DefaultFailureActionStatus);
         logginAID.addProperty(hasStatusProperty);

         // TODO: move this call to CSI Interface when it will be available in CSI
         historyMonitor.logEvent(logginAID);
         System.out.println("recorded failed login to History Monitor.");

         numberOfLoginFailures++;

         throw new Exception("" + numberOfLoginFailures);
      }
      else
      {
         System.out.println("password OK for actor: " + _actorDesc.getAgentNickname());
         OntPropertyDescriptionImpl hasStatusProperty = new OntPropertyDescriptionImpl(ActionStatusConcepts.hasStatus());
         hasStatusProperty.addValue(ActionStatusConcepts.DefaultCompletedActionStatus);
         logginAID.addProperty(hasStatusProperty);
         historyMonitor.logEvent(logginAID);
         System.out.println(" recorded successful login to History Monitor.");
      }
   }


   public void ChangeStateAction(String stateName, String subjectName, String value) throws Exception
   {
      StateManager stateManager = StateManagerImpl.getInstance();
      System.out.println("trying to change " + subjectName + " " + stateName + " to " + value);
      OntInstanceDescription stateDescription = new OntInstanceDescriptionImpl(stateName + "_for_" + subjectName, stateName);
      stateDescription.addProperty(stateName, value);
      stateManager.registerState(stateDescription);

//      PolicyChecking policyChecking = CSIFactory.getPolicyChecking();
//
//      HashMap properties = new HashMap();
//      properties.put(ActionConcepts.hasOldState, _curStatePlace);
//      properties.put(ActionConcepts.hasNewState, stateName);
//
//      ActionInstanceDescription changeStateAID = buildActionInstanceDescription(ActionConcepts.ChangeStateAction, properties);
//      policyChecking.findPolicyDecision(changeStateAID, null);
//      System.out.println("passed policy");
//      _curStatePlace = stateName;

//      stateMonitor.updateProperty(PlaceConcepts.hasPlace, newStatePlace, null);
//      System.out.println("logged to State Monitor");
//      System.out.println("logged:\n" + stateName);

   }


   public static void main(String[] args) throws Exception
   {
      // create agent
      SimpleAgent agent = new SimpleAgent(args);

      // wait around until terminated
      synchronized (Thread.currentThread())
      {
         Thread.currentThread().wait();
      }

   }
}
