package kaos.core.tutorial;

import kaos.core.csi.AgentCapability;
import kaos.core.csi.AgentCapabilityImpl;
import kaos.core.csi.CSIFactory;
import kaos.core.csi.policy.KAoSSecurityException;
import kaos.core.csi.query.QueryRegistration;
import kaos.core.service.directory.KAoSAgentDescription;
import kaos.core.service.directory.KAoSEntityDescription;
import kaos.ontology.vocabulary.ActionConcepts;
import kaos.ontology.vocabulary.ActorConcepts;
import kaos.ontology.vocabulary.EntityConcepts;

import javax.swing.*;
import javax.swing.border.BevelBorder;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.*;
import java.util.List;

public class TutorialDemo
{
   private boolean DEBUG = false;
   private JList agentList;
   private DefaultListModel agentListModel;
   private JList queryResultList;
   private DefaultListModel queryResultListModel;
   private final JComboBox senderCmb = new JComboBox();
   private final JComboBox receiverCmb = new JComboBox();
   private final JComboBox performerCmb = new JComboBox();
   private final JComboBox requestorCmb = new JComboBox();
   private final JTextField distanceTF = new JTextField("1.0");
   private final JComboBox loginCmb = new JComboBox();

   /**
    * newStateCmb: changes the value of the current State variable hasPlace
    * for testing state condition monitoring
    */
   private final JComboBox newStateCmb = new JComboBox();

   private enum AGENT_TYPES {HUMAN, DOCTOR, ROBOT}

   private HashMap agents = new HashMap();

   public TutorialDemo()
   {
      buildGUI();
   }

   private void buildGUI()
   {
      // main panel
      JPanel panel = new JPanel();
      GridBagConstraints gbc = new GridBagConstraints();
      panel.setLayout(new GridBagLayout());
      panel.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
      Dimension dimension = new Dimension(1200, 300);
      panel.setPreferredSize(dimension);

      // layout components
      gbc.gridx = 0;
      gbc.gridy = 0;
      gbc.fill = GridBagConstraints.BOTH;
      gbc.weightx = 1;
      gbc.weighty = 0;
      gbc.gridwidth = 2;
      gbc.gridwidth = 1;

      // create an agent
      final JComboBox typeCmb = new JComboBox();
      typeCmb.addItem(AGENT_TYPES.HUMAN);
      typeCmb.addItem(AGENT_TYPES.ROBOT);

      final JTextField agentNameTF = new JTextField("joe");
      final JTextField agentDomainTF = new JTextField("TeamA");
      JButton createAnAgentBtn = new JButton("Create an agent");
      createAnAgentBtn.addActionListener(new ActionListener()
      {
         public void actionPerformed(ActionEvent ae)
         {
            try
            {
               createAnAgent((AGENT_TYPES) typeCmb.getSelectedItem(), agentNameTF.getText(), agentDomainTF.getText());
            }
            catch (Exception ex)
            {
               ex.printStackTrace();
            }
         }
      });

      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(createAnAgentBtn, gbc);
      gbc.gridx++;
      panel.add(typeCmb, gbc);
      gbc.gridx++;
      JLabel nameLbl = new JLabel("name: ");
      nameLbl.setHorizontalAlignment(JLabel.RIGHT);
      gbc.weightx = 0.5;
      panel.add(nameLbl, gbc);
      gbc.weightx = 1;
      gbc.gridx++;
      panel.add(agentNameTF, gbc);
      gbc.gridx++;
      JLabel domainLbl = new JLabel("domain: ");
      domainLbl.setHorizontalAlignment(JLabel.RIGHT);
      gbc.weightx = 0.5;
      panel.add(domainLbl, gbc);
      gbc.weightx = 1;
      gbc.gridx++;
      panel.add(agentDomainTF, gbc);

      // create team button
      JButton createTeamBtn = new JButton("Create a team");
      createTeamBtn.addActionListener(new ActionListener()
      {
         public void actionPerformed(ActionEvent ae)
         {
            try
            {
               createAnAgent(AGENT_TYPES.HUMAN, "bob", "TeamA");
               createAnAgent(AGENT_TYPES.ROBOT, "r1", "TeamA");
               createAnAgent(AGENT_TYPES.HUMAN, "larry", "TeamB");
               createAnAgent(AGENT_TYPES.ROBOT, "r2", "TeamB");
               createAnAgent(AGENT_TYPES.HUMAN, "sam", "TeamC");
               createAnAgent(AGENT_TYPES.ROBOT, "r3", "TeamC");            }
            catch (Exception ex)
            {
               ex.printStackTrace();
            }
         }
      });
      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(createTeamBtn, gbc);


      // query controls
      final JComboBox ontTypeCmb = new JComboBox();
      ontTypeCmb.addItem(ActorConcepts.Actor());
      ontTypeCmb.addItem(ActorConcepts.Human());
      ontTypeCmb.addItem(ActorConcepts.Robot());

      final JComboBox capabilityCmb = new JComboBox();
      capabilityCmb.addItem("ANY");
      capabilityCmb.addItem(EntityConcepts.Entity());
      JButton queryBtn = new JButton("Query for agents");
      queryBtn.addActionListener(new ActionListener()
      {
         public void actionPerformed(ActionEvent ae)
         {
            queryResultListModel.clear();
            List<KAoSAgentDescription> queryResult = queryForAgents((String) ontTypeCmb.getSelectedItem(), (String) capabilityCmb.getSelectedItem());
            for (KAoSAgentDescription desc : queryResult)
            {
               String agentName = desc.getAgentNickname();
               queryResultListModel.addElement(agentName);
            }
         }
      });

      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(queryBtn, gbc);
      gbc.gridx++;
      panel.add(ontTypeCmb, gbc);
      gbc.gridx++;
      gbc.gridx++;
      panel.add(capabilityCmb, gbc);

      // send message controls
      final JTextField messageTF = new JTextField("hello");
      JButton sendMessagequeryBtn = new JButton("Send message");
      sendMessagequeryBtn.addActionListener(new ActionListener()
      {
         public void actionPerformed(ActionEvent ae)
         {
            SimpleAgent sender = (SimpleAgent) agents.get(senderCmb.getSelectedItem());
            try
            {
               sender.sendMessage(messageTF.getText(), (String) receiverCmb.getSelectedItem());
            }
            catch (Exception ex)
            {
               ex.printStackTrace();
            }
         }
      });

      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(sendMessagequeryBtn, gbc);
      gbc.gridx++;
      gbc.weightx = 1;
      panel.add(messageTF, gbc);
      gbc.gridx++;
      JLabel fromLbl = new JLabel("from: ");
      fromLbl.setHorizontalAlignment(JLabel.RIGHT);
      gbc.weightx = 0.5;
      panel.add(fromLbl, gbc);
      gbc.weightx = 1;
      gbc.gridx++;
      panel.add(senderCmb, gbc);
      gbc.gridx++;
      JLabel toLbl = new JLabel("to: ");
      toLbl.setHorizontalAlignment(JLabel.RIGHT);
      gbc.weightx = 0.5;
      panel.add(toLbl, gbc);
      gbc.weightx = 1;
      gbc.gridx++;
      panel.add(receiverCmb, gbc);

      // perform action controls
      final JButton performBtn = new JButton("Perform Communication Action");
      performBtn.addActionListener(new ActionListener()
      {
         public void actionPerformed(ActionEvent ae)
         {
            // get selected perform
            SimpleAgent agent = (SimpleAgent) agents.get(senderCmb.getSelectedItem());
            try
            {
               agent.CommunicationAction(messageTF.getText(), (String) receiverCmb.getSelectedItem());
               performBtn.setBackground(Color.green);
            }
            catch (Exception ex)
            {
               if (ex instanceof KAoSSecurityException)
                  performBtn.setBackground(Color.red);
               else
                  ex.printStackTrace();
            }

         }
      });

      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(performBtn, gbc);

      // perform action controls

      final JButton moveBtn = new JButton("Perform Move Action");
      moveBtn.addActionListener(new ActionListener()
      {
         public void actionPerformed(ActionEvent ae)
         {
            // get selected perform
            SimpleAgent agent = (SimpleAgent) agents.get(performerCmb.getSelectedItem());
            try
            {
               agent.Move((String) requestorCmb.getSelectedItem(), Double.parseDouble(distanceTF.getText()));
               moveBtn.setBackground(Color.green);
            }
            catch (Exception ex)
            {
               if (ex instanceof KAoSSecurityException)
                  moveBtn.setBackground(Color.red);
            }

         }
      });

      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(moveBtn, gbc);
      gbc.gridx++;
      panel.add(performerCmb, gbc);
      gbc.gridx++;
      JLabel requestorLbl = new JLabel("requestor: ");
      requestorLbl.setHorizontalAlignment(JLabel.RIGHT);
      gbc.weightx = 0.5;
      panel.add(requestorLbl, gbc);
      gbc.weightx = 1;
      gbc.gridx++;
      panel.add(requestorCmb, gbc);
      JLabel distanceLbl = new JLabel("distance: ");
      distanceLbl.setHorizontalAlignment(JLabel.RIGHT);
      gbc.weightx = 1;
      gbc.gridx++;
      panel.add(distanceLbl, gbc);
      gbc.weightx = 1;
      gbc.gridx++;
      panel.add(distanceTF, gbc);

      // launch Login agent
      final JTextField passwordTF = new JTextField("bob");
      final JButton resultBtn = new JButton("Result");
      resultBtn.setEnabled(false);

      // attempt login
      final JButton loginBtn = new JButton("Login");
      loginBtn.addActionListener(new ActionListener()
      {
         public void actionPerformed(ActionEvent ae)
         {
            // get selected perform
            SimpleAgent agent = (SimpleAgent) agents.get(loginCmb.getSelectedItem());
            try
            {
               agent.LoginAction(passwordTF.getText());
               resultBtn.setBackground(Color.GREEN);
               resultBtn.setText("Successfully Logged In");
            }
            catch (Exception ex)
            {
               if (ex instanceof KAoSSecurityException)
               {
                  resultBtn.setBackground(Color.RED);
                  resultBtn.setText("Not Authorized");
               }
               else
               {
                  resultBtn.setBackground(Color.yellow);
                  resultBtn.setText("Invalid Password #" + ex.getMessage());
               }
            }
         }
      });


      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(loginBtn, gbc);
      gbc.gridx++;
      gbc.weightx = 0.5;
      panel.add(loginCmb, gbc);
      gbc.weightx = 0.5;
      gbc.gridx++;
      JLabel passwordLbl = new JLabel("pwd: ");
      passwordLbl.setHorizontalAlignment(JLabel.RIGHT);
      gbc.weightx = 1.0;
      panel.add(passwordLbl, gbc);
      gbc.weightx = 1;
      gbc.gridx++;
      panel.add(passwordTF, gbc);
      gbc.gridx++;
      panel.add(resultBtn, gbc);

      // start section for State controls
      Set<String> placeInstances;

      // query for all Place instances to populate the combo box
      try
      {
//         placeInstances = CSIFactory.getOntologyService().getInstancesOf("http://ontology.ihmc.us/Computing/ComputingEntity.owl#memTotalFree");

//         for (String place : placeInstances)
//         {
            newStateCmb.addItem("http://ontology.ihmc.us/Computing/ComputingEntity.owl#memTotalFree");
//         }
      }
      catch (Exception e1)
      {
         // TODO Auto-generated catch block
         e1.printStackTrace();
      }

      final JTextField stateValue = new JTextField("0");

      final JButton curStateBtn = new JButton("State");
      curStateBtn .setEnabled(false);

      // click to change the hasPlace property of the current state
      final JButton changeStateButton = new JButton("Update State");
      changeStateButton.addActionListener(new ActionListener()
      {
         public void actionPerformed(ActionEvent ae)
         {
            // the selected agent reports the state change
            SimpleAgent agent = (SimpleAgent) agents.get(loginCmb.getSelectedItem());
            try
            {
               agent.ChangeStateAction(newStateCmb.getSelectedItem().toString(), "joe", stateValue.getText());
               curStateBtn .setBackground(Color.GREEN);
               curStateBtn .setText(stateValue.getText());
            }
            catch (Exception ex)
            {
               if (ex instanceof KAoSSecurityException)
               {
                  curStateBtn .setBackground(Color.RED);
                  System.out.println(agent.getName() + " is not authorized to change the value of hasPlace for the current State.");
               }
               else
               {
                  curStateBtn .setBackground(Color.yellow);
                  ex.printStackTrace();
               }
            }
         }
      });


      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(changeStateButton, gbc);
      gbc.gridx++;
      gbc.weightx = 0.5;
      panel.add(newStateCmb, gbc);
      gbc.gridx++;
      gbc.weightx = 0.5;
      panel.add(stateValue, gbc);
      gbc.weightx = 0.5;
      gbc.gridx++;
      gbc.gridx++;
      gbc.gridx++;
      panel.add(curStateBtn , gbc);

      // end section for state controls


      // panel to list all agents created
      gbc.gridx = 0;
      gbc.gridy++;
      gbc.gridwidth = 2;
      gbc.weighty = 1;
      panel.add(buildAgentsPanel(), gbc);

      // panel to display query results
      gbc.gridx = gbc.gridx + 2;
      gbc.gridwidth = 6;
      panel.add(buildQueryPanel(), gbc);

      JFrame f = new JFrame("KAoS Tutorial");
      f.addWindowListener(new WindowAdapter()
      {
         public void windowClosing(WindowEvent e)
         {
            System.exit(0);
         }
      });

      f.getContentPane().add(new JScrollPane(panel), BorderLayout.CENTER);
      f.pack();
      f.setVisible(true);
   }

   private JPanel buildAgentsPanel()
   {
      // main panel
      JPanel panel = new JPanel();
      GridBagConstraints gbc = new GridBagConstraints();
      panel.setLayout(new GridBagLayout());
      panel.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED), "Agents", TitledBorder.LEFT, TitledBorder.TOP));

      // create robot selection list
      JPanel selectionPanel = new JPanel();
      selectionPanel.setLayout(new BoxLayout(selectionPanel, BoxLayout.Y_AXIS));
      selectionPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
      agentListModel = new DefaultListModel();
      agentList = new JList(agentListModel);
      agentList.setVisibleRowCount(6);
      JScrollPane selectionListScrollPane = new JScrollPane(agentList);
      selectionPanel.add(selectionListScrollPane);

      // layout components
      gbc.gridx = 0;
      gbc.gridy = 0;
      gbc.fill = GridBagConstraints.BOTH;
      gbc.weightx = 1;
      gbc.weighty = 1;

      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(selectionPanel, gbc);

      return panel;
   }

   private JPanel buildQueryPanel()
   {
      // main panel
      JPanel panel = new JPanel();
      GridBagConstraints gbc = new GridBagConstraints();
      panel.setLayout(new GridBagLayout());
      panel.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED), "Query Results", TitledBorder.LEFT, TitledBorder.TOP));

      // create robot selection list
      JPanel selectionPanel = new JPanel();
      selectionPanel.setLayout(new BoxLayout(selectionPanel, BoxLayout.Y_AXIS));
      selectionPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
      queryResultListModel = new DefaultListModel();
      queryResultList = new JList(queryResultListModel);
      queryResultList.setVisibleRowCount(6);
      JScrollPane selectionListScrollPane = new JScrollPane(queryResultList);
      selectionPanel.add(selectionListScrollPane);

      // layout components
      gbc.gridx = 0;
      gbc.gridy = 0;
      gbc.fill = GridBagConstraints.BOTH;
      gbc.weightx = 1;
      gbc.weighty = 1;

      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(selectionPanel, gbc);

      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(selectionPanel, gbc);

      return panel;
   }



   private void createAnAgent(AGENT_TYPES type, String name, String domain)
   {
      // add domain to a Vector
      Vector domains = new Vector();
      domains.add(domain);

      // determine the type of agent requested and create
      SimpleAgent agent = null;
      try
      {
         if (type == AGENT_TYPES.HUMAN)
         {
            agent = new SimpleAgentHuman(name, domains);
         }
         else if (type == AGENT_TYPES.ROBOT)
         {
            agent = new SimpleAgentRobot(name, domains);
         }
         else
         {
            System.err.println("type not valid: " + type);

            return;
         }
      }
      catch (Throwable e)
      {
         e.printStackTrace();
      }

      // register agent
      try
      {
         agent.registerWithKAoS();
      }
      catch (Exception ex)
      {
         ex.printStackTrace();
      }

      // track info in GUI
      agents.put(name, agent);
      agentListModel.addElement(name);
      senderCmb.addItem(name);
      receiverCmb.addItem(name);
      performerCmb.addItem(name);
      requestorCmb.addItem(name);
      loginCmb.addItem(name);
   }

   private List<KAoSAgentDescription> queryForAgents(String type, String capability)
   {
      // get query instance
      QueryRegistration query = null;
      try
      {
         query = CSIFactory.getQueryRegistration();
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }

      // get agents matching class
      Hashtable searchAttributes = new Hashtable();
      String key = KAoSEntityDescription.ENTITY_ONTOLOGICAL_TYPE_NAMES;
      Vector ontNames = new Vector();
      ontNames.add(type);
      searchAttributes.put(key, ontNames);
      List<KAoSAgentDescription> typeResult = null;
      try
      {
         typeResult = query.getActorsWithAttributes(searchAttributes);
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }

      // get agenst matching capability
      if (capability.equals("ANY"))
         return typeResult;

      key = KAoSAgentDescription.AGENT_CAPABILITIES;
      AgentCapability ac = new AgentCapabilityImpl();
      ac.setName(capability);
      ArrayList capabilities = new ArrayList();
      capabilities.add(ac);
      searchAttributes.put(key, capabilities);
      List<KAoSAgentDescription> capabilityResult = null;
      try
      {
         capabilityResult = query.getActorsWithAttributes(searchAttributes);
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }

      // merge results
      List<KAoSAgentDescription> queryResult = new ArrayList<KAoSAgentDescription>();
      for (KAoSAgentDescription desc : typeResult)
      {
         for (KAoSAgentDescription desc2 : capabilityResult)
         {
            if (desc2.getAgentNickname().equals(desc.getAgentNickname()))
            {
               queryResult.add(desc);
            }
         }
      }

      return queryResult;
   }

   public static void main(String[] args) throws Exception
   {
      TutorialDemo demo = new TutorialDemo();
   }
}
