/*
 * Created on Jan 28, 2009 To change the template for this generated file go to Window - Preferences - Java - Code Generation - Code and Comments
 */
package kaos.policy.builder;

import com.hp.hpl.jena.n3.N3JenaWriter;

import kaos.core.util.PolicyMsg;
import kaos.ontology.roleValueMap.KAoSRoleValueMapContainer;
import kaos.ontology.util.*;

public interface KAoSPolicyBuilder extends KAoSOntBuilder
{
    // Formats for policy ontology text representation
    public static final String        OWL_SEPERATE_ELEMENTS_FORMAT = "OWL_SEPERATE_ELEMENTS_FORMAT";
    public static final String        OWL_FORMAT                   = "RDF/XML-ABBREV";
    public static final String        N3_PP_FORMAT                 = N3JenaWriter.n3WriterPrettyPrinter;
    public static final String        N3_PLAIN_FORMAT              = N3JenaWriter.n3WriterPlain;
    public static final String        N3_TRIPLES_FORMAT            = N3JenaWriter.n3WriterTriples;
    public static final String        TURTLE_FORMAT                = N3JenaWriter.turtleWriter;
    
    /*
     * Create an authorization policy 'shell'
     * @param policyName - human name for the policy (policy id is generated by this method)
     * @param policyDescription - any human description for the policy
     * @param policyPriority - priority 1 is lowest
     * @param policyModality - positive or negative authorization either PolicyConcepts.PosAuthorizationPolicy() or PolicyConcepts.NegAuthorizationPolicy()
     * @param triggerSubjectScope - policy applies to either an agent - KAoSConstants.AGENT_SCOPE or a class of actors - KAoSConstants.ACTOR_CLASS_SCOPE
     * @param subjectID - agent id or actor class for the policy
     * @param actionClass - class of action that the policy either authorizes or forbids
     * @throws ClassNameNotSet
     * @throws DisjointClassWithAlreadyAdded
     * @throws SpecifiedModalityTypeNotExists
     * @throws PriorityNotValid
     * @throws PolicyNameNotSet
     * @throws RangeIsBasedOnAClass
     */
    public void initAuthorizationPolicy(String policyName, String policyDescription, int policyPriority, String policyModality, String subjectScope,
            String subjectID, String actionClass) throws ClassNameNotSet, DisjointClassWithAlreadyAdded, SpecifiedModalityTypeNotExists, PriorityNotValid,
            PolicyNameNotSet, RangeIsBasedOnAClass;

    /*
     * Create an obligation policy 'shell' with everything necessary EXCEPT the trigger and obligation action property restrictions
     * @param policyName - human name for the policy (policy id is generated by this method)
     * @param policyDescription - any human description for the policy
     * @param policyPriority - priority 1 is lowest
     * @param policyModality - positive or negative obligation either PolicyConcepts.PosObligationPolicy() or PolicyConcepts.NegObligationPolicy()
     * @param triggerSubjectScope - policy applies to either an agent - KAoSConstants.AGENT_SCOPE or a class of actors - KAoSConstants.ACTOR_CLASS_SCOPE
     * @param triggerSubjectID - agent id or actor class for the policy (assuming the same actor for both trigger and obligation actions)
     * @param obligedActionClass - Class of action to be obligated
     * @param triggerActionClass - Class of action that triggers the policy
     * @throws ClassNameNotSet
     * @throws DisjointClassWithAlreadyAdded
     * @throws SpecifiedModalityTypeNotExists
     * @throws PriorityNotValid
     * @throws PolicyNameNotSet
     * @throws RangeIsBasedOnAClass
     */
    public void initObligationPolicy(String policyName, String policyDescription, int policyPriority, String policyModality, String triggerSubjectScope,
            String triggerSubjectID, String obligedActionClass, String triggerActionClass) throws ClassNameNotSet, DisjointClassWithAlreadyAdded,
            SpecifiedModalityTypeNotExists, PriorityNotValid, PolicyNameNotSet, RangeIsBasedOnAClass;

    /*
     * Constructor PolicyMsg for the build policy
     * @throws ValueNotSet
     * @throws PolicyBuildingNotCompleted
     */
    public PolicyMsg getPolicyMsg() throws PolicyBuildingNotCompleted;

    /*
     * Starts building the new policy, delete any existing state of the previously build policy.
     */
    public void resetPolicyBuilding();

    /*
     * Get the value of the time stamp for the current policy.
     * @return the value of the time stamp
     * @exception ValueNotSet if value of the time stamp is not known
     */
    public Long getUpdateTimeStamp() throws ValueNotSet;

    public void setUpdateTimeStamp(Long newTime) throws ValueNotSet;

    /*
     * Set the ID of the defined policy and its modality type. Currently there is no method to change the ID and the modality type of the policy; in this
     * case a new policy should be build or the current policy reset.
     * @param policyID the name of the policy
     * @param modalityType url of the class representing the modality type of policy either authorization or obligation
     * @exception SpecifiedModalityTypeNotExists if the modality type does not exist
     */
    public void setPolicyIDAndModalityType(String policyID, String modalityType) throws SpecifiedModalityTypeNotExists;

    /*
     * Get the id of the current policy.
     * @return the id of the policy
     * @exception ValueNotSet if value of the policy ID is not known
     */
    public String getPolicyID() throws ValueNotSet;

    /*
     * @param myKAoSRoleValueMapContainer
     */
    public void setPolicyRVMContainer(KAoSRoleValueMapContainer myKAoSRoleValueMapContainer);

    /*
     * @return the container with role-value-map constraints for Policy Class
     */
    public KAoSRoleValueMapContainer getPolicyRVMContainer();

    /*
     * Get the value of the modality type for the current policy.
     * @return the name of the modality type
     * @exception ValueNotSet if value of the modality type is not known
     */
    public String getModalityType() throws ValueNotSet;

    /*
     * Set the human name of the defined policy. If the name is already set, it will be replaced by the given name. 
     * NOTE : The name of the policy and the ID of the policy are two separate entities. The name can be changed, but the ID can't be.
     * @param policyName the name of the policy
     */
    public void setPolicyName(String policyName);

    /*
     * Get the value of the policy name for the current policy.
     * @return the name of the policy
     * @exception ValueNotSet if value of the modality type is not known
     */
    public String getPolicyName() throws ValueNotSet;

    /*
     * Set the author of the defined policy.
     * @param authorId URI of the author
     */
    public void setPolicyAuthor(String authorId);

    /*
     * Get the author of the policy
     * @return the uri of the policy author
     * @exception ValueNotSet if value of the modality type is not known
     */
    public String getPolicyAuthor() throws ValueNotSet;

    /*
     * Set the description of the defined policy. If the description is already set, it will be replaced by the given description
     * @param desc description of the policy
     */
    public void setPolicyDescription(String desc);

    /*
     * Get the value of the policy description for the current policy.
     * @return the description of the policy
     * @exception ValueNotSet if value of the modality type is not known
     */
    public String getPolicyDesc() throws ValueNotSet;

    /*
     * Set the priority of the defined policy. If the priority is already set, it will be replaced by the given value.
     * @param priority priority of the policy
     */
    public void setPriority(int priority) throws PriorityNotValid, PolicyNameNotSet;

    /*
     * Get the priority for the current policy.
     * @return the value of the priority
     * @exception ValueNotSet if value of the priority is not known
     */
    public int getPriority() throws ValueNotSet;

    /*
     * Set the action that is controlled (authorized or obliged) by this policy. If the action is already set, it will be replaced by the given one.
     * @param controls the action to be controlled
     */
    public void setControlActionClass(KAoSOntClassBuilder controls);

    /*
     * Get the action that is controlled (authorized or obliged) by this policy.
     * @return the action that is controlled
     */
    public KAoSOntClassBuilder getControlActionClass();

    /*
     * Set the control action role-value-map container
     * @param myKAoSRoleValueMapContainer the container with role-value-map definitions for policy controlled action
     */
    public void setControlActionRVMContainer(KAoSRoleValueMapContainer myKAoSRoleValueMapContainer);

    /*
     * Get the control action role-value-map container
     * @return the container with role-value-map constraints for Control Action
     */
    public KAoSRoleValueMapContainer getControlActionRVMContainer();

    /*
     * Set the trigger action that will make this obligation policy applicable. 
     * If the action is already set, it will be replaced by the given one.
     * @param trigger the triggering action
     */
    public void setTriggerActionClass(KAoSOntClassBuilder trigger);

    /*
     * Get the trigger action for the obligation policy.
     * @return the trigger action
     */
    public KAoSOntClassBuilder getTriggerActionClass();

    /*
     * Set the trigger action role-value-map container
     * @param myKAoSRoleValueMapContainer the container with role-value-map definitions for trigger action
     */
    public void setTriggerActionRVMContainer(KAoSRoleValueMapContainer myKAoSRoleValueMapContainer);
    
    /*
     * Get the trigger action role-value-map container
     * @return the container with role-value-map constraints for Trigger Action
     */
    public KAoSRoleValueMapContainer getTriggerActionRVMContainer();

    /*
     * Set the condition (history and state) of this policy. If the class is already set, it will be replaced by the given one.
     * @param condition condition the condition action
     */
    public void setConditionClass(ConditionBuilder condition);

    /*
     * Get the condition (history and state) for this policy
     * @return the condition action
     */
    public ConditionBuilder getConditionClass();

    /*
     * Set the condition role-value-map container
     * @param the container with role-value-map constraints for Condition
     */
    public void setConditionRVMContainer(KAoSRoleValueMapContainer myKAoSRoleValueMapContainer);
    
    /*
     * Get the condition role-value-map container
     * @return the container with role-value-map constraints for Condition
     */
    public KAoSRoleValueMapContainer getConditionRVMContainer();

    /*
     * Set the obligation constraints for this policy
     * @param myObligationConstraints the constraints for this obligation policy
     */
    public void setObligationConstraints(ObligationConstraintsBuilder myObligationConstraints);

    /*
     * Get the obligation constraints for this policy
     * @return the constraints for this obligation policy 
     */
    public ObligationConstraintsBuilder getObligationConstraints();

    /**
     * Get the text for policy ontology in the specified format
     * @param format for the representation of the policy ontology
     * @return the text representing the policy ontology
     */
    public String getText(String format);
    
    // --- Policy construction exceptions

    public class PolicyBuildingNotCompleted extends Exception
    {
        private static final long serialVersionUID = 1400826844102781536L;

        public PolicyBuildingNotCompleted(String reason)
        {
            super(reason);
        }

        public PolicyBuildingNotCompleted()
        {
            super();
        }
    }

    public class SpecifiedModalityTypeNotExists extends Exception
    {
        private static final long serialVersionUID = -557410486754321624L;

        public SpecifiedModalityTypeNotExists()
        {
            super();
        }

        public SpecifiedModalityTypeNotExists(String s)
        {
            super(s);
        }
    }

    public class PolicyNameNotSet extends Exception
    {
        private static final long serialVersionUID = -2382624374903465940L;

        public PolicyNameNotSet()
        {
            super();
        }

        public PolicyNameNotSet(String s)
        {
            super(s);
        }
    }

    public class PriorityNotValid extends Exception
    {
        private static final long serialVersionUID = -2894651535877200030L;

        public PriorityNotValid()
        {
            super();
        }

        public PriorityNotValid(String s)
        {
            super(s);
        }
    }
}