/*
 * Created on Jan 28, 2009 To change the template for this generated file go to Window - Preferences - Java - Code Generation - Code and Comments
 */
package kaos.policy.builder;

import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntResource;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;
import kaos.ontology.util.OntModelUtils;
import kaos.ontology.util.SerializableOntModelImpl;
import kaos.ontology.vocabulary.DefaultNamespaces;
import kaos.ontology.vocabulary.PolicyConcepts;

import java.util.HashSet;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.Vector;

public class KAoSPolicyBuilderUtil
{
    public static final String CLASS_NAME_POLICY_ID_PREFIX = "-Subclass-For-Policy-";
    public static final String POLICY_ID_PREFIX = DefaultNamespaces.defaultPolicyNamespace;
    public static final String CLASS_NAME_TYPE_PREFIX = "-For-Property-";

    /**
     *  Generates name for a class inside a policy
     *
     *  @param type the type of class (controls, trigger, condition, or obligation)
     *  @param policyId the id of the policy (should begin with POLICY_ID_PREFIX)
     */
    public static String generateClassNameForPolicy (String baseClass, String type, String policyId)
    {
        String className = DefaultNamespaces.defaultKAoSNamespace + KAoSPolicyBuilderUtil.getShortName(baseClass) +
               CLASS_NAME_POLICY_ID_PREFIX + policyId.substring(POLICY_ID_PREFIX.length()) + CLASS_NAME_TYPE_PREFIX + type;

        return className;
    }

    /**
     *  Extract a short name from the URI
     *
     *  @param name to be shorten
     */
    public static String getShortName (String name)
    {
        // Parse short name by taking everything after the #
        StringTokenizer t = new StringTokenizer(name,"#");
        String shortName = name;
        
        while (t.hasMoreTokens())
            shortName = t.nextToken();
        
        // Remove is or has from the property name
        if (shortName.startsWith("is") && Character.isUpperCase(shortName.charAt(2)))
            shortName = Character.toLowerCase(shortName.charAt(2)) + shortName.substring(3);
        else if (shortName.startsWith("has") && Character.isUpperCase(shortName.charAt(3))) 
            shortName = Character.toLowerCase(shortName.charAt(3)) + shortName.substring(4);
        
        return shortName;
    }
    
    /*
     * A method finding ids for all policy in the OWL String
     * @return set of ids of the found policy instances
     */
    public static Set<String> findPoliciesInOWL(String owlString)
    {
        if (owlString == null)
            return null;

        SerializableOntModelImpl policiesModel = OntModelUtils.constructOntModelForRootInstance(owlString);
        
        return findPoliciesInModel(policiesModel);
    }

    /*
     * A method finding ids for all policy in the ontology model
     * @return set of ids of the found policy instances
     */
    public static Set<String> findPoliciesInModel(SerializableOntModelImpl myOntModel)
    {
        if (myOntModel == null)
            return null;

        Set<String> foundPolicies = new HashSet<String>();
        
        // Create a vector with all supported policy classes
        Vector<String> policyClasses = new Vector<String>();
        policyClasses.add(PolicyConcepts.PosAuthorizationPolicy());
        policyClasses.add(PolicyConcepts.NegAuthorizationPolicy());
        policyClasses.add(PolicyConcepts.PosObligationPolicy());
        policyClasses.add(PolicyConcepts.NegObligationPolicy());

        // Iterate through the policy classes and check if instances of them exists in the model
        for (String currentPolicyClassName : policyClasses)
        {
            // Check if the given class is defined in the model
            OntClass currentClass = myOntModel.getOntClass(currentPolicyClassName);

            if (currentClass == null)
                continue;

            // The policy instances
            for (ExtendedIterator<? extends OntResource> policyIT = currentClass.listInstances(); policyIT.hasNext();)
            {
                String policyInstanceURI = policyIT.next().getURI();

                foundPolicies.add(policyInstanceURI);
            }
        }

        // Have not found policy
        return null;
    }
    
    /*
     * A method finding first policy in the ontology model
     * @return URI of the found policy instance
     */
    public static String findFirstPolicyInModel(SerializableOntModelImpl myOntModel)
    {
        if (myOntModel == null)
            return null;

        // Create a vector with all supported policy classes
        Vector<String> policyClasses = new Vector<String>();
        policyClasses.add(PolicyConcepts.PosAuthorizationPolicy());
        policyClasses.add(PolicyConcepts.NegAuthorizationPolicy());
        policyClasses.add(PolicyConcepts.PosObligationPolicy());
        policyClasses.add(PolicyConcepts.NegObligationPolicy());

        // Iterate through the policy classes and check if instances of them exists in the model
        for (String currentPolicyClassName : policyClasses)
        {
            // Check if the given class is defined in the model
            OntClass currentClass = myOntModel.getOntClass(currentPolicyClassName);

            if (currentClass == null)
                continue;

            // The policy instances
            for (ExtendedIterator<? extends OntResource> policyIT = currentClass.listInstances(); policyIT.hasNext();)
            {
                String policyInstanceURI = policyIT.next().getURI();

                return policyInstanceURI;
            }
        }

        // Have not found policy
        return null;
    }
}