package kaos.test;

import kaos.core.csi.AgentCapability;
import kaos.core.csi.AgentCapabilityImpl;
import kaos.core.csi.CSIFactory;
import kaos.core.csi.policy.KAoSSecurityException;
import kaos.core.csi.policy.PolicyManagement;
import kaos.core.csi.query.QueryRegistration;
import kaos.core.service.directory.KAoSAgentDescription;
import kaos.core.service.directory.KAoSEntityDescription;
import kaos.core.util.KAoSConstants;
import kaos.core.util.PolicyMsg;
import kaos.core.util.SubjectMsg;
import kaos.core.util.UniqueIdentifier;
import kaos.metrics.KAoSMetrics;
import kaos.metrics.MetricManager;
import kaos.ontology.util.KAoSOntClassBuilder;
import kaos.ontology.util.KAoSOntClassBuilderImpl;
import kaos.ontology.vocabulary.ActionConcepts;
import kaos.ontology.vocabulary.ActorConcepts;
import kaos.ontology.vocabulary.DefaultNamespaces;
import kaos.ontology.vocabulary.PolicyConcepts;
import kaos.policy.builder.KAoSPolicyBuilder;
import kaos.policy.builder.KAoSPolicyBuilderImpl;
import kaos.policy.builder.KAoSPolicyBuilderUtil;
import kaos.policy.builder.ObligationConstraintsBuilderImpl;
import kaos.test.ontology.TestOntologyConcepts;

import javax.swing.*;
import javax.swing.border.BevelBorder;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.List;

public class DatabasePerformanceTest
{
   private boolean DEBUG = false;
   private static final MetricManager _metricsManager = MetricManager.getInstance();
   Random random = new Random();

   private static int policyCounter = 0;
   private static int changeCounter = 0;
   private static int authorizationCounter = 0;
   private static int obligationCounter = 0;
   private HashMap<String, PolicyMsg> policies = new HashMap<String, PolicyMsg>();
   private final JComboBox senderCmb = new JComboBox();
   private final JComboBox receiverCmb = new JComboBox();
   private final JComboBox actorCmb = new JComboBox();
   private final JComboBox requestorCmb = new JComboBox();
   private final JTextField distanceTF = new JTextField(1);
   private final JComboBox loginCmb = new JComboBox();
   private JRadioButton _randomAuthPoliciesBtn;
   private JRadioButton _equalAuthPoliciesBtn;
   private JFrame _frame;

   // number of test actions in the ontology
   private static final int TEST_ACTION_COUNT = 4;
   private JComboBox _numberOfAuthorizationPoliciesCombo;
   private JComboBox _numberOfObligationPoliciesCombo;
   private JRadioButton _randomObligPoliciesBtn;
   private JRadioButton _equalObligPoliciesBtn;

   private enum AGENT_TYPES {HUMAN, ROBOT}

   private HashMap agents = new HashMap();

   public DatabasePerformanceTest()
   {
      buildGUI();
   }

   private void buildGUI()
   {
      // main panel
      JPanel panel = new JPanel();
      GridBagConstraints gbc = new GridBagConstraints();
      panel.setLayout(new GridBagLayout());
      panel.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
      Dimension dimension = new Dimension(1200, 300);
      panel.setPreferredSize(dimension);

      // layout components
      gbc.gridx = 0;
      gbc.gridy = 0;
      gbc.fill = GridBagConstraints.BOTH;
      gbc.weightx = 1;
      gbc.weighty = 0;
      gbc.gridwidth = 2;
      gbc.gridwidth = 1;

      // create an agent
      final JComboBox typeCmb = new JComboBox();
      typeCmb.addItem(AGENT_TYPES.HUMAN);
      typeCmb.addItem(AGENT_TYPES.ROBOT);

      final JTextField agentNameTF = new JTextField();
      final JTextField agentDomainTF = new JTextField();
      JButton createAnAgentBtn = new JButton("Create an agent");
      createAnAgentBtn.addActionListener(new ActionListener()
      {
         public void actionPerformed(ActionEvent ae)
         {
            try
            {
               createAnAgent((AGENT_TYPES) typeCmb.getSelectedItem(), agentNameTF.getText(), agentDomainTF.getText());
            }
            catch (Exception ex)
            {
               ex.printStackTrace();
            }
         }
      });

      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(createAnAgentBtn, gbc);
      gbc.gridx++;
      panel.add(typeCmb, gbc);
      gbc.gridx++;
      JLabel nameLbl = new JLabel("name: ");
      nameLbl.setHorizontalAlignment(JLabel.RIGHT);
      gbc.weightx = 0.5;
      panel.add(nameLbl, gbc);
      gbc.weightx = 1;
      gbc.gridx++;
      panel.add(agentNameTF, gbc);
      gbc.gridx++;
      JLabel domainLbl = new JLabel("domain: ");
      domainLbl.setHorizontalAlignment(JLabel.RIGHT);
      gbc.weightx = 0.5;
      panel.add(domainLbl, gbc);
      gbc.weightx = 1;
      gbc.gridx++;
      panel.add(agentDomainTF, gbc);

      // create team button
      JButton createActorBtn = new JButton("Create an actor");
      createActorBtn.addActionListener(new ActionListener()
      {
         public void actionPerformed(ActionEvent ae)
         {
            try
            {
               createAnAgent(AGENT_TYPES.ROBOT, "bot", "TeamA");
               _metricsManager.logMetric(KAoSMetrics.Guard_PolicyDB_Initialization);
            }
            catch (Exception ex)
            {
               ex.printStackTrace();
            }
         }
      });
      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(createActorBtn, gbc);

      // create team button
      JButton createTeamBtn = new JButton("Create a team");
      createTeamBtn.addActionListener(new ActionListener()
      {
         public void actionPerformed(ActionEvent ae)
         {
            try
            {
               createAnAgent(AGENT_TYPES.HUMAN, "bob", "TeamA");
               createAnAgent(AGENT_TYPES.ROBOT, "r1", "TeamA");
               createAnAgent(AGENT_TYPES.HUMAN, "larry", "TeamB");
               createAnAgent(AGENT_TYPES.ROBOT, "r2", "TeamB");
               createAnAgent(AGENT_TYPES.HUMAN, "sam", "TeamC");
               createAnAgent(AGENT_TYPES.ROBOT, "r3", "TeamC");
            }
            catch (Exception ex)
            {
               ex.printStackTrace();
            }
         }
      });
      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(createTeamBtn, gbc);

      // add policy
      _numberOfAuthorizationPoliciesCombo = new JComboBox();
      _numberOfAuthorizationPoliciesCombo.setEditable(true);
      initComboItems(_numberOfAuthorizationPoliciesCombo, false);

      JButton createPoliciesBtn = new JButton("Create authorization policies");
      createPoliciesBtn.addActionListener(new ActionListener()
      {
         public void actionPerformed(ActionEvent ae)
         {
            try
            {
               int n = (Integer) _numberOfAuthorizationPoliciesCombo.getSelectedItem();

               _metricsManager.logString(n + " policies");
               _metricsManager.reset();

               if (_randomAuthPoliciesBtn.isSelected())
               {
                  generateRandomAuthorizationPolicy(n);
               }
               else
               {
                  generateEvenlyDistributedAuthorizationPolicies(n);
               }
            }
            catch (Exception ex)
            {
               ex.printStackTrace();
            }
         }
      });
      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(createPoliciesBtn, gbc);
      gbc.gridx++;
      panel.add(_numberOfAuthorizationPoliciesCombo, gbc);

      JPanel radioBtnPanel = new JPanel();
      _randomAuthPoliciesBtn = new JRadioButton("Random", true);
      _equalAuthPoliciesBtn = new JRadioButton("Evenly distributed");
      _equalAuthPoliciesBtn.setToolTipText("An equal number of policies will be generated for each action type");
      ButtonGroup bg = new ButtonGroup();
      bg.add(_randomAuthPoliciesBtn);
      bg.add(_equalAuthPoliciesBtn);
      radioBtnPanel.add(_randomAuthPoliciesBtn);
      _randomAuthPoliciesBtn.addItemListener(new ItemListener()
      {
         public void itemStateChanged(ItemEvent e)
         {
            if (e.getStateChange() == ItemEvent.SELECTED)
            {
               initComboItems(_numberOfAuthorizationPoliciesCombo, false);
            }
         }
      });
      _equalAuthPoliciesBtn.addItemListener(new ItemListener()
      {
         public void itemStateChanged(ItemEvent e)
         {
            if (e.getStateChange() == ItemEvent.SELECTED)
            {
               initComboItems(_numberOfAuthorizationPoliciesCombo, true);
            }
         }
      });

      radioBtnPanel.add(_equalAuthPoliciesBtn);
      gbc.gridx++;
      panel.add(radioBtnPanel, gbc);

      _numberOfObligationPoliciesCombo = new JComboBox();
      _numberOfObligationPoliciesCombo.setEditable(true);
      initComboItems(_numberOfObligationPoliciesCombo, false);

      JButton createObligationPoliciesBtn = new JButton("Create obligation policies");
      createObligationPoliciesBtn.addActionListener(new ActionListener()
      {
         public void actionPerformed(ActionEvent ae)
         {
            try
            {
               int n = (Integer) _numberOfObligationPoliciesCombo.getSelectedItem();

               if (_randomObligPoliciesBtn.isSelected())
               {
                  _metricsManager.logString(n + " obligation policies");
                  _metricsManager.reset();

                  generateRandomObligation(n);

                  while (_metricsManager.getDataPointCount(KAoSMetrics.Guard_Add_Policy) < n)
                  {
                     Thread.sleep(1);
                  }

                  if ((n == 1) && (policyCounter == 1))
                  {
                     _metricsManager.logString("first policy");
                  }

                  if ((n == 1) && (policyCounter == 2))
                  {
                     _metricsManager.logString("second policy");
                  }

                  if ((n == 1) && (policyCounter == 3))
                  {
                     _metricsManager.logString("third policy");
                  }

                  _metricsManager.logMetric(KAoSMetrics.Guard_Add_Policy);
               }
               else
               {
                  if (n % TEST_ACTION_COUNT != 0)
                  {
                     JOptionPane.showMessageDialog(_frame, "Number of policies must be evenly divisible by number of actions (" + TEST_ACTION_COUNT + ")");

                     return;
                  }

                  int numOfEachAction = n / TEST_ACTION_COUNT;
                  String baseActionName = TestOntologyConcepts.TestAction();
                  for (int i = 1; i <= TEST_ACTION_COUNT; i++)
                  {
                     String actionName = baseActionName + "_" + i;
                     String propValue = TestOntologyConcepts.value_1();
                     for (int j = numOfEachAction - 1; j >= 0; j--)
                     {
                        // the lowest priority policy has a different property value from the other policies, so that we can test
                        // the performance of going through all of the policies to find the applicable one
                        if (j == 0)
                           propValue = TestOntologyConcepts.value_2();
                        generateObligationPolicy(ActorConcepts.Actor(), KAoSConstants.ACTOR_CLASS_SCOPE, actionName, propValue,
                                                 TestOntologyConcepts.TestAction, j);
                     }
                  }
               }

            }
            catch (Exception ex)
            {
               ex.printStackTrace();
            }
         }
      });
      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(createObligationPoliciesBtn, gbc);
      gbc.gridx++;
      panel.add(_numberOfObligationPoliciesCombo, gbc);

      JPanel obligRadioBtnPanel = new JPanel();
      _randomObligPoliciesBtn = new JRadioButton("Random", true);
      _equalObligPoliciesBtn = new JRadioButton("Evenly distributed");
      _equalObligPoliciesBtn.setToolTipText("An equal number of policies will be generated for each action type");
      ButtonGroup obligBg = new ButtonGroup();
      obligBg.add(_randomObligPoliciesBtn);
      obligBg.add(_equalObligPoliciesBtn);
      obligRadioBtnPanel.add(_randomObligPoliciesBtn);
      _randomObligPoliciesBtn.addItemListener(new ItemListener()
      {
         public void itemStateChanged(ItemEvent e)
         {
            if (e.getStateChange() == ItemEvent.SELECTED)
            {
               initComboItems(_numberOfObligationPoliciesCombo, false);
            }
         }
      });
      _equalObligPoliciesBtn.addItemListener(new ItemListener()
      {
         public void itemStateChanged(ItemEvent e)
         {
            if (e.getStateChange() == ItemEvent.SELECTED)
            {
               initComboItems(_numberOfObligationPoliciesCombo, true);
            }
         }
      });

      obligRadioBtnPanel.add(_equalObligPoliciesBtn);
      gbc.gridx++;
      panel.add(obligRadioBtnPanel, gbc);

      final JComboBox numberOfRemovedPoliciesCombo = new JComboBox();
      numberOfRemovedPoliciesCombo.setEditable(true);
      initComboItems(numberOfRemovedPoliciesCombo, false);

      JButton removePoliciesBtn = new JButton("Remove policies");
      removePoliciesBtn.addActionListener(new ActionListener()
      {
         public void actionPerformed(ActionEvent ae)
         {
            try
            {
               int n = (Integer) numberOfRemovedPoliciesCombo.getSelectedItem();

               _metricsManager.logString(n + " policies");
               _metricsManager.reset();

               for (int i = 0; i < n; i++)
               {
                  removePolicy();
               }

               while (_metricsManager.getDataPointCount(KAoSMetrics.Guard_Remove_Policy) < n)
               {
                  Thread.sleep(1);
               }

               _metricsManager.logMetric(KAoSMetrics.Guard_Remove_Policy);
            }
            catch (Exception ex)
            {
               ex.printStackTrace();
            }
         }
      });
      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(removePoliciesBtn, gbc);
      gbc.gridx++;
      panel.add(numberOfRemovedPoliciesCombo, gbc);

      final JComboBox numberOfChangedPolicies = new JComboBox();
      numberOfChangedPolicies.setEditable(true);
      initComboItems(numberOfChangedPolicies, false);
      JButton changePolicyBtn = new JButton("Change policy");
      changePolicyBtn.addActionListener(new ActionListener()
      {
         public void actionPerformed(ActionEvent ae)
         {
            try
            {
               int n = (Integer) numberOfChangedPolicies.getSelectedItem();

               _metricsManager.logString(n + " policies");
               _metricsManager.reset();

               for (int i = 0; i < n; i++)
               {
                  changeCounter++;

                  if (changeCounter < policyCounter)
                  {
                     changeProperty(changeCounter);
                  }
                  else
                  {
                     System.out.println("Not enough policies to update");
                     n = i - 1;
                     changeCounter--;

                     break;
                  }
               }

               while (_metricsManager.getDataPointCount(KAoSMetrics.Guard_Change_Policy) < n)
               {
                  Thread.sleep(1);
               }

               _metricsManager.logMetric(KAoSMetrics.Guard_Change_Policy);
            }
            catch (Exception ex)
            {
               ex.printStackTrace();
            }
         }
      });
      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(changePolicyBtn, gbc);
      gbc.gridx++;
      panel.add(numberOfChangedPolicies, gbc);

      final JComboBox numberOfTestActions = new JComboBox();
      numberOfTestActions.setEditable(true);
      initComboItems(numberOfTestActions, false);

      final JComboBox actionNameCombo = new JComboBox();
      actionNameCombo.addItem(TestOntologyConcepts.TestAction_1());
      actionNameCombo.addItem(TestOntologyConcepts.TestAction_2());
      actionNameCombo.addItem(TestOntologyConcepts.TestAction_3());
      actionNameCombo.addItem(TestOntologyConcepts.TestAction_4());

      final JComboBox propertyValueCombo = new JComboBox();
      propertyValueCombo.addItem("null");
      propertyValueCombo.addItem(TestOntologyConcepts.value_1());
      propertyValueCombo.addItem(TestOntologyConcepts.value_2());

      // Test Action
      final JButton testActionBtn = new JButton("Perform TestAction");
      testActionBtn.addActionListener(new ActionListener()
      {
         public void actionPerformed(ActionEvent ae)
         {
            // get selected perform
            TestAgent agent = (TestAgent) agents.get(actorCmb.getSelectedItem());

            int n = (Integer) numberOfTestActions.getSelectedItem();

            _metricsManager.logString(n + " policies");
            _metricsManager.reset();

            for (int i = 0; i < n; i++)
            {
               String actionName = (String) actionNameCombo.getSelectedItem();
               String propertyValue = (String) propertyValueCombo.getSelectedItem();
               if (propertyValue.equals("null"))
                  propertyValue = null;

               long startTime = System.currentTimeMillis();
               try
               {
                  agent.TestAction(actionName, propertyValue);
                  testActionBtn.setBackground(Color.green);
               }
               catch (Exception ex)
               {
                  if (ex instanceof KAoSSecurityException)
                     testActionBtn.setBackground(Color.red);
                  else
                     ex.printStackTrace();
               }

               long stopTime = System.currentTimeMillis();
               _metricsManager.add(KAoSMetrics.CHECK_PERMISSION, (stopTime - startTime));
            }

            _metricsManager.logString(n + " actions");
            _metricsManager.logMetric(KAoSMetrics.CHECK_PERMISSION);


            while (_metricsManager.getDataPointCount(KAoSMetrics.Guard_Get_Sorted_Authorization) < n)
            {
               try
               {
                  Thread.sleep(1);
               }
               catch (InterruptedException e)
               {
                  e.printStackTrace();
               }
            }

            _metricsManager.logMetric(KAoSMetrics.Guard_Get_Sorted_Authorization);
         }
      });
      gbc.weightx = 1;
      gbc.gridx = 0;
      gbc.gridy++;
      panel.add(testActionBtn, gbc);
      gbc.gridx++;
      panel.add(numberOfTestActions, gbc);

      JLabel actorLbl = new JLabel("actor: ");
      actorLbl.setHorizontalAlignment(JLabel.RIGHT);
      gbc.gridx++;
      panel.add(actorLbl, gbc);
      gbc.gridx++;
      panel.add(actorCmb, gbc);

      JLabel actionLbl = new JLabel("action: ");
      actionLbl.setHorizontalAlignment(JLabel.RIGHT);
      gbc.gridx = 2;
      gbc.gridy++;
      panel.add(actionLbl, gbc);
      gbc.gridx++;
      panel.add(actionNameCombo, gbc);

      JLabel propertyLbl = new JLabel("property: ");
      propertyLbl.setHorizontalAlignment(JLabel.RIGHT);
      gbc.gridx = 2;
      gbc.gridy++;
      panel.add(propertyLbl, gbc);
      gbc.gridx++;
      panel.add(propertyValueCombo, gbc);

      // end section for state controls

      JFrame f = new JFrame("Database Performance Test Manager");
      f.addWindowListener(new WindowAdapter()
      {
         public void windowClosing(WindowEvent e)
         {
            System.exit(0);
         }
      });

      f.getContentPane().add(new JScrollPane(panel), BorderLayout.CENTER);
      f.pack();
      f.setVisible(true);
      _frame = f;
   }

   private void initComboItems(JComboBox box, boolean basedOnActionCount)
   {
      box.removeAllItems();

      if (basedOnActionCount)
      {
         box.addItem(1);
         int num = TEST_ACTION_COUNT;
         for (int i = 0; i < 10; i++)
         {
            box.addItem(num);
            num *= 2;
         }
      }
      else
      {
         box.addItem(1);
         box.addItem(10);
         box.addItem(20);
         box.addItem(30);
         box.addItem(40);
         box.addItem(50);
         box.addItem(100);
      }
   }

   private void createAnAgent(AGENT_TYPES type, String name, String domain)
   {
      // add domain to a Vector
      Vector domains = new Vector();
      domains.add(domain);

      // determine the type of agent requested and create
      TestAgent agent = null;
      try
      {
         if (type == AGENT_TYPES.HUMAN)
         {
            agent = new TestAgent(name, domains, ActorConcepts.Human());
         }
         else if (type == AGENT_TYPES.ROBOT)
         {
            agent = new TestAgent(name, domains, ActorConcepts.Robot());
         }
         else
         {
            System.err.println("type not valid: " + type);

            return;
         }
      }
      catch (Throwable e)
      {
         e.printStackTrace();
      }

      // register agent
      try
      {
         agent.registerWithKAoS();
      }
      catch (Exception ex)
      {
         ex.printStackTrace();
      }

      // track info in GUI
      agents.put(name, agent);
      senderCmb.addItem(name);
      receiverCmb.addItem(name);
      actorCmb.addItem(name);
      requestorCmb.addItem(name);
      loginCmb.addItem(name);
   }

   private List<KAoSAgentDescription> queryForAgents(String type, String capability)
   {
      // get query instance
      QueryRegistration query = null;
      try
      {
         query = CSIFactory.getQueryRegistration();
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }

      // get agents matching class
      Hashtable searchAttributes = new Hashtable();
      String key = KAoSEntityDescription.ENTITY_ONTOLOGICAL_TYPE_NAMES;
      Vector ontNames = new Vector();
      ontNames.add(type);
      searchAttributes.put(key, ontNames);
      List<KAoSAgentDescription> typeResult = null;
      try
      {
         typeResult = query.getActorsWithAttributes(searchAttributes);
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }

      // get agenst matching capability
      if (capability.equals("ANY"))
         return typeResult;

      key = KAoSAgentDescription.AGENT_CAPABILITIES;
      AgentCapability ac = new AgentCapabilityImpl();
      ac.setName(capability);
      ArrayList capabilities = new ArrayList();
      capabilities.add(ac);
      searchAttributes.put(key, capabilities);
      List<KAoSAgentDescription> capabilityResult = null;
      try
      {
         capabilityResult = query.getActorsWithAttributes(searchAttributes);
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }

      // merge results
      List<KAoSAgentDescription> queryResult = new ArrayList<KAoSAgentDescription>();
      for (KAoSAgentDescription desc : typeResult)
      {
         for (KAoSAgentDescription desc2 : capabilityResult)
         {
            if (desc2.getAgentNickname().equals(desc.getAgentNickname()))
            {
               queryResult.add(desc);
            }
         }
      }

      return queryResult;
   }


   public void generateRandomAuthorizationPolicy(int numberOfPolicies)
   {
      if (numberOfPolicies < 1)
         return;

      for (int i = 0; i < numberOfPolicies; i++)
      {
         int policyPriority = generateRandomPriority();
         String subjectScope = getRandomScope();
         String subjectID = null;
         if (subjectScope == KAoSConstants.ACTOR_CLASS_SCOPE)
         {
            subjectID = generateRandomSubjectClass();
         }
         else
         {
            KAoSAgentDescription agentDescription = null;
            while (agentDescription == null)
            {
               String subjectClass = generateRandomSubjectClass();
               agentDescription = generateRandomInstanceFromClass(subjectClass);
            }

            subjectID = agentDescription.getEntityNameAsString();
         }

         // create the controls action class
         String action = generateRandomAction();
         String propertyValue = generateRandomProperty();

         generateAuthorizationPolicy(subjectID, subjectScope, action, propertyValue, policyPriority);
      }
   }

   public void generateEvenlyDistributedAuthorizationPolicies(int numberOfPolicies)
   {
      if (numberOfPolicies < 1)
         return;

      // Evenly distribute the number of policies among the actions
      // Each policy is the same, but the priority differs
      // The policy of lowest priority has a different property value than the other policies of higher priority
      // This allows testing the process of going through all of the policies until you find the final one that matches (e.g. worst case result)
      if (numberOfPolicies % TEST_ACTION_COUNT != 0)
      {
         JOptionPane.showMessageDialog(_frame, "Number of policies must be evenly divisible by number of actions (" + TEST_ACTION_COUNT + ")");

         return;
      }

      int numOfEachAction = numberOfPolicies / TEST_ACTION_COUNT;
      String baseActionName = TestOntologyConcepts.TestAction();
      for (int i = 1; i <= TEST_ACTION_COUNT; i++)
      {
         String actionName = baseActionName + "_" + i;
         String propValue = TestOntologyConcepts.value_1();
         for (int j = numOfEachAction - 1; j >= 0; j--)
         {
            // the lowest priority policy has a different property value from the other policies, so that we can test
            // the performance of going through all of the policies to find the applicable one
            if (j == 0)
               propValue = TestOntologyConcepts.value_2();
            generateAuthorizationPolicy(ActorConcepts.Actor(), KAoSConstants.ACTOR_CLASS_SCOPE, actionName, propValue, j);
         }
      }
   }

   public void generateAuthorizationPolicy(String subject, String subjectScope, String action, String propValue, int priority)
   {
      try
      {
         // create policy id and description
         policyCounter++;
         System.out.println("creating policy " + policyCounter);
         String policyName = "Policy_" + policyCounter;
         String policyDescription = "This is the " + policyCounter + " policy ";
         String policyID = KAoSPolicyBuilderUtil.POLICY_ID_PREFIX + UniqueIdentifier.GenerateUID();

         KAoSOntClassBuilder controls = new KAoSOntClassBuilderImpl(KAoSPolicyBuilderUtil.generateClassNameForPolicy(action,
                                           DefaultNamespaces.controlActionPostfix, policyID));
         controls.setMainSuperClass(action);

         // add actor
         if (subjectScope.equals(KAoSConstants.AGENT_SCOPE))
         {
            controls.addPropertyRangeInstance(ActionConcepts.performedBy(), subject);
         }
         else
         {
            controls.setPropertyRangeClass(ActionConcepts.performedBy(), subject);
         }

         // set property
         if (propValue != null)
         {
            controls.addPropertyRangeInstance(TestOntologyConcepts.hasTestProperty(), propValue);
         }

         // create the damlPolicy
         KAoSPolicyBuilder damlPolicyBuilder = new KAoSPolicyBuilderImpl();
         String modality = PolicyConcepts.NegAuthorizationPolicy();
         damlPolicyBuilder.setPolicyIDAndModalityType(policyID, modality);

         // Set Policy Name
         damlPolicyBuilder.setPolicyName(policyName);

         // Set Priority
         damlPolicyBuilder.setPriority(priority);

         // add the controls action to the damlPolicy
         damlPolicyBuilder.setControlActionClass(controls);

         // create the PolicyMsg and add the damlPolicy as an attribute
         SubjectMsg subjectMsg = new SubjectMsg(subject, subject, subjectScope);
         Vector subjects = new Vector();
         subjects.addElement(subjectMsg);
         PolicyMsg policyMsg = new PolicyMsg(policyID, policyName, policyDescription, action, "", subjects, true);
         policyMsg.setModality(modality);
         policyMsg.setPriority(priority);
         policyMsg.setOntologyPolicy(damlPolicyBuilder.getOWL());

         policies.put(policyMsg.getName(), policyMsg);

         PolicyManagement pm = CSIFactory.getPolicyManagement();
         pm.addPolicy(policyMsg);
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }
   }

   private int generateRandomPriority()
   {
      int priority = random.nextInt(5);
      priority++;

      return priority;
   }

   private String getRandomScope()
   {
      String subjectScope = KAoSConstants.ACTOR_CLASS_SCOPE;
      boolean classScope = random.nextBoolean();
      if (!classScope)
         subjectScope = KAoSConstants.AGENT_SCOPE;

      return subjectScope;
   }

   private String generateRandomSubjectClass()
   {
      List<String> domains = queryForDomains();
      domains.add(ActorConcepts.Human());
      domains.add(ActorConcepts.Robot());

      int index = random.nextInt(domains.size());

      return domains.get(index);
   }

   private KAoSAgentDescription generateRandomInstanceFromClass(String className)
   {
      List<KAoSAgentDescription> agentsOfClass = null;
      if (className.contains("KAoSGroup"))
      {
         agentsOfClass = queryForAgentsInDomain(className);
         System.out.println("no members in domain " + className);
      }
      else
      {
         agentsOfClass = queryForAgentsOfType(className);
         System.out.println("no members found for " + className);
      }

      if ((agentsOfClass == null) || (agentsOfClass.size() == 0))
      {
         return null;
      }

      System.out.println("found " + agentsOfClass.size() + " " + className);
      int actionID = random.nextInt(agentsOfClass.size());

      return agentsOfClass.get(actionID);
   }

   private String generateRandomAction()
   {
      String action = TestOntologyConcepts.TestAction();
      int actionID = random.nextInt(4);
      if (actionID == 0)
         action = TestOntologyConcepts.TestAction_1();
      if (actionID == 1)
         action = TestOntologyConcepts.TestAction_2();
      if (actionID == 2)
         action = TestOntologyConcepts.TestAction_3();
      if (actionID == 3)
         action = TestOntologyConcepts.TestAction_4();

      return action;
   }

   private String generateRandomProperty()
   {
      String property = null;
      int actionID = random.nextInt(3);
      if (actionID == 1)
         property = TestOntologyConcepts.value_1();
      if (actionID == 2)
         property = TestOntologyConcepts.value_2();

      return property;
   }

   public void generateRandomObligation(int numberOfPolicies)
   {
      if (numberOfPolicies < 1)
         return;

      for (int i = 0; i < numberOfPolicies; i++)
      {
         int policyPriority = generateRandomPriority();
         String subjectScope = getRandomScope();
         String subjectID = null;
         if (subjectScope == KAoSConstants.ACTOR_CLASS_SCOPE)
         {
            subjectID = generateRandomSubjectClass();
         }
         else
         {
            KAoSAgentDescription agentDescription = null;
            while (agentDescription == null)
            {
               String subjectClass = generateRandomSubjectClass();
               agentDescription = generateRandomInstanceFromClass(subjectClass);
            }

            subjectID = agentDescription.getEntityNameAsString();
         }

         // create the controls action class
         String triggerAction = generateRandomAction();
         String triggerPropValue = generateRandomProperty();
         String obligatedAction = generateRandomAction();
         generateObligationPolicy(subjectID, subjectScope, triggerAction, triggerPropValue, obligatedAction, policyPriority);
      }
   }

   private List<KAoSAgentDescription> queryForAgentsInDomain(String domain)
   {
      // get query instance
      QueryRegistration query = null;
      try
      {
         query = CSIFactory.getQueryRegistration();
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }

      List<KAoSAgentDescription> domainMembers = null;
      try
      {
         domainMembers = query.getGroupMembers(domain);
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }

      return domainMembers;
   }

   private List<KAoSAgentDescription> queryForAgentsOfType(String type)
   {
      // get query instance
      QueryRegistration query = null;
      try
      {
         query = CSIFactory.getQueryRegistration();
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }

      // get agents matching class
      Hashtable searchAttributes = new Hashtable();
      String key = KAoSEntityDescription.ENTITY_ONTOLOGICAL_TYPE_NAMES;
      Vector ontNames = new Vector();
      ontNames.add(type);
      searchAttributes.put(key, ontNames);
      List<KAoSAgentDescription> typeResult = null;
      try
      {
         typeResult = query.getActorsWithAttributes(searchAttributes);
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }

      return typeResult;
   }

   private List<String> queryForDomains()
   {
      // get query instance
      List<String> domainClasses = new ArrayList<String>();
      try
      {
         QueryRegistration query = CSIFactory.getQueryRegistration();
         List<String> domains = query.getRootDomains();
         for (String domain : domains)
         {
            if (!domain.contains("Policy"))
            {
               String domainClass = domain.replace("#", "Group#MembersOfDomain");
               domainClasses.add(domainClass);
               System.out.println("added " + domainClass);
            }
         }
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }

      return domainClasses;
   }

   public void generateObligationPolicy(String subjectID, String subjectScope, String triggerAction, String triggerPropValue, String obligatedAction,
           int priority)
   {
      // create policy id and description
      policyCounter++;
      System.out.println("creating policy " + policyCounter);
      String policyName = "Policy_" + policyCounter;
      String policyDescription = "This is the " + policyCounter + " policy ";

      PolicyMsg policyMsg = createObligationPolicy(policyName, policyDescription, priority, subjectScope, subjectID, subjectID, obligatedAction, triggerAction,
                               triggerPropValue, PolicyConcepts.PosObligationPolicy());
      try
      {
         policies.put(policyMsg.getName(), policyMsg);

         PolicyManagement pm = CSIFactory.getPolicyManagement();
         pm.addPolicy(policyMsg);
      }
      catch (Exception xcp)
      {
         xcp.printStackTrace();
      }
   }

   public PolicyMsg createObligationPolicy(String policyName, String policyDescription, int policyPriority, String subjectScope, String subjectID,
           String subjectName, String action, String triggerAction, String triggerPropValue, String modality)
   {
      // create policy id
      String policyID = KAoSPolicyBuilderUtil.POLICY_ID_PREFIX + UniqueIdentifier.GenerateUID();

      try
      {
         // create the controls action class
         KAoSOntClassBuilder controls = new KAoSOntClassBuilderImpl(KAoSPolicyBuilderUtil.generateClassNameForPolicy(action,
                                           DefaultNamespaces.controlActionPostfix, policyID));

         controls.setMainSuperClass(action);

         // Set Actor Class or Agent Instance for trigger
         if (subjectScope.equals(KAoSConstants.ACTOR_CLASS_SCOPE))
         {
            controls.setPropertyRangeClass(ActionConcepts.performedBy(), subjectID);
         }
         else
         {
            controls.addPropertyRangeInstance(ActionConcepts.performedBy(), subjectID);
         }

         // create the damlPolicy
         KAoSPolicyBuilder damlPolicyBuilder = new KAoSPolicyBuilderImpl();

         // Set Id and Modality
         damlPolicyBuilder.setPolicyIDAndModalityType(policyID, PolicyConcepts.PosObligationPolicy());

         // Set Policy Name
         damlPolicyBuilder.setPolicyName(policyName);

         // Set Priority
         damlPolicyBuilder.setPriority(policyPriority);

         // add trigger
         KAoSOntClassBuilder trigger = new KAoSOntClassBuilderImpl(KAoSPolicyBuilderUtil.generateClassNameForPolicy(triggerAction,
                                          DefaultNamespaces.controlActionPostfix, policyID));
         trigger.setMainSuperClass(triggerAction);

         // Set Actor Class or Agent Instance for trigger
         if (subjectScope.equals(KAoSConstants.ACTOR_CLASS_SCOPE))
         {
            trigger.setPropertyRangeClass(ActionConcepts.performedBy(), subjectID);
         }
         else
         {
            trigger.addPropertyRangeInstance(ActionConcepts.performedBy(), subjectID);
         }

         trigger.addPropertyRangeInstance(TestOntologyConcepts.hasTestProperty(), triggerPropValue);

         damlPolicyBuilder.setTriggerActionClass(trigger);

         // ad obligation constraint
         String baseClass = PolicyConcepts.FinishBeforeRefActionStart();
         String className = KAoSPolicyBuilderUtil.generateClassNameForPolicy(baseClass, DefaultNamespaces.obligationConstraintPostfix, policyID);
         ObligationConstraintsBuilderImpl builder = new ObligationConstraintsBuilderImpl(className);
         try
         {
            builder.setMainSuperClass(baseClass);
         }
         catch (Exception xcp)
         {
            xcp.printStackTrace();
         }

         damlPolicyBuilder.setObligationConstraints(builder);

         // add the controls action to the damlPolicy
         damlPolicyBuilder.setControlActionClass(controls);

         // create the PolicyMsg and add the damlPolicy as an attribute
         SubjectMsg subjectMsg = new SubjectMsg(subjectID, subjectName, subjectScope);
         Vector subjects = new Vector();
         subjects.addElement(subjectMsg);
         PolicyMsg policyMsg = new PolicyMsg(policyID, policyName, policyDescription, triggerAction, "",    // admin
            subjects, true);
         policyMsg.setModality(modality);
         policyMsg.setPriority(policyPriority);
         policyMsg.setOntologyPolicy(damlPolicyBuilder.getOWL());

         return policyMsg;
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }

      return null;
   }

   public void changeProperty(int policyNumber)
   {
      System.out.println("changing property of " + policyNumber);
      String policyName = "Policy_" + policyNumber;
      PolicyMsg policyMsg = policies.get(policyName);

      try
      {
         // create the controls action class
         KAoSOntClassBuilder controls = new KAoSOntClassBuilderImpl(policyMsg.getId());

         controls.setMainSuperClass(policyMsg.getType());

         // create the damlPolicy
         KAoSPolicyBuilder damlPolicyBuilder = new KAoSPolicyBuilderImpl();

         // Set Id and Modality
         damlPolicyBuilder.setPolicyIDAndModalityType(policyMsg.getId(), PolicyConcepts.NegAuthorizationPolicy());

         // Set Policy Name
         damlPolicyBuilder.setPolicyName(policyName);

         // Set Priority
         damlPolicyBuilder.setPriority(policyMsg.getPriority());

         // Set Actor Class or Agent Instance
         controls.setPropertyRangeClass(ActionConcepts.hasDestination(), ActorConcepts.Human());

         // add the controls action to the damlPolicy
         damlPolicyBuilder.setControlActionClass(controls);

         // create the PolicyMsg and add the damlPolicy as an attribute
         SubjectMsg subjectMsg = new SubjectMsg(ActorConcepts.Robot(), ActorConcepts.Robot(), KAoSConstants.ACTOR_CLASS_SCOPE);
         Vector subjects = new Vector();
         subjects.addElement(subjectMsg);
         PolicyMsg policyMsg2 = new PolicyMsg(policyMsg.getId(), policyMsg.getName(), policyMsg.getDescription(), policyMsg.getType(), "", subjects, true);
         policyMsg2.setModality(policyMsg.getModality());
         policyMsg2.setPriority(policyMsg.getPriority());
         policyMsg2.setOntologyPolicy(damlPolicyBuilder.getOWL());


         PolicyManagement pm = CSIFactory.getPolicyManagement();
         pm.changePolicy(policyMsg2);
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }
   }

   public void removePolicy()
   {
      System.out.println("removing policy " + policyCounter);
      String policyName = "Policy_" + policyCounter;
      String policyID = policies.get(policyName).getId();

      try
      {
         PolicyManagement pm = CSIFactory.getPolicyManagement();
         pm.removePolicy(policyID);
         policyCounter--;
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }
   }


   public static void main(String[] args) throws Exception
   {
      DatabasePerformanceTest demo = new DatabasePerformanceTest();
   }
}
