package kaos.test;

import kaos.core.csi.*;
import kaos.core.csi.policy.ObligationActionDescription;
import kaos.core.csi.policy.PolicyChecking;
import kaos.core.util.NamespaceValidator;
import kaos.ontology.vocabulary.ActionConcepts;
import kaos.ontology.vocabulary.ActorConcepts;
import kaos.policy.builder.KAoSPolicyBuilderUtil;
import kaos.test.ontology.TestOntologyConcepts;

import java.io.Serializable;
import java.lang.reflect.Method;
import java.net.URISyntaxException;
import java.util.*;

//import kaos.robots.ontology.TeleoperationConcepts;

public class TestAgent extends KAoSActorImpl
{
   private String ontologicalType = ActorConcepts.Actor();

   public TestAgent(String[] args) throws Exception
   {
      super(args);
      initialize();
   }

   public TestAgent(String name, Vector domainNames) throws Exception
   {
      super(name, domainNames);
      initialize();
   }

   public TestAgent(String name, Vector domainNames, String ontologicalType) throws Exception
   {
      super(name, domainNames);
      this.ontologicalType = ontologicalType;
      initialize();
   }


   private void initialize() throws Exception
   {
      // modify default agent description
      setEntityOntologicalType();
      List<AgentCapability> capabilites = getAgentCapabilities();
      _actorDesc.setCapabilities(capabilites);

      // register agent
      super.registerWithKAoS();
   }

   /**
    *  define the type of entity the agent is
    */
   protected void setEntityOntologicalType()
   {
      addOntologicalType(ontologicalType);
   }

   protected List getActorProperties()
   {
      return null;
   }

   public void registerWithKAoS() throws Exception
   {
      List properties = getActorProperties();
      if (properties == null)
      {
         super.registerWithKAoS();
      }
      else
      {
         super.registerWithKAoS(getActorProperties());
      }
   }

   /**
    * define properties that describe what capabilities the agent has
    */
   protected List<AgentCapability> getAgentCapabilities()
   {
      return new ArrayList<AgentCapability>();
   }

   public void receiveMessage(Serializable messageContent, KAoSActor sender)
   {
      System.out.println(_actorDesc.getAgentNickname() + " received message from " + sender.getName() + ": " + messageContent);
   }


   public ActionInstanceDescription buildActionInstanceDescription(String action, HashMap properties)
   {
      ActionInstanceDescription aid = null;
      try
      {
         if (properties == null)
            aid = new ActionInstanceDescriptionImpl(null, action, NamespaceValidator.validateURI(this.getName()));
         else
            aid = new ActionInstanceDescriptionImpl(null, action, NamespaceValidator.validateURI(this.getName()), properties);
      }
      catch (URISyntaxException ex)
      {
         ex.printStackTrace();
      }

      return aid;
   }

   protected void enforcePolicies(ActionInstanceDescription aid) throws Exception
   {
      PolicyChecking policyChecking = CSIFactory.getPolicyChecking();
      List<ObligationActionDescription> obligations = policyChecking.findPolicyDecision(aid, null);
      for (ActionInstanceDescription action : obligations)
      {
         performAction(action);
      }
   }

   private void performAction(ActionInstanceDescription aid) throws Exception
   {
      // get action name from aid
      String actionName = aid.getActionClassName();

      // get short name from fully qualified name
      String methodName = KAoSPolicyBuilderUtil.getShortName(actionName);

      Map<String, OntPropertyDescription> properties = aid.getPropertiesMap();
      Object[] argObjects = {properties};

      // find method in class
      Method methods[] = this.getClass().getMethods();
      for (int i = 0; i < methods.length; i++)
      {
         if (methods[i].getName().equals(methodName))
         {
            // System.out.println("found method match");
            // if found
            // check number of args
            Class[] parameters = methods[i].getParameterTypes();
            if (parameters.length != argObjects.length)
            {
               // System.out.println("\t but number of args does not match");
               continue;
            }

            // check parameter type
            boolean match = true;
            for (int j = 0; j < parameters.length; j++)
            {
               if (!parameters[j].isInstance(argObjects[j]))
               {
                  // System.out.println("\t but arg tpes don't match");
                  match = false;

                  break;
               }
            }

            if (!match)
               continue;
            else
            {
               // System.out.println("\t arg number and type seem OK");
               // execute and return
               methods[i].invoke(this, argObjects);

               return;
            }
         }
      }

      // System.out.println("Did not find method for " + methodName);
   }

   public void TestAction(String testActionName, String propertyValue) throws Exception
   {
      HashMap properties = new HashMap();
      properties.put(ActionConcepts.performedBy(), this._actorDesc.getEntityNameAsString());

      if (propertyValue != null)
      {
         properties.put(TestOntologyConcepts.hasTestProperty(), propertyValue);
      }

      ActionInstanceDescription aid = buildActionInstanceDescription(testActionName, properties);

      // perform enforcement
      enforcePolicies(aid);

      // perform move
      System.out.println(_actorDesc.getAgentNickname() + ": " + testActionName + " with property " + propertyValue);
   }


   public static void main(String[] args) throws Exception
   {
      // create agent
      TestAgent agent = new TestAgent(args);

      // wait around until terminated
      synchronized (Thread.currentThread())
      {
         Thread.currentThread().wait();
      }

   }
}
