package kaos.tools.webServiceTool;

import javax.swing.text.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

// @author S. Kumar T - santhosh@in.fiorano.com
/**
 * @version: $Revision: 1.2 $
 */

public abstract class AutoCompleter
{
    public AutoCompleter (JTextComponent comp)
    {
        _list = new JList();
        _popup = new JPopupMenu();
        _textComp = comp;
        _textComp.putClientProperty(AUTOCOMPLETER, this);
        JScrollPane scroll = new JScrollPane(_list);
        scroll.setBorder(null);

        _list.setFocusable( false );
        scroll.getVerticalScrollBar().setFocusable( false );
        scroll.getHorizontalScrollBar().setFocusable( false );

        _popup.setBorder(BorderFactory.createLineBorder(Color.black));
        _popup.add(scroll);

        if(_textComp instanceof JTextField){
            _textComp.registerKeyboardAction(showAction, KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, 0), JComponent.WHEN_FOCUSED);
            _textComp.getDocument().addDocumentListener(documentListener);
        }
        else {
            _textComp.registerKeyboardAction(showAction, KeyStroke.getKeyStroke(KeyEvent.VK_SPACE, KeyEvent.CTRL_MASK), JComponent.WHEN_FOCUSED);
        }
        _textComp.registerKeyboardAction(upAction, KeyStroke.getKeyStroke(KeyEvent.VK_UP, 0), JComponent.WHEN_FOCUSED);
        _textComp.registerKeyboardAction(hidePopupAction, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), JComponent.WHEN_FOCUSED);

        _popup.addPopupMenuListener(new PopupMenuListener(){
            public void popupMenuWillBecomeVisible(PopupMenuEvent e){
            }

            public void popupMenuWillBecomeInvisible(PopupMenuEvent e){
                _textComp.unregisterKeyboardAction(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0));
            }

            public void popupMenuCanceled(PopupMenuEvent e){
            }
        });
        _list.setRequestFocusEnabled(false);
    }

    static Action acceptAction = new AbstractAction(){
        public void actionPerformed(ActionEvent e){
            JComponent tf = (JComponent)e.getSource();
            AutoCompleter completer = (AutoCompleter)tf.getClientProperty(AUTOCOMPLETER);
            completer._popup.setVisible(false);

            completer.acceptedListItem((String)completer._list.getSelectedValue());
        }
    };

    DocumentListener documentListener = new DocumentListener(){
        public void insertUpdate(DocumentEvent e){
            showPopup();
        }

        public void removeUpdate(DocumentEvent e){
            showPopup();
        }

        public void changedUpdate(DocumentEvent e){}
    };

    private void showPopup()
    {
        _popup.setVisible(false);
        if(_textComp.isEnabled() && updateListData() && _list.getModel().getSize()!=0){
            if(!(_textComp instanceof JTextField)) {
                _textComp.getDocument().addDocumentListener(documentListener);
            }
            _textComp.registerKeyboardAction(acceptAction, KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), JComponent.WHEN_FOCUSED);
            int size = _list.getModel().getSize();
            _list.setVisibleRowCount(size<10 ? size : 10);

            int x = 0;
            try {
                int pos = Math.min (_textComp.getCaret().getDot(), _textComp.getCaret().getMark());
                x = _textComp.getUI().modelToView(_textComp, pos).x;
            }
            catch(BadLocationException e){
                // this should never happen!!!
                e.printStackTrace();
            }
            _popup.show(_textComp, x, _textComp.getHeight());
        }
        else {
            _popup.setVisible(false);
        }
        _textComp.requestFocus();
    }

    static Action showAction = new AbstractAction(){
        public void actionPerformed(ActionEvent e){
            JComponent tf = (JComponent)e.getSource();
            AutoCompleter completer = (AutoCompleter)tf.getClientProperty(AUTOCOMPLETER);
            if(tf.isEnabled()){
                if(completer._popup.isVisible()) {
                    completer.selectNextPossibleValue();
                }
                else {
                    completer.showPopup();
                }
            }
        }
    };

    static Action upAction = new AbstractAction(){
        public void actionPerformed(ActionEvent e){
            JComponent tf = (JComponent)e.getSource();
            AutoCompleter completer = (AutoCompleter)tf.getClientProperty(AUTOCOMPLETER);
            if(tf.isEnabled()){
                if(completer._popup.isVisible()) {
                    completer.selectPreviousPossibleValue();
                }
            }
        }
    };

    static Action hidePopupAction = new AbstractAction(){
        public void actionPerformed(ActionEvent e){
            JComponent tf = (JComponent)e.getSource();
            AutoCompleter completer = (AutoCompleter)tf.getClientProperty(AUTOCOMPLETER);
            if(tf.isEnabled()) {
                completer._popup.setVisible(false);
            }
        }
    };

    /**
     * Selects the next item in the list.  It won't change the selection if the
     * currently selected item is already the last item.
     */
    protected void selectNextPossibleValue()
    {
        int si = _list.getSelectedIndex();
        if (si < _list.getModel().getSize() - 1) {
            _list.setSelectedIndex(si + 1);
            _list.ensureIndexIsVisible(si + 1);
        }
    }

    /**
     * Selects the previous item in the list.  It won't change the selection if the
     * currently selected item is already the first item.
     */
    protected void selectPreviousPossibleValue()
    {
        int si = _list.getSelectedIndex();
        if(si > 0) {
            _list.setSelectedIndex(si - 1);
            _list.ensureIndexIsVisible(si - 1);
        }
    }

    //Variables
    private static final String AUTOCOMPLETER = "AUTOCOMPLETER";
    protected JList _list;
    protected JPopupMenu _popup;
    protected JTextComponent _textComp;

    protected abstract boolean updateListData();
    protected abstract void acceptedListItem(String selected); 
}
