package kaos.tools.webServiceTool;

import kaos.kpat.applet.ExampleFileFilter;

import javax.swing.*;
import java.util.Hashtable;
import java.util.Map;
import java.util.Iterator;
import java.util.Vector;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.File;

/**
 * Builds the XSL file frm WSDL
 *
 * @author Maggie Breedy
 * @version: $Revision 1.2$
 *
 * Date: Aug 31, 2010
 * Time: 2:16:40 PM
 */
public class BuildXSLFile
{
    public BuildXSLFile (Map<String, ServicesObject> wsdlFileInfoMap, String servName, String filePath)
    {
        _wsdlFileInfoMap = wsdlFileInfoMap;
        _servName = servName;

        if (filePath.equals("")) {
            _filePath = findAnotherPath();
        }
        else {
            File f = new File (filePath);
            File f2 = new File (f.getParent());
            File f3 = new File (f2.getParent());
            File f4 = new File (f3.getParent());
            if (f4.exists()) {
                //System.out.println ("**XSL file location: " + f4);
                _filePath = f3.getAbsolutePath();
            }
            else {
                _filePath = findAnotherPath();
            }
        }
    }

    /**
     * Find the directory path for saving xsl files
     *
     * @return string of the selected path
     */
    protected String findAnotherPath()
    {
        JFileChooser chooser = new JFileChooser (".");
        chooser.setFileSelectionMode (JFileChooser.DIRECTORIES_ONLY); 
        ExampleFileFilter configFilter = new ExampleFileFilter();
        configFilter.addExtension ("xsl");
        configFilter.setDescription ("File in XSL");
        chooser.addChoosableFileFilter (configFilter);
        chooser.setFileFilter (configFilter);
        chooser.setDialogTitle ("Please choose path for XSL files");
        int returnVal = chooser.showSaveDialog (new JFrame());
        if (returnVal != JFileChooser.APPROVE_OPTION) {
            return null;
        }
        chooser.setName ("Choose path for XSL files");
        return chooser.getSelectedFile().getPath();
    }

    /**
     * Created the XSL file from data
     */
    public void getXSLDataForFile()
    {
        //write the file header
        String props = null;
        String serviceName = "";
        if (_servName.contains (".wsdl")) {
           //System.out.println ("**XSL_servName: " + _servName);
           String[] array = _servName.split (".wsdl");
           //System.out.println ("**array[0]: " + array[0]);
           serviceName = array[0];
        }
        else if (_servName.contains (".xsd")) {
           //System.out.println ("**XSL_servName: " + _servName);
           String[] array = _servName.split (".xsd");
           //System.out.println ("**array[0]: " + array[0]);
           serviceName = array[0];
        }
        else {
            serviceName = _servName;
        }

        //System.out.println("***serviceNameXSL : "+ serviceName);

        ServicesObject servObj = _wsdlFileInfoMap.get (_servName);
        Map<String, OperationsObject> operationsMap = servObj._operationsMap;

        //System.out.println("***table : "+ table);

        Iterator it2 = operationsMap.keySet().iterator();
        while (it2.hasNext()) {
            StringBuffer sb = new StringBuffer();
            //System.out.println("***serviceName for XSL header : "+ serviceName);
            sb.append (setXSLHeader (serviceName));
            String methodName = (String) it2.next();
            //System.out.println("***methodName XSL: "+ methodName);
            if (methodName!= null) {
                props = "\t<xsl:template match=\"ns2:" + methodName + "\">\n" +
                "\t<rdf:Description rdf:about=\"REPLACE-WITH-KAOS-URI\">\n" +
                "    <rdf:type rdf:resource=\"http://ontology.ihmc.us/" + serviceName + "Action.owl#" +
                        methodName + "Action\"/>\n";


                sb.append (props);
                //System.out.println("***methodName XSL: "+ methodName);
                OperationsObject oo = operationsMap.get (methodName);
                //System.out.println("***OperationsObject: "+ oo);
                Map<String, ParametersObject> paramsMap = oo._paramsMap;

                //System.out.println("***paramsMap: "+ paramsMap.size());
                //System.out.println("***paramsMap: "+ paramsMap);
                ParametersObject po = paramsMap.get (methodName);
                if (po == null) {
                    return;
                }
                else {
                    String body = setBodyOfFile (po);
                    sb.append (body);
                }

                String desc = "  </rdf:Description>\n" +
                              "</xsl:template>";
                sb.append (desc);
                sb.append (setFinalStatement());
                buildXSLFile (sb.toString(), methodName);
            }
        }
    }

    /**
     * Creates the XSL header of the file
     *
     * @param serviceName service file name
     * @return the string header
     */
    protected String setXSLHeader (String serviceName)
    {
        String header = "<xsl:stylesheet version=\"1.0\" \n" +
                "\txmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\"\n" +
                "\txmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"\n" +
                "xmlns:action=\"http://ontology.ihmc.us/" + serviceName + "Action.owl#\"\n" +
                "\txmlns:ns2=\"http://ont.ray.com/" + serviceName + "/\"\n" +
                "\txmlns:java=\"http://xml.apache.org/xalan/java\"\n" +
                "    exclude-result-prefixes=\"java\">";
        //System.out.println ("**Header: " + header);
        return header;
    }

    /**
     * Creates the XSL body of the file
     *
     * @param po the parameters object
     * @return the string body text
     */
    protected String setBodyOfFile (ParametersObject po)
    {
        StringBuffer allPropsStr = new StringBuffer();

        Vector<String> typeVector = po._owlPropertyRange;
        Vector<String> paramVector = po._owlPropertyName;
        //System.out.println ("**paramVector.size(): " + paramVector.size());
        if (paramVector.size() > 0) {
            for (int i=0; i<paramVector.size(); i++) {
                String prop = "";
                //System.out.println ("**paramVector.elementAt(i): " + paramVector.elementAt(i));
                if (paramVector.elementAt(i).contains ("dm")) {
                    String[] array = paramVector.elementAt(i).split(":");
                    //System.out.println ("**dm prop: " + array[1]);
                    prop = array[1];
                }
                else {
                    if (paramVector.elementAt(i).trim().contains (" ")) {
                        //do nothing it contains no attributes
                    }
                    else {
                        //System.out.println ("**prop: " + prop);
                        prop = paramVector.elementAt(i);
                    }
                }
                String props2 =  "    <action:has" + prop + " rdf:resource=\"{" + prop + "}\"/>\n" + "";

                allPropsStr.append (props2);
            }
        }

        return allPropsStr.toString();
    }

    /**
     * Creates the final statement of the XSL file
     *
     * @return the string statement
     */
    protected String setFinalStatement()
    {
        return "</xsl:stylesheet>";
    }

    /**
     * Saves the data as XSL an file.
     */
    protected void buildXSLFile (String data, String methodName)
    {
        //System.out.println ("**XSLdata: " + data);
        String filename = _filePath + File.separator + methodName;
        //System.out.println ("**filename: " + filename);
        if (filename.length() < 0) {
            return;
        }
        if (filename.indexOf (".xsl") < 0) {
            filename = filename + "2Ont.xsl";
        }
        try {
            BufferedWriter out = new BufferedWriter (new FileWriter (filename));
            if (data != null) {
                out.write (data);
            }
            out.close();
        }
        catch (IOException ioe) {
            ioe.printStackTrace();
        }
    }

    //Variables
    //protected static String _configPath = "C:\\develop\\delegation\\code\\DelegationManagement\\services\\";
    protected Map<String, ServicesObject> _wsdlFileInfoMap; //Maps the service name with the services object.
    protected String _servName;
    protected String _filePath = "";
}
