package kaos.tools.webServiceTool;

import net.percederberg.mibble.Mib;
import net.percederberg.mibble.MibImport;
import net.percederberg.mibble.MibSymbol;
import net.percederberg.mibble.MibTypeSymbol;
import net.percederberg.mibble.MibType;
import net.percederberg.mibble.MibValueSymbol;
import net.percederberg.mibble.MibValue;
import net.percederberg.mibble.snmp.SnmpTextualConvention;
import net.percederberg.mibble.snmp.SnmpModuleIdentity;
import net.percederberg.mibble.snmp.SnmpObjectIdentity;
import net.percederberg.mibble.snmp.SnmpObjectType;
import net.percederberg.mibble.snmp.SnmpNotificationType;
import net.percederberg.mibble.snmp.SnmpTrapType;
import net.percederberg.mibble.snmp.SnmpObjectGroup;
import net.percederberg.mibble.snmp.SnmpNotificationGroup;
import net.percederberg.mibble.snmp.SnmpModuleCompliance;
import net.percederberg.mibble.snmp.SnmpAgentCapabilities;
import net.percederberg.mibble.snmp.SnmpType;
import net.percederberg.mibble.type.SequenceType;
import net.percederberg.mibble.type.ElementType;
import net.percederberg.mibble.type.Constraint;
import net.percederberg.mibble.type.IntegerType;
import net.percederberg.mibble.type.BitSetType;
import net.percederberg.mibble.type.StringType;
import net.percederberg.mibble.type.SequenceOfType;

import java.util.Map;
import java.util.Hashtable;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;
import java.util.List;
import java.util.HashMap;
import java.util.ArrayList;

/**
 * User: mbreedy
 * Date: Apr 11, 2011
 * Time: 1:40:05 PM
 * @author Maggie Breedy
 * @version: $Revision: 1.11 $
 */
public class MIBFilesHandler
{
    /**
     * Costructor
     * @param mib   mib file
     */
    public MIBFilesHandler (Mib mib)
    {
        _mib = mib;
        _textualConvMap = new HashMap<String, Vector>();
        init();
    }

    /**
     * Initialization of the class handler
     * @return mainServiceMap   maps the service file name with its service object
     */
    protected Map<String, ServicesObject> init()
    {
        //System.out.println ("**handleMIBFile:mib: " + mib);
        String serviceName = _mib.getName();
        //System.out.println ("**handleMIBFile:mib serviceName: " + serviceName);
        //Mib file name
        String logicalURL = "http://ontology.ihmc.us/services/" + serviceName + ".owl";
        //System.out.println ("**logicalURL: " + logicalURL);

        //Imported Ontologies
        Map<String, String> importedOntMap = new Hashtable<String, String>();
        String defAbbr = "KaosAction";
        String defImpOnt = "http://ontology.ihmc.us/Action.owl";
        importedOntMap.put (defAbbr, defImpOnt);
        Collection imports = _mib.getAllImports();
        HashMap<String,Vector> importsSymbMap = new HashMap<String, Vector>();
        if (imports.size() > 0) {
            Iterator it = imports.iterator();
            while (it.hasNext()) {
                MibImport mi = (MibImport) it.next();
                //System.out.println ("**Import: " + mi.getName());
                Vector<String> impStrVector = new Vector<String>();
                Collection c = mi.getAllSymbolNames();
                Iterator it1 = c.iterator();
                while (it1.hasNext()) {
                    Object o  = it1.next();
                    impStrVector.addElement (o.toString());
                }
                //System.out.println ("**Import:Objects:impStrVector: " + impStrVector);
                importsSymbMap.put (mi.getName(), impStrVector);
                //Mib mibImp = mi.getMib();
                //System.out.println ("**Imported file: " + mibImp.getFile().getAbsolutePath());
                //importedOntMap.put (mi.getName(), "http://ontology.ihmc.us/services/" + mi.getName() + "Data" + ".owl");
                importedOntMap.put (mi.getName(), "http://ontology.ihmc.us/Computing/Network/mib/" + mi.getName() + ".owl");
            }
        }

        Map<String, OperationsObject> allOperationsMap = new Hashtable<String, OperationsObject>();
        ParametersObject paramsObj = null;
        Map<String, Vector> textualTypesData = new HashMap<String, Vector>();
        Collection symbols = _mib.getAllSymbols();
        Map<String, ParametersObject> paramsMap = new Hashtable<String, ParametersObject>();
        Vector<String> pmVector = null;
        Vector<String> typVector = null;
        Vector<String> owlParVector = null;
        Vector<String> owlRangeVector = null;
        //HashMap<String,HashMap> tableDataMap = new HashMap<String, HashMap>();

        if (symbols.size() > 0) {
            Iterator it1 = symbols.iterator();
            while (it1.hasNext()) {
                MibSymbol ms = (MibSymbol) it1.next();
                //System.out.println ("**All Symbols: " + ms.getName());
                Map<String, ParametersObject> sequenceMap = new Hashtable<String, ParametersObject>();
                Vector<String> seqVector = new Vector<String>();
                Vector<String> textualVector = new Vector<String>();
                String objectID = "";

                if (ms instanceof MibTypeSymbol) {
                    //System.out.println ("*****It is a MibTypeSymbol*****");
                    //System.out.println ("**Mib Type Symbol: " + ms.getName());
                    pmVector = new Vector<String>();
                    typVector = new Vector<String>();
                    owlParVector = new Vector<String>();
                    owlRangeVector = new Vector<String>();

                    MibTypeSymbol mts = (MibTypeSymbol) ms;
                    MibType mt = mts.getType();

                    //System.out.println ("**Mib Type Symbol: " + mt.getName());

                    /*if (mt instanceof SequenceType) {
                        SequenceType st = (SequenceType) mt;
                        //String seqType = st.getName();
                        System.out.println ("**Type Symbol:SequenceType: " + ms.getName());
                        if (ms.getName().contains ("Entry")) {
                            break;
                        }
                        //System.out.println ("**SequenceType: " + seqType);
                        ElementType[] elemType = st.getAllElements();
                        for (int i = 0; i <elemType.length; i++) {
                            String name = elemType[i].getName();
                            seqVector.addElement (name);
                            //System.out.println ("**Sequence elem name: " + name); // name
                            MibType smt = elemType[i].getType();
                            //System.out.println ("**Sequence type: " + smt.getName());
                            //System.out.println ("**Sequence str: " + smt.toString()); //it is the type syntax
                            pmVector.addElement (smt.getName());
                            owlParVector.addElement (smt.getName());
                            typVector.addElement (smt.toString());
                            owlRangeVector.addElement (smt.toString());
                        }
                    }*/
                    if (mt instanceof SequenceOfType) {
                        SequenceOfType seqOfType = (SequenceOfType) mt;
                        //System.out.println ("**SEQUENCE OF TYPE: " + seqOfType.getElementType());
                    }
                    else if (mt instanceof SnmpTrapType) {
                        //System.out.println ("**Type is SnmpTrapType:  " + mt);
                        SnmpTrapType trapType = (SnmpTrapType) mt;
                        String trpname = trapType.getName();
                        System.out.println ("**Type Trap name: " + trpname);

                        List trapListArray = trapType.getVariables();
                        for (Object name : trapListArray) {
                            MibValueSymbol nameValue = (MibValueSymbol) name;
                            //System.out.println ("**Trap list name: " + name.toString());
                            //System.out.println ("**Type Trap nameValue: " + nameValue.getType().getName());
                            //System.out.println ("**Trap Object ID: " + nameValue.getValue().toObject()); //Object ID
                        }
                    }
                    else if (mt instanceof SnmpNotificationType) {
                        System.out.println ("**Type is SnmpNotificationType:  " + mt);
                    }
                    else if (mt instanceof SnmpObjectGroup) {
                        System.out.println ("**Type is SnmpObjectGroup:  " + mt);
                    }
                    else if (mt instanceof SnmpNotificationGroup) {
                        System.out.println ("**Type is SnmpNotificationGroup:  " + mt);
                    }

                    if (mt instanceof SnmpTextualConvention) {
                        //System.out.println ("**Type is SnmpTextualConvention:  " + mt);
                        SnmpTextualConvention stc = (SnmpTextualConvention) mt;
                        //SnmpTextualConvention texConv = stc.findReference(mt);
                        String textualConvName = stc.getName();
                        //String name = stc.getName();
                        //System.out.println ("**textualConvName: " + textualConvName);
                        //System.out.println ("**Type SnmpTextualConvention: " + ms.getName() + "****");
                        MibType stype = stc.getSyntax();

                        //System.out.println ("**textualConvName type syntax: " + stype.getName());
                        Vector<String> tcVector = getMIBTextualConventions (stype);
                        if (!tcVector.isEmpty()) {
                            for (String str : tcVector) {
                                //System.out.println ("**textualConvName: " + str);
                                textualVector.addElement (str);
                            }
                        }
                    }

                    textualTypesData.put (ms.getName(), textualVector); //TC name and a vector of types (string, integer, etc)

                    if (pmVector.isEmpty() && typVector.isEmpty()) {
                        pmVector.addElement ("NoAttribute");
                        owlParVector.addElement ("NoAttribute");
                        typVector.addElement ("NoType");
                        owlRangeVector.addElement ("NoType");
                    }
                    paramsObj = new ParametersObject (pmVector, typVector, owlParVector, owlRangeVector,
                                                      new Vector<String>(), new Vector<String>(), new Vector<Object>());

                    paramsMap.put (ms.getName(), paramsObj);
                    String action = ms.getName();
                    String superAction = serviceName;

                    //System.out.println ("**Mib Type Symb Name:<" + ms.getName() + ">");
                    OperationsObject oo = new OperationsObject (ms.getName(), action, superAction,  paramsMap,
                                                                objectID);
                    allOperationsMap.put (ms.getName(),  oo);

                    //To ensure that the sequence is not a table entry sequence
                    
                    /*if (ms.getName().contains ("Entry")) {
                        String[] strArr = ms.getName().split ("Entry");
                        if (strArr.length > 0) {
                            String seqStr = strArr[0];
                            System.out.println ("**-------------seqStr1:  " + seqStr);
                            seqStr = seqStr.substring (0,1).toLowerCase() + seqStr.substring(1, seqStr.length()) + "Table";
                            System.out.println ("**-------------seqStr2:  " + seqStr);
                            if (!allOperationsMap.containsKey (seqStr)) {
                                action = ms.getName();
                                superAction = serviceName;
                                //System.out.println ("**Mib Type Symb Name:<" + ms.getName() + ">");
                                oo = new OperationsObject (ms.getName(), action, superAction,  paramsMap,
                                                                            objectID);
                                allOperationsMap.put (ms.getName(),  oo);
                            }
                        }
                        break;
                    }
                    else {
                        paramsMap.put (ms.getName(), paramsObj);
                        action = ms.getName();
                        superAction = serviceName;

                        //System.out.println ("**Mib Type Symb Name:<" + ms.getName() + ">");
                        oo = new OperationsObject (ms.getName(), action, superAction,  paramsMap,
                                                                    objectID);
                        allOperationsMap.put (ms.getName(),  oo);
                    }*/
                 }
                 else if (ms instanceof MibValueSymbol) {
                    //System.out.println ("------It is a MibValueSymbol--------");
                    //System.out.println ("------MibValueSymbol:ms:" + ms.getName() + "------");
                    MibValueSymbol value = (MibValueSymbol) ms;
                    //System.out.println ("------MibValueSymbol:value:" + value + "------");
                    //MibType type =  value.getType();
                    //this is causing problems!!!!!!!
                    /*if (ms.getName().contains ("Entry")) {
                        break;
                    }*/
                    //System.out.println ("-----MibValueSymbol:type:" + type); //long description
                    MibValue mvalue = value.getValue();
                    //System.out.println ("-----MibValueSymbol:mvalue:" + mvalue); //Object Id value

                    //System.out.println ("------MibValueSymbol:Object ID: " + value.getValue().toObject());
                    pmVector = new Vector<String>();
                    typVector = new Vector<String>();
                    owlParVector = new Vector<String>();
                    owlRangeVector = new Vector<String>();

                    String access = "";
                    String symbName = "";
                    String action = "";
                    String superAction = "";

                    //MibType mType =  value.getType();
                    //System.out.println ("------MibValueSymbol:ms:" + mType.getName() + "------");

                    //if (mType.getName().equals ("OBJECT-TYPE")) {
                        //System.out.println ("------MibValueSymbol:OBJECT-TYPE:" + ms.getName() + "------");

                    if (value.isTable()) {
                        //System.out.println ("----**Table name:symbName:<" + value.getName() + ">");
                        MibValueSymbol[] tarray = value.getChildren();
                        //System.out.println ("----**Table name:symbName:<" + value.getName() + ">");
                        symbName = value.getName();
                        //System.out.println ("----**Table type:<" + value.getType().getName() + ">");
                        //System.out.println ("------Table children:<" + tarray.length + ">");
                        //Vector<String> accessVector = new Vector<String>();
                        Vector<Object> tcVector = new Vector<Object>();

                        if (tarray.length > 0) { //There is a table with elements in it.
                            for (int i=0; i< tarray.length; i++) {
                                MibType aType =  tarray[i].getType();
                                //System.out.println ("----**tarray[i]:<" + tarray[i].getName() + ">");
                                MibValueSymbol[] tarray2 = tarray[i].getChildren();
                                //System.out.println ("------Table children:tarray2<" + tarray2.length + ">");
                                if (tarray2.length > 0) { //Means the table has a sequence of elements.
                                    for (int j=0; j< tarray2.length; j++) {
                                        //Map tableName and map of element names with the map of types.
                                        //System.out.println ("----**tarray2:<" + tarray2[j].getName() + ">"); //NameEntry on the Table
                                        MibType mtype =  tarray2[j].getType();
                                        //System.out.println ("------Elem types: " +  mtype.getName()); //no need
                                        SnmpObjectType obType = (SnmpObjectType) mtype;
                                        //System.out.println ("------Table: " +  tarray2[j].getValue());
                                        MibType mtSyntx = obType.getSyntax();

                                        //SnmpTextualConvention stc = (SnmpTextualConvention) mtSyntx;
                                        //SnmpTextualConvention texConv = stc.findReference (mtype);
                                        //System.out.println ("---*Table snmp obj type texConv: " + texConv);

                                        if (mtSyntx instanceof IntegerType) {
                                            IntegerType st = (IntegerType) mtSyntx;
                                            MibSymbol[] msa = st.getAllSymbols();
                                            Vector<String> tableElemVector = new Vector<String>();
                                            for (int k=0; k< msa.length; k++) {
                                                //System.out.println ("---*Table snmp obj type IntegerType: " + msa[k].getName());
                                                tableElemVector.addElement (msa[k].getName());
                                            }
                                            //System.out.println ("---*Table stableElemVector: " +tableElemVector);
                                            if (tableElemVector.size() > 0) {
                                                Iterator iter = textualTypesData.keySet().iterator();
                                                while (iter.hasNext()) {
                                                    String key = (String) iter.next();
                                                    Vector<String> ttdVec = textualTypesData.get(key);
                                                    //System.out.println ("---*ttdVec textual conv: " + ttdVec);
                                                    if (ttdVec.equals (tableElemVector)) {
                                                        //System.out.println ("---*tcVec textual conv: " + key);
                                                        tcVector.addElement (key);
                                                        //typVector.addElement (key);
                                                        //owlRangeVector.addElement (key);
                                                    }
                                                    else {
                                                        tcVector.addElement (getOwlDataTypes("INTEGER"));
                                                    }
                                                }
                                            }
                                            else {
                                                //System.out.println ("---*Diff type from table: " + st.getName());
                                                String syntax = getOwlDataTypes (obType.getSyntax().getName());
                                                tcVector.addElement (syntax);
                                                //System.out.println ("------Syntax:OWL:" + syntax + ">");
                                                //typVector.addElement (syntax);
                                                //owlRangeVector.addElement (syntax);
                                            }
                                        }
                                        else {
                                             System.out.println ("------Table snmp obj type Else: " +  mtSyntx.getName());
                                        }
                                        objectID = tarray2[j].getValue().toObject().toString(); //This gives Object ID

                                        pmVector.addElement (tarray2[j].getName());
                                        owlParVector.addElement (tarray2[j].getName());

                                        String accessStr = obType.getAccess().toString();
                                        if (accessStr != null && !accessStr.equalsIgnoreCase ("not-accessible")) {
                                            //System.out.println ("------Table access2:<" + accessStr + ">");
                                            typVector.addElement (accessStr);
                                            owlRangeVector.addElement (accessStr);
                                        }
                                        else {
                                            String syntax = getOwlDataTypes (obType.getSyntax().getName());
                                            //System.out.println ("------Syntax:OWL:" + syntax + ">");
                                            typVector.addElement (syntax);
                                            owlRangeVector.addElement (syntax);
                                        }
                                        /*System.out.println ("------Table tarray2[j].getName(): " + tarray2[j].getName());
                                        System.out.println ("------Table pmVector: " + pmVector);
                                        System.out.println ("------Table owlParVector: " + owlParVector);
                                        System.out.println ("------Table typVector: " + typVector);
                                        System.out.println ("------Table owlRangeVector: " + owlRangeVector);*/
                                    }
                                }
                            }
                        }
                        paramsObj = new ParametersObject (pmVector, typVector, owlParVector, owlRangeVector,
                                                          new Vector<String>(), new Vector<String>(), tcVector);
                        paramsMap.put (value.getName(), paramsObj);

                        if (!symbName.equals("")) {
                            action = "Access" + value.getName() + "Action";
                            superAction = serviceName;

                            //System.out.println ("------Tables symbName:<" + symbName + ">");
                            /*System.out.println ("------Tables action:<" + action + ">");
                            System.out.println ("------Tables superAction:<" + superAction + ">");
                            System.out.println ("------Tables objectID:<" + objectID + ">");*/
                            OperationsObject oo = new OperationsObject (symbName, action, superAction,  paramsMap,
                                                                        objectID);
                            //allOperationsMap.put (symbName,  oo);
                            allOperationsMap.put (value.getName(),  oo);
                        }
                    }
                    else if (value.isScalar()) {
                        //System.out.println ("----**Scalar name:symbName:<" + value.getName() + ">");
                        //System.out.println ("----**Scalar name:symbName:<" + value.getName() + ">");
                        //System.out.println ("------Scalar:Object ID: " + value.getValue().toObject()); //Object ID
                        symbName = value.getName();

                        objectID = value.getValue().toObject().toString();   //This gives Object ID
                        SnmpType stype = (SnmpType) value.getType();
                        //type.getDescription();
                        SnmpObjectType sot = (SnmpObjectType) stype;
                        //System.out.println ("------Scalar Type: " + sot.getName());

                        //MibType mt = sot.getSyntax();
                        //System.out.println ("------Scalar Syntax: " + mt.getName());

                        if (sot == null) {
                            System.out.println ("*SnmpObjectType is null");
                        }
                        else {
                            //Add access and objectId as param and type for now !!!!
                            access = sot.getAccess().toString();
                            //System.out.println ("------Scalar Name: " + symbName + " access:<" + access + ">");
                        }
                        action = "Access" + value.getName() + "Action";
                        superAction = serviceName;

                        if (access != null && !access.equals("") ) {
                            //System.out.println ("------* access to parse:<" + access + ">");
                            String[] strArray = parseMibAccessStr (access);
                            if (strArray.length > 1) {
                                for (int i=0; i< strArray.length; i++) {
                                    //System.out.println ("------strArray[i]: " + strArray[i]);
                                    pmVector.addElement (strArray[i]);
                                    owlParVector.addElement (strArray[i]);
                                    typVector.addElement (strArray[i]);
                                    owlRangeVector.addElement (strArray[i]);
                                }

                            }
                            else {
                                //System.out.println ("------strArray[0]: " + strArray[0]);
                                pmVector.addElement (strArray[0]);
                                owlParVector.addElement (strArray[0]);
                                typVector.addElement (strArray[0]);
                                owlRangeVector.addElement (strArray[0]);
                            }
                            paramsObj = new ParametersObject (pmVector, typVector, owlParVector, owlRangeVector,
                                                              new Vector<String>(), new Vector<String>(), new Vector<Object>());
                            //System.out.println ("------symbName before the object: " + symbName);
                            if (paramsObj != null) {
                                if (seqVector.contains (symbName)) {
                                    sequenceMap.put (symbName, paramsObj);
                                }
                                else {
                                    paramsMap.put (symbName, paramsObj);
                                }
                             }
                             OperationsObject oo = new OperationsObject (symbName, action, superAction,  paramsMap,
                                                                         objectID);
                             allOperationsMap.put (symbName,  oo);
                        }
                    }
                    else { //No scalar No table
                        symbName = value.getName();
                        //System.out.println ("---No scalar type--*:symbName: " + symbName);
                        MibType mtype = value.getType();
                        //System.out.println ("------No scalar type1: " + mtype.getName()); //But has access!!!!

                        if (mtype.getName().equals ("NOTIFICATION-TYPE")) {   //TRAPS
                            objectID = value.getValue().toObject().toString();
                            System.out.println ("------ASymbName:trap: " + symbName + " .Object ID:no scalar: " + objectID);
                            //System.out.println ("---No scalar type--*:symbName:Trap: " + symbName);
                            if (mtype instanceof SnmpObjectType) {
                                //System.out.println ("------MibValueSymbol:NOTIFICATION-TYPE:" + ms.getName() + "------");
                                objectID = value.getValue().toObject().toString();   //This gives Object ID
                                //System.out.println ("------Trap objectID: " + objectID);
                                SnmpType stype = (SnmpType) value.getType();
                                if (stype instanceof Object) {
                                    System.out.println ("**Type is Snmp ObjectType:  " + stype);
                                    //System.out.println ("**Type is Snmp Ref Symb:  " + stype.getClass().toString());
                                }

                                SnmpObjectType sotype = (SnmpObjectType) stype;
                                //System.out.println ("------sotype: " +  sotype);
 
                                //System.out.println ("**Type is Snmp ObjectType:  " + stype);
                                if (stype instanceof SnmpTrapType) {
                                    //System.out.println ("**Type is SnmpTrapType:  " + mt);
                                    SnmpTrapType trapType = (SnmpTrapType) stype;
                                    String trpname = trapType.getName();
                                    System.out.println ("**Type Trap name: " + trpname);
                                    List trapListArray = trapType.getVariables();
                                    for (Object name : trapListArray) {
                                        MibValueSymbol nameValue = (MibValueSymbol) name;
                                        System.out.println ("**Trap list name: " + name.toString());
                                        //System.out.println ("**Type Trap nameValue: " + nameValue.getType().getName());
                                        //System.out.println ("**Trap Object ID: " + nameValue.getValue().toObject()); //Object ID
                                    }
                                }
                            }
                            action = symbName;
                            superAction = serviceName;
                        }
                        else {
                            System.out.println ("------No Scalar NOTIFICATION-TYPE:name: " + symbName);
                        }
                    }
                }
                else {
                    System.out.println ("**NO MibValue No Mibtype Symbol---->: " + ms.getName());
                }
            }
        }
        
        ServicesObject so = new ServicesObject (serviceName, logicalURL, importedOntMap, allOperationsMap, null,
                                                defAbbr, defImpOnt, textualTypesData);

        Map<String, ServicesObject> mainServicesMap = new Hashtable<String, ServicesObject>();
        mainServicesMap.put (_mib.getName(), so);

        return mainServicesMap;
    }

    /**
     * Parses the the access when it contains two words separated by -.
     * Example: read-write
     *
     * @param access
     * @return a string array of the elements .
     */
    public String[] parseMibAccessStr (String access)
    {
        String[] accessArray = null;
        if (access.contains ("write")) {
            accessArray = new String [access.length()];
            String accessStr = access.substring (0, access.length());
            accessArray = accessStr.split ("-");
            //System.out.println ("**accessArray: " + accessArray.length);
        }
        else if (access.contains ("read")) {
            accessArray = new String [access.length()];
            String accessStr = access.substring (0, access.length());
            accessArray = accessStr.split ("-");
            //System.out.println ("**accessArray 1: " + accessArray[1]);
            //System.out.println ("**accessArray: " + accessArray.length);
        }
        else {
            String accessStr = access.substring (0, access.length());
            accessArray = new String[1];
            accessArray[0] = accessStr;
        }

        if (accessArray.length > 0) {
            if (accessArray[1].equalsIgnoreCase ("only")) {
                //System.out.println ("**accessArray 1: " + accessArray[1]);
                String[] sArr = new String [1];
                sArr[0] = accessArray[0];
                return sArr;
            }
            else {
                return accessArray;
            }
        }
        else {
            return new String[0];
        }
    }

    /**
     * Gets the Snmp object type for this type
     * @param type
     */
    protected void getSnmpObjectType (SnmpObjectType type)
    {
        System.out.println ("**SYNTAX: " + type.getSyntax());
        if (type.getUnits() != null) {
            System.out.println ("**UNITS: " + type.getUnits());
        }
        System.out.println ("**ACCESS: " + type.getAccess());
        //System.out.println ("**STATUS: " + type.getStatus());
        if (type.getDescription() != null) {
            //System.out.println ("**Description: " + type.getDescription());
        }
        if (type.getReference() != null) {
            System.out.println ("**REFERENCE: " + type.getReference());
        }
        if (type.getIndex() != null && type.getIndex().size() > 0) {
            System.out.println ("**INDEX: " + type.getIndex());
        }
        if (type.getAugments() != null) {
            System.out.println ("**AUGMENTS: " + type.getAugments());
        }
        if (type.getDefaultValue() != null) {
            System.out.println ("**DEFVAL: " + type.getDefaultValue());
        }
    }

    /**
     * Gets Mib textual convention types.
     *
     * @param   stype    the MibType
     * @return  tcVector    a vector of names for each type
     */
    protected Vector<String> getMIBTextualConventions (MibType stype)
    {
        Vector<String> tcVector = new Vector<String>();
        if (stype instanceof IntegerType) {
            IntegerType it = (IntegerType) stype;
            MibValueSymbol[] mvs = it.getAllSymbols();
            if (mvs.length > 0) {
                for (MibSymbol msym : mvs) {
                    //System.out.println ("**It is a IntegerType***: " + msym.getName());
                    tcVector.addElement (msym.getName());
                }
            }
        }
        else if (stype instanceof BitSetType) {
            //System.out.println ("**It is a BitSetType***: " + stype.getName());
            BitSetType bt = (BitSetType) stype;
            MibValueSymbol[] mvs = bt.getAllSymbols();
            if (mvs.length > 0) {
                for (MibSymbol msym : mvs) {
                    //System.out.println ("**It is a BitSetType***: " + msym.getName());
                    //System.out.println ("**It is a BitSetType***Constraint: " + bt.getConstraint().toString());
                    tcVector.addElement (msym.getName());
                }
            }
        }
        else if (stype instanceof StringType) {
            StringType st = (StringType) stype;
            //System.out.println ("**It is a StringType***: " + stype.getName());
            //System.out.println ("**It is a StringType***: " + st.getName());
            if (st.hasConstraint()) {
                Constraint constr = st.getConstraint();
                //tcVector.addElement (constr.toString());    //Not added for now
                //System.out.println ("**It is a constr***: " + constr.toString()); //Gives the size of string.
            }
        }
        else {
            System.out.println ("**Unknown textual convention type**");
        }

        return tcVector;
    }

    protected String getOwlDataTypes (String syntax)
    {
        String synx = "";
        if (syntax.equalsIgnoreCase("INTEGER")) {
            synx = "xsd:int";
        }
        else if (syntax.equalsIgnoreCase ("OCTET STRING")) {
            synx = "xsd:string";
        }
        else {
            synx = syntax;
        }
        return synx;
    }

     /**
     * Returns a MIB type constraint. If the type didn't have any
     * constraint, null will be returned.
     *
     * @param type           the MIB type
     *
     * @return the MIB type constraint, or
     *         null if no constraint was set
     */
    private Constraint getConstraint (MibType type)
     {
        if (type instanceof IntegerType) {
            return ((IntegerType) type).getConstraint();
        }
        else if (type instanceof BitSetType) {
            return ((BitSetType) type).getConstraint();
        }
        else if (type instanceof StringType) {
            return ((StringType) type).getConstraint();
        }
        else if (type instanceof SnmpTextualConvention) {
            return getConstraint(((SnmpTextualConvention) type).getSyntax());
        }
        else {
            return null;
        }
     }

    public class SequenceTypeObject extends Object
    {
        public SequenceTypeObject (String stName, String seqName, String seqType)
        {
            _stName = stName;
            _seqName = seqName;
            _seqType = seqType;
        }
        protected String _stName; //Sequence type name
        protected String _seqName; //Sequence element name
        protected String _seqType; //Sequence element type
    }

    public class TableDataObject extends Object
    {
        public TableDataObject (String stName, HashMap<String, Vector> tableDataTypes)
        {
            _stName = stName;
            _tableDataTypes = tableDataTypes;
        }
        protected String _stName; //Table name
        protected HashMap<String, Vector> _tableDataTypes = new HashMap<String, Vector>(); //Maps the element
                                                                                            //names with a vector of
                                                                                            //their objectID and access
    }

    //Variables
    protected Mib _mib;
    protected HashMap<String,Vector> _textualConvMap;
}
