package kaos.tools.webServiceTool;

import net.percederberg.mibble.Mib;
import net.percederberg.mibble.MibImport;
import net.percederberg.mibble.MibSymbol;
import net.percederberg.mibble.MibTypeSymbol;
import net.percederberg.mibble.MibType;
import net.percederberg.mibble.MibValueSymbol;
import net.percederberg.mibble.MibValue;
import net.percederberg.mibble.snmp.SnmpTextualConvention;
import net.percederberg.mibble.snmp.SnmpModuleIdentity;
import net.percederberg.mibble.snmp.SnmpObjectIdentity;
import net.percederberg.mibble.snmp.SnmpObjectType;
import net.percederberg.mibble.snmp.SnmpNotificationType;
import net.percederberg.mibble.snmp.SnmpType;
import net.percederberg.mibble.type.SequenceType;
import net.percederberg.mibble.type.ElementType;
import net.percederberg.mibble.type.Constraint;
import net.percederberg.mibble.type.IntegerType;
import net.percederberg.mibble.type.BitSetType;
import net.percederberg.mibble.type.StringType;
import net.percederberg.mibble.type.SequenceOfType;

import javax.swing.*;
import java.util.Map;
import java.util.Hashtable;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;
import java.util.List;
import java.util.HashMap;
import java.util.ArrayList;
import java.net.URL;
import java.net.MalformedURLException;
import java.io.BufferedInputStream;
import java.io.IOException;

/**
 * User: mbreedy
 * Date: Jul 11, 2011
 * Time: 12:21:24 PM
 *
 * @author Maggie Breedy
 * @version: $Revision: 1.13 $
 */
public class MIBFilesParser
{
     /**
     * Costructor
     * @param mib   mib file
     */
    public MIBFilesParser (Mib mib)
    {
        _mib = mib;
    }

    /**
     * Initialization of the class handler
     * @return mainServiceMap   maps the service file name with its service object
     */
    public Map<String, ServicesObject> init()
    {
        _textualConvMap = new HashMap<String, Vector>();
        _objectIDsMap = new HashMap<String, String>();
        _mapOfTables = new HashMap<String, ParametersObject>();
        _mapOfScalars = new HashMap<String,Vector>();
        _mapOfScalarObjects = new HashMap<String,ParametersObject>();

        //System.out.println ("**handleMIBFile:mib: " + mib);
        String serviceName = _mib.getName();

        String logicalURL = "http://ontology.ihmc.us/Computing/Network/mib/" + serviceName + ".owl";

        //Imported Ontologies
        Map<String, String> importedOntMap = new Hashtable<String, String>();
        String defAbbr = "KaosAction";
        String defImpOnt = "http://ontology.ihmc.us/Action.owl";
        importedOntMap.put (defAbbr, defImpOnt);
        Collection imports = _mib.getAllImports();
        HashMap<String,Vector> importsSymbMap = new HashMap<String, Vector>();
        if (imports.size() > 0) {
            Iterator it = imports.iterator();
            while (it.hasNext()) {
                MibImport mi = (MibImport) it.next();
                //System.out.println ("**Import: " + mi.getName());
                Vector<String> impStrVector = new Vector<String>();
                Collection c = mi.getAllSymbolNames();
                Iterator it1 = c.iterator();
                while (it1.hasNext()) {
                    Object o  = it1.next();
                    impStrVector.addElement (o.toString());
                }
                //System.out.println ("**Import:Objects:impStrVector: " + impStrVector);
                importsSymbMap.put (mi.getName(), impStrVector);
                //Mib mibImp = mi.getMib();
                //System.out.println ("**Imported file: " + mibImp.getFile().getAbsolutePath());
                //importedOntMap.put (mi.getName(), "http://ontology.ihmc.us/services/" + mi.getName() + "Data" + ".owl");
                //System.out.println ("**Import name: " + mi.getName());
                importedOntMap.put (mi.getName(), "http://ontology.ihmc.us/Computing/Network/mib/" + mi.getName() + ".owl");
                //importedOntMap.put (mi.getName(), "http://localhost:8080/" + mi.getName() + ".owl");
            }
        }

        Map<String, OperationsObject> allOperationsMap = new Hashtable<String, OperationsObject>();
        ParametersObject paramsObj = null;
        //Map<String, Vector> textualTypesData = new HashMap<String, Vector>();
        Collection symbols = _mib.getAllSymbols();
        Map<String, ParametersObject> paramsMap = new Hashtable<String, ParametersObject>();
        Vector<String> pmVector = null;
        Vector<String> typVector = null;
        Vector<String> owlParVector = null;
        Vector<String> owlRangeVector = null;

        if (symbols.size() > 0) {
            Iterator it1 = symbols.iterator();
            while (it1.hasNext()) {
                MibSymbol ms = (MibSymbol) it1.next();
                //System.out.println ("**All Symbols: " + ms.getName());
                //Map<String, ParametersObject> sequenceMap = new Hashtable<String, ParametersObject>();
                //Vector<String> seqVector = new Vector<String>();
                Vector<String> textualVector = new Vector<String>();
                String objectID = "";

                if (ms instanceof MibTypeSymbol) {
                    //System.out.println ("*****It is a MibTypeSymbol*****");
                    //System.out.println ("**Mib Type Symbol: " + ms.getName());
                    pmVector = new Vector<String>();
                    typVector = new Vector<String>();
                    owlParVector = new Vector<String>();
                    owlRangeVector = new Vector<String>();

                    MibTypeSymbol mts = (MibTypeSymbol) ms;
                    MibType mt = mts.getType();

                    if (mt instanceof SnmpTextualConvention) {
                        //System.out.println ("**Type is SnmpTextualConvention:  " + mt);
                        //System.out.println ("**TextualConvention:  " + ms.getName());
                        SnmpTextualConvention stc = (SnmpTextualConvention) mt;
                        MibType stype = stc.getSyntax();

                        //System.out.println ("**textualConvName type syntax: " + stype.getName());
                        Vector<String> tcVector = getMIBTextualConventions (stype, ms);
                        //System.out.println ("**textualConv tcVector: " + tcVector);
                        if (!tcVector.isEmpty()) {
                            for (String str : tcVector) {
                                //System.out.println ("**textualConvName: " + str);
                                textualVector.addElement (str);
                            }
                        }
                    }

                    if (textualVector.size() > 0) {
                        _textualConvMap.put (ms.getName(), textualVector); //TC name and a vector of types (string, integer, etc)
                    }
                    //textualTypesData.put (ms.getName(), textualVector); //TC name and a vector of types (string, integer, etc)

                    if (pmVector.isEmpty() && typVector.isEmpty()) {
                        pmVector.addElement ("NoAttribute");
                        owlParVector.addElement ("NoAttribute");
                        typVector.addElement ("NoType");
                        owlRangeVector.addElement ("NoType");
                    }
                    paramsObj = new ParametersObject (pmVector, typVector, owlParVector, owlRangeVector,
                                                      new Vector<String>(), new Vector<String>(), new Vector<Object>());

                    paramsMap.put (ms.getName(), paramsObj);
                    String action = ms.getName();
                    String superAction = serviceName;

                    _objectIDsMap.put (ms.getName(), objectID);

                    OperationsObject oo = new OperationsObject (ms.getName(), action, superAction,  paramsMap,
                                                                objectID);
                    if (!ms.getName().contains("Entry")) {
                        allOperationsMap.put (ms.getName(),  oo);
                    }
                }
                else {
                    //System.out.println ("**NO ----Mib Type Symbol: " + ms.getName());
                    if (ms instanceof MibValueSymbol) {
                        //System.out.println ("**NO ----Mib Type Symbol class2: " + ms.getClass().getName());
                        MibValueSymbol value = (MibValueSymbol) ms;
                        MibType mType =  value.getType();

                        if (mType.getName().equals ("OBJECT-TYPE")) {
                            //System.out.println ("------MibValueSymbol:OBJECT-TYPE:" + ms.getName() + "------");
                            if (value.isScalar()) {
                                //System.out.println ("----**OBJECT-TYPE It is a Scalar :symbName:<" + value.getName() + ">");
                                allOperationsMap = parseScalarData (ms, serviceName, paramsMap, allOperationsMap);
                            }
                            else if (value.isTable()) {
                                //System.out.println ("----**OBJECT-TYPE: It is a Table name:symbName:<" + value.getName() + ">");
                                allOperationsMap = parseTableData (ms, serviceName, paramsMap, allOperationsMap);
                            }
                            else {
                                //System.out.println ("------MibValueSymbol:OBJECT-TYPE: No table No scalar: " + ms.getName() + "------");
                            }
                        }
                        else if (mType.getName().equals ("NOTIFICATION-TYPE")) {
                            System.out.println ("------MibValueSymbol:NOTIFICATION-TYPE:" + ms.getName() + "------");
                            allOperationsMap = parseTrapsData (ms, serviceName, paramsMap, allOperationsMap);
                        }
                        else {
                            //System.out.println ("------MibValueSymbol:ELSE:" + ms.getName() + "------");
                        }
                    }
                    else {
                        //System.out.println ("**Mib Type Symbol TC: " + ms.getName());
                        //System.out.println ("**NO ----Mib Type Symbol class: " + ms.getClass().getName());
                        if (ms.getName().equals ("TEXTUAL-CONVENTION")) {
                            System.out.println ("**Mib Type Symbol TC: " + ms.toString());
                        }
                    }
                }
            }
        }

        ServicesObject so = new ServicesObject (serviceName, logicalURL, importedOntMap, allOperationsMap, null,
                                                defAbbr, defImpOnt, _textualConvMap);

        Map<String, ServicesObject> mainServicesMap = new Hashtable<String, ServicesObject>();
        mainServicesMap.put (_mib.getName(), so);

        checkIfImportedOntologiesExists (importedOntMap);

        return mainServicesMap;
    }

    /**
     * Added code to check if the imported ontologies are loaded into a web server.
     * If not it displays a message warning the user to build and load those ontologies.
     *
     * @param importedOntMap  a map with the name and url of the imported ontologies.
     */
    protected void checkIfImportedOntologiesExists (Map<String, String> importedOntMap)
    {
        Vector<String> noOntFoundVector = new Vector<String>();
        Iterator iter = importedOntMap.keySet().iterator();
        while (iter.hasNext()) {
            String key = (String) iter.next();
            String urlStr = importedOntMap.get(key);
            //System.out.println ("---*urlStr: " + urlStr);
            BufferedInputStream bis = null;
            try {
                URL url = new URL (urlStr);
                bis = new BufferedInputStream (url.openStream());
            }
            catch (Exception e) {
                noOntFoundVector.addElement (key);
            }

            if (bis == null && !noOntFoundVector.contains (key)) {
                noOntFoundVector.addElement (key);
            }
        }
        if (!noOntFoundVector.isEmpty()) {
            StringBuffer sb = new StringBuffer();
            for (String str : noOntFoundVector) {
                sb.append (str + "  \n");
            }

            JOptionPane.showMessageDialog (new JFrame(), sb.toString() + "\n Please create those ontologies and load them into the web server",
                                           "THE FOLLOWING ONTOLOGIES WERE NOT FOUND",
                                           JOptionPane.INFORMATION_MESSAGE);
        }
    }

    /**
     * Handles parsing the tables of the MIB files
     * 
     * @param ms            MibSymbol represents a table
     * @param serviceName   Mib file
     * @param paramsMap         Map of parameters of the table
     * @param allOperationsMap  Map of symbol name and its operations object
     * @return  map of allOperationsMap
     */
    protected Map<String, OperationsObject> parseTableData (MibSymbol ms, String serviceName,
                                                            Map<String, ParametersObject> paramsMap,
                                                            Map<String, OperationsObject> allOperationsMap)
    {
        MibValueSymbol value = (MibValueSymbol) ms;
        Vector<String> pmVector = new Vector<String>();
        Vector<String> typVector = new Vector<String>();
        Vector<String> owlParVector = new Vector<String>();
        Vector<String> owlRangeVector = new Vector<String>();
        Vector<Object> tcVector = new Vector<Object>();

        //String access = "";

        MibValueSymbol[] tarray = value.getChildren();
        //System.out.println ("----**Table name:symbName:<" + value.getName() + ">");
        //System.out.println ("----**Table name:symbName objID:<" + value.getValue().toObject().toString() + ">");
        String symbName = value.getName();
        String objectID = value.getValue().toObject().toString();   //This gives Object ID
        _objectIDsMap.put (symbName, objectID);

        if (tarray.length > 0) { //There is a table with elements in it.
            for (int i=0; i< tarray.length; i++) {
                //System.out.println ("----**tarray[i]:<" + tarray[i].getName() + ">");
                MibValueSymbol[] tarray2 = tarray[i].getChildren();
                //System.out.println ("------Table children:tarray2<" + tarray2.length + ">");
                if (tarray2.length > 0) { //Means the table has a sequence of elements.
                    for (int j=0; j< tarray2.length; j++) {
                        //Map tableName and map of element names with the map of types.
                        //System.out.println ("----**tarray2:<" + tarray2[j].getName() + ">"); //NameEntry on the Table
                        MibType mtype =  tarray2[j].getType();
                        //System.out.println ("------Elem types: " +  mtype.getName()); //no need
                        SnmpObjectType obType = (SnmpObjectType) mtype;
                        //System.out.println ("------Table: " +  tarray2[j].getValue());
                        MibType mtSyntx = obType.getSyntax();

                        if (mtSyntx instanceof IntegerType) {
                            IntegerType st = (IntegerType) mtSyntx;
                            MibSymbol[] msa = st.getAllSymbols();
                            Vector<String> tableElemVector = new Vector<String>();
                            for (int k=0; k< msa.length; k++) {
                                //System.out.println ("---*Table snmp obj type IntegerType: " + msa[k].getName());
                                //System.out.println ("------Table elemts: " + msa[k].getName());
                                tableElemVector.addElement (msa[k].getName());
                            }
                            //System.out.println ("---*Table tableElemVector: " +tableElemVector);
                            if (tableElemVector.size() > 0) {
                                //Check if it is a textual convention in the table
                                Iterator iter = _textualConvMap.keySet().iterator();
                                while (iter.hasNext()) {
                                    String key = (String) iter.next();
                                    //System.out.println ("---*textual conv:key: " + key);
                                    Vector<String> ttdVec = _textualConvMap.get(key);
                                    //System.out.println ("---*ttdVec textual conv: " + ttdVec);
                                    if (ttdVec.equals (tableElemVector)) {
                                        //System.out.println ("---*tcVec textual conv: " + key);
                                        tcVector.addElement (key);
                                    }
                                    else {
                                        //This element of the table can have a sequence !!!!!!!!!!!!!!!!!!!!
                                        //System.out.println ("----**tarray2:<" + tarray2[j].getName() + ">");
                                        Vector<Object> v = new Vector<Object>();
                                        Iterator it = tableElemVector.iterator();
                                        while (it.hasNext()) {
                                            String str = (String) it.next();
                                            //System.out.println ("---*tcVec adding str No TC-Is a SEQ: " + str);
                                            v.addElement (str);
                                        }
                                        //System.out.println ("---*tcVec NO textual conv: " + key);
                                        //tcVector.addElement (v); //Just deleted
                                        String tcName = getTextualConventionName (v);
                                        //System.out.println ("---*!!Seq inside table: " + v);
                                        //System.out.println ("---*!!Seq inside table:tcName: " + tcName);
                                        //System.out.println ("---*!!Seq inside table:tarray2[j].getName(): " + tarray2[j].getName());
                                        if (tcName.equals ("")) {
                                            String scalar = getScalarsNameFromMap (v);
                                            if (scalar.equals("")) {
                                                _mapOfScalars.put (tarray2[j].getName() + "Type", v);
                                                //_textualConvMap.put (tarray2[j].getName() + "Type", v);
                                                tcVector.addElement (tarray2[j].getName() + "Type");
                                            }
                                            else {
                                                System.out.println ("---*!!Seq inside table?: " + v);
                                                tcVector.addElement (scalar);
                                            }
                                        }
                                        else {
                                            tcVector.addElement (tcName);
                                        }
                                    }
                                }
                            }
                            else { //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                                //This elements are no sequences, can be Integers or Textual Conventions.
                                //System.out.println ("----**tarray2:<" + tarray2[j].getName() + ">");
                                //System.out.println ("---*Diff type from table: " + st.getName());
                                String syntax = getOwlDataTypes (obType.getSyntax().getName());
                                tcVector.addElement (syntax);
                                //System.out.println ("---*Diff type from table syntax: " + obType.getSyntax());
                                //System.out.println ("---*Diff type ot: " + value.getValue().toObject().toString());
                            }
                        }
                        else {
                            //System.out.println ("----**tarray[i] no integer type:<" + tarray[i].getName() + ">");
                            //System.out.println ("------Table snmp obj no integer type: " +  mtSyntx.getName());
                            String syntax = getOwlDataTypes (obType.getSyntax().getName());
                            tcVector.addElement (syntax);
                        }
                        objectID = tarray2[j].getValue().toObject().toString(); //This gives Object ID
                        //System.out.println ("------Table tarray2[j] objID:<" + objectID + ">");
                        _objectIDsMap.put (tarray2[j].getName(), objectID);

                        pmVector.addElement (tarray2[j].getName());

                        owlParVector.addElement (tarray2[j].getName());

                        String accessStr = obType.getAccess().toString();
                        if (accessStr != null && !accessStr.equalsIgnoreCase ("not-accessible")) {
                            //System.out.println ("------Table access2:<" + accessStr + ">");
                            typVector.addElement (accessStr);
                            owlRangeVector.addElement (accessStr);
                        }
                        else {
                            String syntax = getOwlDataTypes (obType.getSyntax().getName());
                            //System.out.println ("------Syntax:OWL:" + syntax + ">");
                            typVector.addElement (syntax);
                            owlRangeVector.addElement (syntax);
                        }
                        /*System.out.println ("------Table tarray2[j].getName(): " + tarray2[j].getName());
                        System.out.println ("------Table pmVector: " + pmVector);
                        System.out.println ("------Table owlParVector: " + owlParVector);
                        System.out.println ("------Table typVector: " + typVector);
                        System.out.println ("------Table owlRangeVector: " + owlRangeVector);*/
                    }
                }

            }
        }
        //System.out.println ("------Tables value:<" + value.getName() + ">");
        //System.out.println ("------Tables owlParVector:" + owlParVector);
        //System.out.println ("------Tables owlrangeVector:" + owlRangeVector);
        //System.out.println ("------Table:tcvector: " + tcVector);
        ParametersObject paramsObj = new ParametersObject (pmVector, typVector, owlParVector, owlRangeVector,
                                          new Vector<String>(), new Vector<String>(), tcVector);
        _mapOfTables.put (value.getName(), paramsObj);
        //_mapOfScalars.put (value.getName() + "Tbl", paramsObj);
        paramsMap.put (value.getName(), paramsObj);

        String action = "";
        String superAction = "";

        if (!symbName.equals("")) {
            action = "Access" + value.getName() + "Action";
            superAction = serviceName;

            //System.out.println ("------Tables symbName:<" + symbName + ">");
            /*System.out.println ("------Tables action:<" + action + ">");
            System.out.println ("------Tables superAction:<" + superAction + ">");
            System.out.println ("------Tables objectID:<" + objectID + ">");*/
            OperationsObject oo = new OperationsObject (symbName, action, superAction,  paramsMap,
                                                        objectID);
            allOperationsMap.put (value.getName(),  oo);
        }

        return allOperationsMap;
    }

    /**
     * Handles parsing the scalars of the MIB files
     * @param ms                MibSymbol represents a scalar
     * @param serviceName       Mib file
     * @param paramsMap         Map of parameters of the scalar
     * @param allOperationsMap  Map of symbol name and its operations object
     * @return  map of allOperationsMap
     */
    protected Map<String, OperationsObject> parseScalarData (MibSymbol ms, String serviceName,
                                                             Map<String, ParametersObject> paramsMap,
                                                             Map<String, OperationsObject> allOperationsMap)
    {
       // System.out.println ("*-----*List of scalar name:<" + ms.getName() + ">");
        MibValueSymbol value = (MibValueSymbol) ms;
        Vector<String> pmVector = new Vector<String>();
        Vector<String> typVector = new Vector<String>();
        Vector<String> owlParVector = new Vector<String>();
        Vector<String> owlRangeVector = new Vector<String>();
        Vector<Object> seqVector = new Vector<Object>();
        String access = "";

        String objectID = value.getValue().toObject().toString();   //This gives Object ID
        String symbName = value.getName();
       // System.out.println ("*-----*List of scalar symbName:<" + symbName + ">");
        _objectIDsMap.put (symbName, objectID);

        SnmpType stype = (SnmpType) value.getType();
        SnmpObjectType sot = (SnmpObjectType) stype;
        //System.out.println ("------Scalar Type: " + sot.getName());

        MibType mtSyntx = sot.getSyntax();
        Vector<String> scalarElemVector = new Vector<String>();
        //Scalars can have sequences associated with the syntax which are translated as textual conventions
        if (mtSyntx instanceof IntegerType) {
            //System.out.println ("**integer scalar name: " + symbName + "----synt: " + mtSyntx);
            IntegerType st = (IntegerType) mtSyntx;
            MibSymbol[] msa = st.getAllSymbols();
            //System.out.println ("**msa lenght: " + msa.length);
            if (msa.length > 0) {
                for (int k=0; k< msa.length; k++) {
                    //System.out.println ("---*scalarElemVector type IntegerType: " + msa[k].getName());
                    scalarElemVector.addElement (msa[k].getName());
                }
                if (scalarElemVector.size() > 0) {
                    //System.out.println ("---*scalarElemVector type IntegerType Adding to TCmap: " + ms.getName() + " and vec: " + scalarElemVector);
                    _textualConvMap.put (ms.getName() + "Type", scalarElemVector);
                    _mapOfScalars.put (ms.getName() + "Type", scalarElemVector);
                    seqVector.addElement (scalarElemVector);
                }
            }
            else {
                String synt = getOwlDataTypes (mtSyntx.getName());
                //System.out.println ("integer no sequence scalar name: " + symbName + "----synt: " + synt);
                scalarElemVector.addElement(synt);
                _mapOfScalars.put (ms.getName()+ "Type", scalarElemVector);
            }
        }
        else {
            String synt = getOwlDataTypes (mtSyntx.getName());
            //System.out.println ("*No integer scalar name: " + symbName + "----synt: " + synt);
            scalarElemVector.addElement(synt);
            _mapOfScalars.put (ms.getName()+ "Type", scalarElemVector);
        }

        if (sot == null) {
            System.out.println ("*SnmpObjectType is null");
        }
        else {
            //Add access and objectId as param and type for now !!!!
            access = sot.getAccess().toString();
            //System.out.println ("------Scalar Name: " + symbName + " access:<" + access + ">");
        }
        String action = "Access" + value.getName() + "Action";
        String superAction = serviceName;

        if (access != null && !access.equals("") ) {
            //System.out.println ("------* access to parse:<" + access + ">");
            String[] strArray = parseMibAccessStr (access);
            //System.out.println ("------* access to parse strArray length:<" + strArray.length + ">");
            if (strArray.length > 1) {
                for (int i=0; i< strArray.length; i++) {
                    //System.out.println ("------strArray[i]: " + strArray[i]);
                    pmVector.addElement (strArray[i]);
                    owlParVector.addElement (strArray[i]);
                    typVector.addElement (strArray[i]);
                    owlRangeVector.addElement (strArray[i]);
                }
            }
            else if (strArray.length > 0) {
                //System.out.println ("------strArray[0]: " + strArray[0]);
                pmVector.addElement (strArray[0]);
                owlParVector.addElement (strArray[0]);
                typVector.addElement (strArray[0]);
                owlRangeVector.addElement (strArray[0]);
            }
            ParametersObject paramsObj = new ParametersObject (pmVector, typVector, owlParVector, owlRangeVector,
                                              new Vector<String>(), new Vector<String>(), seqVector);
            if (paramsObj != null) {
                _mapOfScalarObjects.put (symbName, paramsObj);
                paramsMap.put (symbName, paramsObj);

                //System.out.println ("------scalar to map: " + symbName);

                OperationsObject oo = new OperationsObject (symbName, action, superAction,  paramsMap,
                                                             objectID);
                allOperationsMap.put (symbName,  oo);
            }
        }
        return allOperationsMap;
    }

    /**
     * Handles parsing the Mib traps
     * @param ms                MibSymbol represents a traps
     * @param serviceName       Mib file
     * @param paramsMap         Map of parameters of the traps
     * @param allOperationsMap  Map of symbol name and its operations object
     * @return  map of allOperationsMap
     */
    protected Map<String, OperationsObject> parseTrapsData (MibSymbol ms, String serviceName,
                                                            Map<String, ParametersObject> paramsMap,
                                                            Map<String, OperationsObject> allOperationsMap)
    {
        MibValueSymbol value = (MibValueSymbol) ms;

        String objectID = value.getValue().toObject().toString();
        _objectIDsMap.put (ms.getName(), objectID);
        //System.out.println ("------ASymbName:trap: " + symbName + " .Object ID:trap: " + objectID);
        SnmpType stype = (SnmpType) value.getType();

        if (stype instanceof SnmpNotificationType) {
            System.out.println ("----------------------------------------------------------------------------");
            //System.out.println ("**SnmpNotificationType------");
            //String keyObjName = "";
            Vector<String> owlParVector = new Vector<String>();
            Vector<String> owlRangeVector = new Vector<String>();
            Vector<Object> tcVector = new Vector<Object>();

            SnmpNotificationType snt = (SnmpNotificationType) stype;
            ArrayList al = snt.getObjects();
            //System.out.println ("**SnmpNotificationType:Array of objects " + al.size());
            if (al.size() > 0) {
                for (Object objID : al) {
                    //Object ID of the object defined by the trap.
                    //System.out.println ("**Object ID of the object defined: " + objID);
                    Iterator itert = _objectIDsMap.keySet().iterator();
                    //System.out.println ("**_objectIDsMap size: " + _objectIDsMap.size());
                    while (itert.hasNext()) {
                        String keyObjName = (String) itert.next();
                        //System.out.println ("**********************keyObjName from ObjIDMap: " + keyObjName);
                        String id = _objectIDsMap.get (keyObjName);
                        //System.out.println ("**id: " + id);
                        //System.out.println ("**Object ID of the object defined: " + objID);
                        if (!objID.toString().equals("") && (id.equals (objID.toString()))) {
                            //This is the object of the trap
                            owlParVector.addElement (keyObjName);
                        }
                    }
                }
            }
            System.out.println ("**TRAPowlParVector: " + owlParVector);
            if (owlParVector.size() > 0) {
                //for (String keyObjName : owlParVector) {
                for (int i=0; i<owlParVector.size(); i++) {
                    String keyObjName = owlParVector.elementAt(i);
                    //System.out.println ("------NoRange/NoTC>>>owlParVector:" + owlParVector);
                    //System.out.println ("------NoRange/NoTC>>>owlParVector.element at 0:<" + owlParVector.elementAt(0) + ">");

                    //Needs to find where the object of the trap is defined.
                    //Check if it is a scalar with sequences!!!!
                    System.out.println ("------keyObjName:" + keyObjName);
                    if (!keyObjName.contains ("Type")) {
                        keyObjName = keyObjName + "Type";
                    }
                    //System.out.println ("------_mapOfScalars:" + _mapOfScalars);
                    if (_mapOfScalars.containsKey (keyObjName)) {
                        System.out.println ("------_mapOfScalars contains key");
                        Iterator iter = _mapOfScalars.keySet().iterator();
                        while (iter.hasNext()) {
                            String name = (String) iter.next();
                            //System.out.println ("------*scalars Name:<" + name +">------");
                            //System.out.println ("------*scalars keyObjName:<" + keyObjName +">------");
                            if (keyObjName.equals (name)) {
                                //System.out.println ("------MATCH!! scalars Name:" + name + "--keyObjName: " + keyObjName+ "------");
                                tcVector.addElement (name); //??????
                                ParametersObject pobj = getScalarObjectFromMap (name);
                                //System.out.println ("------Scalar po :" + pobj + "------");
                                if (pobj != null) {
                                    String range = pobj._owlPropertyRange.elementAt (0);
                                    //System.out.println ("------Scalar range :" + range + "------");
                                    if (range.equals ("read")) {
                                        range = name + "Read";
                                    }
                                    else if (range.equals ("write")) {
                                        range = name + "Write";
                                    }
                                    owlRangeVector.addElement (range);
                                    Object elemObj = pobj._sequencesVector;
                                    System.out.println ("------Scalar elemObj :" + pobj._sequencesVector + "------");
                                    if (elemObj != null) {
                                        if (elemObj instanceof Vector) {
                                            Vector elemVector = (Vector) elemObj;
                                            //Check the textual Conventions map to find the name
                                            String tcName = getTextualConventionName (elemVector);
                                            if (tcName.equals("")) { //Check the scalars map
                                                String scalar =  getScalarsNameFromMap (elemVector);
                                                if (!scalar.equals("")) {
                                                    tcVector.addElement (scalar);
                                                }
                                            }
                                            else {
                                                tcVector.addElement (tcName);
                                            }
                                        }
                                        else {
                                            tcVector.addElement (elemObj.toString());
                                        }
                                    }
                                }
                                else {
                                    //It is a scalar but is part of a table. po == null
                                    System.out.println ("------*Po is null:scalars keyObjName:<" + keyObjName +">------");
                                    Vector<Object> vecOfObj = checkTableMap (keyObjName, owlRangeVector, tcVector);
                                    keyObjName = vecOfObj.elementAt(0).toString();
                                    owlRangeVector = (Vector<String>) vecOfObj.elementAt(1);
                                    tcVector =  (Vector<Object>)  vecOfObj.elementAt(2);
                                }
                            }
                            else {
                                //System.out.println ("------*ELSE:scalars keyObjName:<" + keyObjName +">------");
                            }
                        }
                    }
                    else {
                        //Check if it is a parameter in a table
                        //System.out.println ("------It is a table keyObj:" + keyObjName + "------");
                        if (keyObjName.contains ("Index")) {
                            String[] sa = keyObjName.split("Index");
                            if (sa.length > 0) {
                                keyObjName = sa[0] + "Table";
                            }
                        }
                        else if (keyObjName.contains ("Type")) {
                            String[] sa = keyObjName.split ("Type");
                            if (sa.length > 0) {
                                keyObjName = sa[0];
                            }
                        }

                        if (_mapOfTables.containsKey (keyObjName)) {
                            //System.out.println ("------It is a KEY in Table keyObj:" + keyObjName + "------");
                            ParametersObject po = _mapOfTables.get (keyObjName);
                            if (po == null) {
                                continue;
                            }
                            //Vector<String> parVector = po._owlPropertyName;
                            Vector<String> rgeVector = po._owlPropertyRange;
                            Vector<Object> seqVector = po._sequencesVector;

                            if (rgeVector.size() > 0) {
                                String range = rgeVector.elementAt (0);
                                System.out.println ("------rangeR:" + range + "------");
                                if (range.contains ("write")) {
                                    //System.out.println ("------rangeW:" + tableName + "Write" + "------");
                                    owlRangeVector.addElement (keyObjName + "Write");
                                }
                                else if (range.contains ("read-only")){
                                    //System.out.println ("------rangeR:" + tableName + "Read" + "------");
                                    owlRangeVector.addElement (keyObjName + "Read");
                                }
                                else {
                                    //System.out.println ("------rangeR:no read No write: " + keyObjName
                                    //                     + "------range: " + range);
                                    owlRangeVector.addElement (range);
                                }
                            }
                            if (tcVector.size() > 0) {
                                tcVector.addElement (seqVector.elementAt(0));
                            }

                        }
                        else {
                            Iterator ite = _mapOfTables.keySet().iterator();
                            while (ite.hasNext()) {
                                String tableName = (String) ite.next();
                                //System.out.println ("------tableName:" + tableName + "------");
                                //System.out.println ("------keyObj:" + keyObjName + "------");
                                ParametersObject po = _mapOfTables.get (tableName);
                                if (po == null) {
                                    continue;
                                }
                                Vector<String> parVector = po._owlPropertyName;
                                Vector<String> rgeVector = po._owlPropertyRange;
                                Vector<Object> seqVector = po._sequencesVector;

                                //Check if it is a parameter of this table
                                for (int k=0; k<parVector.size(); k++) {
                                    if (parVector.elementAt(k).equals (keyObjName)) {
                                        //If Found, get the range
                                        String range = rgeVector.elementAt(k); ///??????i
                                        //System.out.println ("------rangeR:" + range + "------");
                                        if (range.contains ("write")) {
                                            //System.out.println ("------rangeW:" + tableName + "Write" + "------");
                                            owlRangeVector.addElement (tableName + "Write");
                                        }
                                        else if (range.contains ("read-only")){
                                            System.out.println ("------rangeR:" + tableName + "Read" + "------");
                                            owlRangeVector.addElement (tableName + "Read");
                                        }
                                        else {
                                            /*System.out.println ("------rangeR:no read No write: " + tableName
                                                                 + "------range: " + range);*/
                                            owlRangeVector.addElement (range);
                                        }
                                        //Check for sequences
                                        Object elemObj = seqVector.elementAt(k);
                                        //System.out.println ("------TrapName:elemObj: " + elemObj);
                                        if (elemObj != null) {
                                            if (elemObj instanceof Vector) {
                                                Vector elemVector = (Vector) elemObj;
                                                //Check the textual Conventions map to find the name
                                                String tcName = getTextualConventionName (elemVector);
                                                if (tcName.equals("")) { //Check the scalars map
                                                    String scalar =  getScalarsNameFromMap (elemVector);
                                                    if (!scalar.equals("")) {
                                                        tcVector.addElement (scalar);
                                                    }
                                                }
                                                else {
                                                    tcVector.addElement (tcName);
                                                }
                                            }
                                            else {
                                                tcVector.addElement (seqVector.elementAt(k));
                                            }
                                        }
                                    }
                                }
                            } //end of the while
                        }//end else
                    } //table
                }
                //Get the map of operations
                String action = "Receive" + ms.getName() + "Action";
                String superAction = serviceName;
                String trapName = "";
                if (!ms.getName().contains("Trap")) {
                    trapName = ms.getName() + "Trap";
                }
                else {
                    trapName = ms.getName();
                }

                /*System.out.println ("------TrapName: " + trapName);
                System.out.println ("------Trap owlParVector:" + owlParVector); //holds the trap objects name
                System.out.println ("------Trap owlrangeVector:" + owlRangeVector); //Holds where the trap objects are defined
                System.out.println ("------Trap:tcvector: " + tcVector); //Holds the type of each object.
                System.out.println ("-------------------------------------------------");*/

                ParametersObject paramsObj = new ParametersObject (new Vector<String>(), new Vector<String>(),
                                                                  owlParVector, owlRangeVector, new Vector<String>(),
                                                                  new Vector<String>(), tcVector);

                paramsMap.put (trapName, paramsObj);

                OperationsObject oo = new OperationsObject (trapName, action, superAction,  paramsMap,
                                                            objectID);
                allOperationsMap.put (trapName,  oo);
            }
            else {
                //do nothing
            }
        }
        return allOperationsMap;
    }

    protected Vector<Object> checkTableMap (String keyObjName,  Vector<String> owlRangeVector,
                                            Vector<Object> tcVector)
    {
        //Check if it is a parameter in a table
        //System.out.println ("------It is a table keyObj:" + keyObjName + "------");
        if (keyObjName.contains ("Index")) {
            String[] sa = keyObjName.split("Index");
            if (sa.length > 0) {
                keyObjName = sa[0] + "Table";
            }
        }
        else if (keyObjName.contains ("Type")) {
            String[] sa = keyObjName.split ("Type");
            if (sa.length > 0) {
                keyObjName = sa[0];
            }
        }

        if (_mapOfTables.containsKey (keyObjName)) {
            //System.out.println ("------It is a KEY in Table keyObj:" + keyObjName + "------");
            ParametersObject po = _mapOfTables.get (keyObjName);
            if (po == null) {
                //continue;
            }
            //Vector<String> parVector = po._owlPropertyName;
            Vector<String> rgeVector = po._owlPropertyRange;
            Vector<Object> seqVector = po._sequencesVector;

            if (rgeVector.size() > 0) {
                String range = rgeVector.elementAt (0);
                //System.out.println ("------rangeR:" + range + "------");
                if (range.contains ("write")) {
                    //System.out.println ("------rangeW:" + tableName + "Write" + "------");
                    owlRangeVector.addElement (keyObjName + "Write");
                }
                else if (range.contains ("read-only")){
                    //System.out.println ("------rangeR:" + tableName + "Read" + "------");
                    owlRangeVector.addElement (keyObjName + "Read");
                }
                else {
                    //System.out.println ("------rangeR:no read No write: " + keyObjName
                    //                     + "------range: " + range);
                    owlRangeVector.addElement (range);
                }
            }
            if (tcVector.size() > 0) {
                tcVector.addElement (seqVector.elementAt(0));
            }
        }
        else {
            Iterator ite = _mapOfTables.keySet().iterator();
            while (ite.hasNext()) {
                String tableName = (String) ite.next();
                //System.out.println ("------tableName:" + tableName + "------");
                //System.out.println ("------keyObj:" + keyObjName + "------");
                ParametersObject po = _mapOfTables.get (tableName);
                if (po == null) {
                    continue;
                }
                Vector<String> parVector = po._owlPropertyName;
                Vector<String> rgeVector = po._owlPropertyRange;
                Vector<Object> seqVector = po._sequencesVector;

                //Check if it is a parameter of this table
                for (int k=0; k<parVector.size(); k++) {
                    if (parVector.elementAt(k).equals (keyObjName)) {
                        //If Found, get the range
                        String range = rgeVector.elementAt(k);
                        //System.out.println ("------rangeR:" + range + "------");
                        if (range.contains ("write")) {
                            //System.out.println ("------rangeW:" + tableName + "Write" + "------");
                            owlRangeVector.addElement (tableName + "Write");
                        }
                        else if (range.contains ("read-only")){
                            //System.out.println ("------rangeR:" + tableName + "Read" + "------");
                            owlRangeVector.addElement (tableName + "Read");
                        }
                        else {
                            /*System.out.println ("------rangeR:no read No write: " + tableName
                                                 + "------range: " + range);*/
                            owlRangeVector.addElement (range);
                        }
                        //Check for sequences
                        Object elemObj = seqVector.elementAt(k);
                        //System.out.println ("------TrapName:elemObj: " + elemObj);
                        if (elemObj != null) {
                            if (elemObj instanceof Vector) {
                                Vector elemVector = (Vector) elemObj;
                                //Check the textual Conventions map to find the name
                                String tcName = getTextualConventionName (elemVector);
                                if (tcName.equals("")) { //Check the scalars map
                                    String scalar =  getScalarsNameFromMap (elemVector);
                                    if (!scalar.equals("")) {
                                        tcVector.addElement (scalar);
                                    }
                                }
                                else {
                                    tcVector.addElement (tcName);
                                }
                            }
                            else {
                                tcVector.addElement (seqVector.elementAt(k));
                            }
                        }
                    }
                }
            } //end of the while
        }//end else

        Vector<Object> voo = new Vector<Object>();
        voo.addElement(keyObjName);
        voo.addElement(owlRangeVector);
        voo.addElement(tcVector);

        return voo;
    }



    /**
     * Finds the name of the textual convention which vector of data is equal to this vector
     *
     * @param tcVector:             Vector of data to search the name of the textual convention
     * @return:                     the name of the textual convention
     */
    protected String getTextualConventionName (Vector<Object> tcVector)
    {
        String tcName = "";
        Iterator iter = _textualConvMap.keySet().iterator();
        while (iter.hasNext()) {
            String key = (String) iter.next();
            //System.out.println ("---*textual conv:key: " + key);
            Vector<String> texConvVector = _textualConvMap.get (key);
            //System.out.println ("---*textual conv:texConvVector: " + texConvVector);
            if (texConvVector.equals (tcVector)) {
                //System.out.println ("---*FOUND!!!!!!**textual conv:key: " + key);
                tcName = key;
            }
            else {
                tcName = "";
            }
        }
        return tcName;
    }

    /**
    * Finds the name of the scalar which vector of data is equal to this vector
    *
    * @param scalarVector:             Vector of data to search the name of the textual convention
    * @return:                     the name of the textual convention
    */
    protected String getScalarsNameFromMap (Vector<Object> scalarVector)
    {
       String tcName = "";
       Iterator iter = _mapOfScalars.keySet().iterator();
       while (iter.hasNext()) {
           String key = (String) iter.next();
           //System.out.println ("---*textual conv:key: " + key);
           Vector<String> scVector = _mapOfScalars.get (key);
           //System.out.println ("---*ScalarVector: " + scVector);
           if (scVector.equals (scalarVector)) {
               //System.out.println ("---*FOUND!!!!!!**Scalar:key: " + key);
               tcName = key;
           }
           else {
               tcName = "";
           }
       }
       return tcName;
    }

    /**
     * Gets the parameters object from the scalars map for parsing traps.
     *
     * @param scalarName
     * @return the parameters object
     */
    protected ParametersObject getScalarObjectFromMap (String scalarName)
    {
        ParametersObject po = null;
        if (scalarName.contains("Type")) {
            String[] sa = scalarName.split("Type");
            if (sa.length > 0) {
                scalarName = sa[0];
            }
        }
        Iterator iter = _mapOfScalarObjects.keySet().iterator();
        while (iter.hasNext()) {
            String key = (String) iter.next();
            //System.out.println ("---*Scalar conv:key: " + key);
            if (key.equals (scalarName)) {
                po = _mapOfScalarObjects.get (key);
            }
        }
        return po;
    }


     /**
     * Parses the the access when it contains two words separated by -.
     * Example: read-write
     *
     * @param access
     * @return a string array of the elements .
     */
    public String[] parseMibAccessStr (String access)
    {
        String[] accessArray = null;
        if (access.contains ("write")) {
            accessArray = new String [access.length()];
            String accessStr = access.substring (0, access.length());
            accessArray = accessStr.split ("-");
            //System.out.println ("**accessArray: " + accessArray.length);
        }
        else if (access.contains ("read")) {
            accessArray = new String [access.length()];
            String accessStr = access.substring (0, access.length());
            accessArray = accessStr.split ("-");
            //System.out.println ("**accessArray 1: " + accessArray[1]);
            //System.out.println ("**accessArray: " + accessArray.length);
        }
        else {
            String accessStr = access.substring (0, access.length());
            accessArray = new String[1];
            accessArray[0] = accessStr;
        }

        if (accessArray.length > 1) {
            if (accessArray[1].equalsIgnoreCase ("only")) {
                //System.out.println ("**accessArray 1: " + accessArray[1]);
                String[] sArr = new String [1];
                sArr[0] = accessArray[0];
                return sArr;
            }
            else {
                return accessArray;
            }
        }
        else {
            return new String[0];
        }
    }

    /**
     * Gets the Snmp object type for this type
     * @param type
     */
    protected void getSnmpObjectType (SnmpObjectType type)
    {
        System.out.println ("**SYNTAX: " + type.getSyntax());
        if (type.getUnits() != null) {
            System.out.println ("**UNITS: " + type.getUnits());
        }
        System.out.println ("**ACCESS: " + type.getAccess());
        //System.out.println ("**STATUS: " + type.getStatus());
        if (type.getDescription() != null) {
            //System.out.println ("**Description: " + type.getDescription());
        }
        if (type.getReference() != null) {
            System.out.println ("**REFERENCE: " + type.getReference());
        }
        if (type.getIndex() != null && type.getIndex().size() > 0) {
            System.out.println ("**INDEX: " + type.getIndex());
        }
        if (type.getAugments() != null) {
            System.out.println ("**AUGMENTS: " + type.getAugments());
        }
        if (type.getDefaultValue() != null) {
            System.out.println ("**DEFVAL: " + type.getDefaultValue());
        }
    }

    /**
     * Gets Mib textual convention types.
     *
     * @param   stype    the MibType
     * @return  tcVector    a vector of names for each type
     */
    protected Vector<String> getMIBTextualConventions (MibType stype, MibSymbol ms)
    {
        Vector<String> tcVector = new Vector<String>();
        if (stype instanceof IntegerType) {
            IntegerType it = (IntegerType) stype;
            MibValueSymbol[] mvs = it.getAllSymbols();
            if (mvs.length > 0) {
                for (MibSymbol msym : mvs) {
                    //System.out.println ("**It is a IntegerType***: " + msym.getName());
                    tcVector.addElement (msym.getName());
                }
            }
            else {
                tcVector.addElement ("INTEGER");
            }
        }
        else if (stype instanceof BitSetType) {
            //System.out.println ("**It is a BitSetType***: " + stype.getName());
            BitSetType bt = (BitSetType) stype;
            MibValueSymbol[] mvs = bt.getAllSymbols();
            if (mvs.length > 0) {
                for (MibSymbol msym : mvs) {
                    //System.out.println ("**It is a BitSetType***: " + msym.getName());
                    //System.out.println ("**It is a BitSetType***Constraint: " + bt.getConstraint().toString());
                    tcVector.addElement (msym.getName());
                }
            }
        }
        else if (stype instanceof StringType) {
            StringType st = (StringType) stype;
            //System.out.println ("**It is a StringType***: " + stype.getName());
            if (stype.getName().equalsIgnoreCase ("OCTET STRING")) {
                tcVector.addElement ("STRING");
            }
            //System.out.println ("**It is a StringType***: " + st.getName());
            if (st.hasConstraint()) {
                Constraint constr = st.getConstraint();
                //tcVector.addElement (constr.toString());    //Not added for now
                //System.out.println ("**It is a constr***: " + constr.toString()); //Gives the size of string.
            }
        }
        else {
            System.out.println ("**Unknown textual convention type**: " + stype.getName());
            //MibTypeSymbol value = stype.getReferenceSymbol();
            if (ms instanceof MibValueSymbol) {
                System.out.println ("**NO ----Mib Type Symbol class2: " + ms.getClass().getName());
                MibValueSymbol value = (MibValueSymbol) ms;
                if (value.isScalar()) {
                    System.out.println ("----**OBJECT-TYPE It is a Scalar :symbName:<" + value.getName() + ">");
                }
                else if (value.isTable()) {
                    System.out.println ("----**OBJECT-TYPE: It is a Table name:symbName:<" + value.getName() + ">");
                }
            }
            else {
                //System.out.println ("------MibValueSymbol:OBJECT-TYPE: No table No scalar: " + ms.getName() + "------");
            }
        }

        return tcVector;
    }

    protected String getOwlDataTypes (String syntax)
    {
        String synx = "";
        if (syntax.equalsIgnoreCase("INTEGER")) {
            synx = "xsd:int";
        }
        else if (syntax.equalsIgnoreCase ("OCTET STRING")) {
            synx = "xsd:string";
        }
        else {
            synx = syntax;
        }
        return synx;
    }

    public class SequenceTypeObject extends Object
    {
        public SequenceTypeObject (String stName, String seqName, String seqType)
        {
            _stName = stName;
            _seqName = seqName;
            _seqType = seqType;
        }
        protected String _stName; //Sequence type name
        protected String _seqName; //Sequence element name
        protected String _seqType; //Sequence element type
    }

    public class TableDataObject extends Object
    {
        public TableDataObject (String stName, HashMap<String, Vector> tableDataTypes)
        {
            _stName = stName;
            _tableDataTypes = tableDataTypes;
        }
        protected String _stName; //Table name
        protected HashMap<String, Vector> _tableDataTypes = new HashMap<String, Vector>(); //Maps the element
                                                                                            //names with a vector of
                                                                                            //their objectID and access
    }

    //Variables
    protected Mib _mib;
    protected HashMap<String,String> _objectIDsMap;             //Maps the mib symbol with its object ID
    protected HashMap<String,ParametersObject> _mapOfTables;    //Maps the table name with all the parameters
    protected HashMap<String,Vector> _mapOfScalars;             //Maps the scalar name with its elements
    protected HashMap<String,ParametersObject> _mapOfScalarObjects; //Maps the scalar name with its parameter object for traps.
    protected HashMap<String,Vector> _textualConvMap;           //Maps the name of the textual convention with its elements.
}
