package kaos.tools.webServiceTool;

import kaos.core.csi.ontology.OntologyService;
import kaos.ontology.repository.InternalOntologyRepository;
import kaos.core.csi.query.QueryState.UnknownConceptException;

import javax.agent.service.directory.DirectoryFailure;
import javax.swing.text.*;
import javax.swing.*;
import java.util.Map;
import java.util.Set;
import java.util.Vector;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

/**
 * Class that performs the code completion in the text fields after
 * the user type a ":".
 *
 * @author Maggie Breedy <mbreedy@ihmc.us>
 * @version: $Revision: 1.7 $
 *
 */
public class TextAutoCompleter extends AutoCompleter
{
    public TextAutoCompleter (JTextComponent comp, boolean usingParamTable, Map<String, ServicesObject> mainServicesMap,
                              Map<String, OntologyService> ontologyRepMap, int columnSelected, int rowSelected,
                              ServicesManagementTool servicesTool)
    {
        super (comp);
        _usingParamsTable = usingParamTable;
        _mainServicesMap = mainServicesMap;
        _ontologyRepMap = ontologyRepMap;
        _columnSelected = columnSelected;
        _rowSelected = rowSelected;
        _servicesTool = servicesTool;
        _textComp = comp;
    }

    protected boolean updateListData()
    {
        String value = _textComp.getText();
        int index = value.indexOf(':');
        if (index == -1) {
            _masterItemsList = null;
            return false;
        }

        //if first time seeing ':' then get the master list.
        if (_masterItemsList == null) {
            Set actionClassesSet = null;
            Vector<String> v = new Vector<String>();
            String abbr = value.substring (0, index);
            //System.out.println ("Building master list from:" + abbr);
            Map<String, String> impMap = getImpMap();
            Iterator itr = impMap.keySet().iterator();
            while (itr.hasNext()) {
                // Get key Object
                String impAbbr = (String) itr.next();
                //System.out.println ("Building master list from impAbbr:" + impAbbr);
                if (abbr.equalsIgnoreCase (impAbbr)) {
                    String impURL = impMap.get (impAbbr);
                    //System.out.println ("************impURL:" + impURL);
                    int idnx = impURL.lastIndexOf ("/");
                    int indx = impURL.lastIndexOf (".owl");

                    String impURLStr = impURL.substring (idnx+1, indx);
                    //System.out.println ("************impURLStr:" + impURLStr);
                    //System.out.println ("Building master list from:" + impURL + "#" + impURLStr);
                    try {
                        //System.out.println ("**_usingParamsTable: " + _usingParamsTable);
                        if (_usingParamsTable == false) {
                            //System.out.println ("**_ontologyRepMap: " + _ontologyRepMap);
                            OntologyService ontRep = _ontologyRepMap.get (abbr);
                            actionClassesSet = ontRep.getSubClassesOf (impURL + "#" + impURLStr);
                            if (actionClassesSet.size() > 0) {
                                _masterItemsList = new String [actionClassesSet.size()];
                                //System.out.println ("**actionClasses size: " + actionClassesSet.size());
                                Iterator it = actionClassesSet.iterator();
                                while (it.hasNext()) {
                                    String actionStr = (String) it.next();
                                    v.addElement (getShortName (actionStr));
                                }
                            }
                        }
                        else {     //Code completion for the params table
                            int idx = 0;
                            String superAction = _servicesTool.getSuperActionText();
                            String action = _servicesTool.getActionText();
                            _wsdlTableModel = _servicesTool.getParamsTableModel();
                            if (action.contains (":")) {
                                //System.out.println ("**action contains the char: " + superAction);
                                idx = action.indexOf (':');
                                if (abbr.equalsIgnoreCase (action.substring (0, idx))) {
                                    String classStr = action.substring (idx+1, action.length());
                                    //System.out.println ("**classStr: " + classStr);
                                    String url = impMap.get (abbr);
                                    OntologyService ontRep = _ontologyRepMap.get (abbr);

                                    //System.out.println ("**_columnSelected: " + _columnSelected);
                                    //System.out.println ("**_rowSelected: " + _rowSelected);
                                    if (_columnSelected == 2 ) {
                                        //System.out.println ("**action class url: " + url + "#" + classStr);
                                        List<String> list = ontRep.getPropertiesApplicableTo (url + "#" + classStr);
                                        //System.out.println ("**action list: " + list.size());
                                        if (list == null || list.isEmpty()) {
                                            v.addElement ("No data available");
                                        }
                                        else {
                                            Iterator it = list.iterator();
                                            while (it.hasNext()) {
                                                String str = (String) it.next();
                                                v.addElement (getShortName (str));
                                            }
                                        }
                                    }
                                    else if (_columnSelected == 3) {
                                        //Get property name from column 2.
                                        //System.out.println ("**_wsdlTableModel: " + _wsdlTableModel);
                                        String properName = _wsdlTableModel.getValueAt (_rowSelected, 2).toString();
                                        //System.out.println ("**properName: " + properName);
                                        String range = ontRep.getRangeOnPropertyForClass (url + "#" + classStr, url
                                                                                          + "#" + properName);
                                        if (range == null) {
                                            v.addElement ("No data available");
                                        }
                                        Set classesOfRange = ontRep.getSubClassesOf (range);
                                        if (classesOfRange.size() > 0) {
                                            //System.out.println ("**classesOfRange size: " + classesOfRange.size());
                                            Iterator it = classesOfRange.iterator();
                                            while (it.hasNext()) {
                                                String rangeStr = (String) it.next();
                                                v.addElement (getShortName (rangeStr));
                                            }
                                        }
                                        else {
                                             v.addElement (getShortName (range));
                                        }
                                    }
                                }
                            }
                            else if (superAction.contains(":")) {
                                //System.out.println ("**superAction contains the char: " + superAction);
                                idx = superAction.indexOf (':');
                                if (abbr.equalsIgnoreCase (superAction.substring (0, idx))) {
                                    //System.out.println ("**superAction abbr: " + abbr);
                                    String classStr = superAction.substring (idx+1, superAction.length());
                                    String url = impMap.get (abbr);
                                    OntologyService ontRep = _ontologyRepMap.get (abbr);
                                    if (_columnSelected == 2 ) {
                                        List<String> list = ontRep.getPropertiesApplicableTo (url + "#" + classStr);
                                        //System.out.println ("**superAction list: " + list.size());
                                        if (list == null || list.isEmpty()) {
                                            v.addElement ("No data available");
                                        }
                                        else {
                                            Iterator it = list.iterator();
                                            while (it.hasNext()) {
                                                String str = (String) it.next();
                                                //System.out.println ("**superAction str: " + str);
                                                v.addElement (getShortName (str));
                                            }
                                        }
                                    }
                                    else if (_columnSelected == 3) {
                                        String properName = _wsdlTableModel.getValueAt (_rowSelected, 2).toString();
                                        String range = ontRep.getRangeOnPropertyForClass (url + "#" + classStr, url
                                                                                          + "#" +properName);
                                        if (range == null) {
                                            v.addElement ("No data available");
                                        }
                                        Set spclassesOfRange = ontRep.getSubClassesOf (range);
                                        if (spclassesOfRange.size() > 0) {
                                            //System.out.println ("**spclassesOfRange size: " + spclassesOfRange.size());
                                            Iterator it = spclassesOfRange.iterator();
                                            while (it.hasNext()) {
                                                String rangeStr = (String) it.next();
                                                v.addElement (getShortName (rangeStr));
                                            }
                                        }
                                        else {
                                             v.addElement (getShortName (range));
                                        }
                                    }
                                }
                            }
                            else {
                                 v.addElement ("No data available");
                            }
                        }
                    }
                    catch (OntologyService.ReasoningException re) {
                        System.out.println ("Error: Inconsistency in the ontology");
                        JOptionPane.showMessageDialog (new JFrame(), "Error: Inconsistency in the ontology");
                        re.printStackTrace();
                    }
                    catch (UnknownConceptException uce) {
                        System.out.println ("Error retrieving the Action Classes");
                        JOptionPane.showMessageDialog (new JFrame(), "Error retrieving the Action Classes");
                    }
                    catch (DirectoryFailure e)
                    {
                        System.out.println ("Error retrieving the Action Classes");
                        JOptionPane.showMessageDialog (new JFrame(), "Error retrieving the Action Classes");
                    }
                }
            }
            _masterItemsList = new String [v.size()];
            for (int i=0; i<v.size(); i++) {
                _masterItemsList[i] = v.elementAt(i);
            }
            //_masterItemsList = new String[] {"item0", "item1", "item2", "item3"};
        }

        // Pare down the list to only those items starting with what the user has typed in.
        String prefix = index + 1 >= value.length() ? "" : value.substring (index+1);
        //System.out.println ("Paring with prefix: " + prefix);
        Vector<String> v = new Vector<String>(_masterItemsList.length);
        for (int i = 0; i < _masterItemsList.length; i++) {
            if (_masterItemsList[i].toLowerCase().startsWith(prefix.toLowerCase())) {
                v.add (_masterItemsList[i]);
            }
        }
        //_usingParamsTable = false;
        String[] items = new String [v.size()];
        v.toArray (items);

        //If there are not items in the list or if the only item matches the typed in value
        // then return new String[0]
        if (items == null){
           _list.setListData (new String[0]);
           return true;
        }
        else {
           if(items.length==1 && items[0].equalsIgnoreCase(prefix)) {
               _list.setListData (new String[0]);
           }
           else {
               //There are items to display so set the list data
               _list.setListData (items);
           }
           return true;
       }
    }

    protected  Map<String, String> getImpMap()
    {
        String serviceName = _servicesTool.getServiceName();

        ServicesObject servObj = _mainServicesMap.get (serviceName);
        Map<String, String> importedOntMap = servObj._importedOntMap;
        return importedOntMap;
    }

    protected void acceptedListItem(String selected)
    {
        if(selected==null) {
           return;
        }

        String value = _textComp.getText();
        int index = value.indexOf (':');
        //System.out.println ("index: " + index);

        if (index == -1) {
           return;
        }
        int prefixlen = _textComp.getDocument().getLength()-index-1;
        try {
           _textComp.getDocument().insertString (_textComp.getCaretPosition(), selected.substring(prefixlen), null);
        }
        catch (BadLocationException e){
           e.printStackTrace();
        }
    }

    /**
     *  Return the short name for this url.
     */
    protected String getShortName (String name)
    {
        if (name != null) {
            //parse shortname by taking everything after the #
            for (StringTokenizer t = new StringTokenizer (name, "#"); t.hasMoreTokens(); ) {
                name = t.nextToken();
            }
        }
        else {
            System.out.println (">>Error!. Name is null<<");
        }
        return name;
    }

    protected String FILE_BREAK = System.getProperty ("file.separator");
    protected boolean _usingParamsTable = false;
    protected int _columnSelected;
    protected int _rowSelected;
    protected String[] _masterItemsList = null;
    protected Map<String, ServicesObject> _mainServicesMap; //Maps the service name with the services object.
    protected Map<String, OntologyService> _ontologyRepMap; //Maps an instance of the ontology repository for each
                                                               //imported ontology.
    protected ServicesManagementTool.WsdlDataTableModel _wsdlTableModel;
    protected ServicesManagementTool _servicesTool;
    protected JTextComponent _textComp;
}