/**
 * Last Modified by: $Author: auszok $ On: $Date: 2012/03/26 20:37:21 $
 */

package kaos.xmlschema.classifier;

import kaos.core.csi.extension.KAoSExtensionComponentImpl;
import kaos.core.csi.extension.classification.ClassifierException;
import kaos.core.csi.extension.classification.InstanceClassifier;
import kaos.core.service.directory.DirectoryService;
import kaos.core.service.directory.KAoSDirectoryService;
import kaos.xmlschema.util.XMLSchemaBuilder;
import kaos.xmlschema.util.XMLSchemaBuilderImpl;
import kaos.xmlschema.util.XMLSchemaSupportedTypes;
import org.relaxng.datatype.DatatypeException;

import javax.agent.service.directory.DirectoryFailure;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

/**
 * Implementation of the XMLSchema value classifier $Revision: 1.21 $
 */
public class XMLSchemaClassifier extends KAoSExtensionComponentImpl implements InstanceClassifier
{
    private Map<String, XMLSchemaBuilder>  _cachedSchemaBuilders;
    private DirectoryService _dirProxy;
    
    private static XMLSchemaClassifier myInstance;
    
    public static XMLSchemaClassifier getInstance()
    {
        if (myInstance == null)
            myInstance = new XMLSchemaClassifier();
        
        return myInstance;
    }
    
    private XMLSchemaClassifier()
    {
        super("XMLSchemaClassifier", XMLSchemaSupportedTypes.XMLSchemaRootType);

        _cachedSchemaBuilders = new Hashtable<String, XMLSchemaBuilder>();
    }

    /**
     * Check if the instance is in the given class range
     * 
     * @param className ontology name of the class defining the range
     * @param instance the instance to be checked
     * @param instDesc not used (should be passed null)
     * @param checkedPolicyType the type of the checked policy
     * @return True if the instance is of the specified class, false if not.
     * @exception throws UnknownConceptException if it is not possible to obtain information about the schema
     */
    public boolean classify(String className, Object classDesc, String instance, Object instDesc) throws ClassifierException
    {
        if ((className == null) || (instance == null))
            throw new ClassifierException("Unable to contact directory to get XML schema for " + className + " instance " + instance);
        
        XMLSchemaBuilder xsb = null;
        
        // Get cashed schema builder for the specified name of the schema
        xsb = _cachedSchemaBuilders.get(className);
    
        // If builder is not cached
        if (xsb == null)
        {
            List<String> schema = null;
            
            // Check if the schema definition is provided
            if ((classDesc != null) && (classDesc instanceof List))
                schema = (List<String>)classDesc;
                
            // If not provided try DS
            if ((schema == null) && (_dirProxy != null))
                try
                {
                    // Makes a call to directory service to get the schema for the given data type...
                    schema = _dirProxy.getSchema(className); 
                }
                catch (DirectoryFailure e)
                {
                    throw new ClassifierException("Unable to contact directory to get XML schema for " + className);
                }
                catch (DatatypeException e)
                {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            
            // Parse the XML Schema
            try
            {
                xsb = new XMLSchemaBuilderImpl(schema);
            }
            catch (DatatypeException e)
            {
                throw new ClassifierException("Incorrect XML schema for " + className);
            }
            
            // Cache the schema
            _cachedSchemaBuilders.put(xsb.getCurrentDataTypeName(), xsb);
        }
        
        if (xsb != null)
            // Perform the classification; by asking if the instance is a valid instance of the given schema
            return xsb.isValid(instance);
        else
            return false;
    }

    /**
     * Set OntologyService.
     * 
     * @param ontInterfaces OntologyService to be used, if needed for instance classification.
     */
    public void setOntologyInterfaces(DirectoryService ontInterfaces)
    {
        if (KAoSDirectoryService._localDirectoryService != null)
            _dirProxy = KAoSDirectoryService._localDirectoryService;
        else
            _dirProxy = ontInterfaces;
    }
}